@@ -1,9 +1,31 @@
+/*
+ * ComplexSubstitutionModelParser.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beagle.evomodel.parsers;
 
-import dr.app.beagle.evomodel.substmodel.ComplexSubstitutionModel;
-import dr.app.beagle.evomodel.substmodel.FrequencyModel;
-import dr.app.beagle.evomodel.substmodel.SVSComplexSubstitutionModel;
-import dr.app.beagle.evomodel.substmodel.SubstitutionModel;
+import dr.app.beagle.evomodel.substmodel.*;
 import dr.evolution.datatype.DataType;
 import dr.inference.model.BayesianStochasticSearchVariableSelection;
 import dr.inference.model.Parameter;
@@ -26,6 +48,8 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
     public static final String INDICATOR = "rateIndicator";
     public static final String BSSVS_TOLERANCE = "bssvsTolerance";
     public static final String BSSVS_SCALAR = "bssvsScalar";
+    public static final String CHECK_CONDITIONING = "checkConditioning";
+    public static final String NORMALIZED = "normalized";
 
     public static final int maxRandomizationTries = 100;
 
@@ -34,7 +58,7 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
     }
 
     public String[] getParserNames() {
-        return new String[] {COMPLEX_SUBSTITUTION_MODEL, SVS_COMPLEX_SUBSTITUTION_MODEL};
+        return new String[]{COMPLEX_SUBSTITUTION_MODEL, SVS_COMPLEX_SUBSTITUTION_MODEL};
     }
 
 
@@ -73,8 +97,18 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
             throw new XMLParseException("Rates parameter in " + getParserName() + " element should have " + rateCount + " dimensions.");
         }
 
+        boolean checkConditioning = xo.getAttribute(CHECK_CONDITIONING, true);
+
         if (!xo.hasChildNamed(INDICATOR)) {
-            return new ComplexSubstitutionModel(COMPLEX_SUBSTITUTION_MODEL,dataType, freqModel, ratesParameter);
+            if (!checkConditioning) {
+                return new ComplexSubstitutionModel(COMPLEX_SUBSTITUTION_MODEL, dataType, freqModel, ratesParameter) {
+                    protected EigenSystem getDefaultEigenSystem(int stateCount) {
+                        return new ComplexColtEigenSystem(false, ColtEigenSystem.defaultMaxConditionNumber, ColtEigenSystem.defaultMaxIterations);
+                    }
+                };
+            } else {
+                return new ComplexSubstitutionModel(COMPLEX_SUBSTITUTION_MODEL, dataType, freqModel, ratesParameter);
+            }
         }
 
         cxo = xo.getChild(INDICATOR);
@@ -102,7 +136,16 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
             }
         }
 
-        SVSComplexSubstitutionModel model = new SVSComplexSubstitutionModel(SVS_COMPLEX_SUBSTITUTION_MODEL,dataType, freqModel, ratesParameter, indicatorParameter);
+        SVSComplexSubstitutionModel model;
+        if (!checkConditioning) {
+            model = new SVSComplexSubstitutionModel(SVS_COMPLEX_SUBSTITUTION_MODEL, dataType, freqModel, ratesParameter, indicatorParameter) {
+                protected EigenSystem getDefaultEigenSystem(int stateCount) {
+                    return new ComplexColtEigenSystem(false, ColtEigenSystem.defaultMaxConditionNumber, ColtEigenSystem.defaultMaxIterations);
+                }
+            };
+        } else {
+            model = new SVSComplexSubstitutionModel(SVS_COMPLEX_SUBSTITUTION_MODEL, dataType, freqModel, ratesParameter, indicatorParameter);
+        }
         boolean randomize = xo.getAttribute(RANDOMIZE, false);
         if (randomize) {
             // Randomization may need multiple tries
@@ -111,12 +154,16 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
 
             while (!valid && tries < maxRandomizationTries) {
                 BayesianStochasticSearchVariableSelection.Utils.randomize(indicatorParameter,
-                    dataType.getStateCount(),false);
+                        dataType.getStateCount(), false);
                 valid = !Double.isInfinite(model.getLogLikelihood());
                 tries++;
             }
             Logger.getLogger("dr.app.beagle.evomodel").info("\tRandomization attempts: " + tries);
         }
+        if (!xo.getAttribute(NORMALIZED, true)) {
+            model.setNormalization(false);
+            Logger.getLogger("dr.app.beagle.evomodel").info("\tNormalization: false");
+        }
         Logger.getLogger("dr.app.beagle.evomodel").info("\t\tPlease cite: Edwards, Suchard et al. (2011)\n");
         return model;
     }
@@ -144,19 +191,21 @@ public class ComplexSubstitutionModelParser extends AbstractXMLObjectParser {
                     new ElementRule(DataType.class),
                     true // Optional
             ),
-            AttributeRule.newBooleanRule(RANDOMIZE,true),
+            AttributeRule.newBooleanRule(RANDOMIZE, true),
             new XORRule(
-                new ElementRule(FREQUENCIES,FrequencyModel.class),
-                new ElementRule(ROOT_FREQUENCIES,FrequencyModel.class)),
+                    new ElementRule(FREQUENCIES, FrequencyModel.class),
+                    new ElementRule(ROOT_FREQUENCIES, FrequencyModel.class)),
             new ElementRule(RATES,
-                new XMLSyntaxRule[]{
-                    new ElementRule(Parameter.class, true)}
+                    new XMLSyntaxRule[]{
+                            new ElementRule(Parameter.class, true)}
             ),
             new ElementRule(INDICATOR,
                     new XMLSyntaxRule[]{
                             new ElementRule(Parameter.class)
-                    },true),
+                    }, true),
             AttributeRule.newDoubleRule(BSSVS_TOLERANCE, true),
             AttributeRule.newDoubleRule(BSSVS_SCALAR, true),
+            AttributeRule.newBooleanRule(CHECK_CONDITIONING, true),
+            AttributeRule.newBooleanRule(NORMALIZED, true),
     };
 }
