@@ -1,7 +1,7 @@
 /*
  * GammaSiteModelParser.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,15 +25,20 @@
 
 package dr.app.beagle.evomodel.parsers;
 
+import java.util.logging.Logger;
+
 import dr.app.beagle.evomodel.sitemodel.BranchSubstitutionModel;
-import dr.app.beagle.evomodel.sitemodel.EpochBranchSubstitutionModel;
 import dr.app.beagle.evomodel.sitemodel.GammaSiteRateModel;
 import dr.app.beagle.evomodel.substmodel.SubstitutionModel;
 import dr.evomodel.sitemodel.SiteModel;
 import dr.inference.model.Parameter;
-import dr.xml.*;
-
-import java.util.logging.Logger;
+import dr.xml.AbstractXMLObjectParser;
+import dr.xml.AttributeRule;
+import dr.xml.ElementRule;
+import dr.xml.XMLObject;
+import dr.xml.XMLParseException;
+import dr.xml.XMLSyntaxRule;
+import dr.xml.XORRule;
 
 /**
  * @author Andrew Rambaut
@@ -104,7 +109,7 @@ public class GammaSiteModelParser extends AbstractXMLObjectParser {
 
 //        	System.err.println("Doing the substitution model stuff");
 
-            // set this to pass it along to the TreeLikelihoodParser...
+            // set this to pass it along to the OldTreeLikelihoodParser...
             substitutionModel = (SubstitutionModel) xo.getElementFirstChild(SUBSTITUTION_MODEL);
             siteRateModel.setSubstitutionModel(substitutionModel);
 
@@ -137,7 +142,7 @@ public class GammaSiteModelParser extends AbstractXMLObjectParser {
                     }),
                     new ElementRule(BRANCH_SUBSTITUTION_MODEL, new XMLSyntaxRule[]{
                             new ElementRule(BranchSubstitutionModel.class)
-                    }), false
+                    }), true
             ),
 
             new XORRule(
