@@ -109,7 +109,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         numCodons = partition[0].getPatternWeights().length;
 
         for (int i = 0; i < 3; i++) {
-            substModelsList.add(partition[i].getSubstitutionModel());
+            substModelsList.add(partition[i].getBranchModel().getRootSubstitutionModel());
             siteRateModelsList.add(partition[i].getSiteRateModel());
             if (partition[i].getPatternWeights().length != numCodons) {
                 throw new RuntimeException("All sequence lengths must be equal in CodonPartitionedRobustCounting");
@@ -222,6 +222,10 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
             if (useUniformization && saveCompleteHistory) {
                 UniformizedSubstitutionModel usModel = (UniformizedSubstitutionModel) markovJumps;
 
+                if (completeHistoryPerNode == null) {
+                    completeHistoryPerNode = new String[tree.getNodeCount()][numCodons];
+                }
+
                 StateHistory history = usModel.getStateHistory();
 
                 // Only report syn or nonsyn changes
@@ -236,20 +240,17 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
                     int n = history.getNumberOfJumps();
                     // MAS may have broken the next line
-                    String hstring = "{" + (i + 1) + "," + history.toStringChanges(usModel.dataType) + "}";
-            if (DEBUG) {
+                    String hstring = history.toStringChanges(i + 1, usModel.dataType, false);
+                    if (DEBUG) {
                         System.err.println("site " + (i + 1) + " : "
                                 + history.getNumberOfJumps()
                                 + " : "
-                                + history.toStringChanges(usModel.dataType)
+                                + history.toStringChanges(i + 1, usModel.dataType)
                                 + " " + codonLabeling.getText());
                     }
-
-                    if (completeHistoryPerNode == null) {
-                        completeHistoryPerNode = new String[tree.getNodeCount()][numCodons];
-            }
-//                    completeHistoryPerNode[child.getNumber()][i] = usModel.getCompleteHistory(parentTime, childTime); // TODO Should reformat
                     completeHistoryPerNode[child.getNumber()][i] = hstring;
+                } else {
+                    completeHistoryPerNode[child.getNumber()][i] = null;
                 }
             }
 
@@ -291,12 +292,48 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
                     return Intent.BRANCH;
                 }
 
+                public boolean getFormatAsArray() {
+                    return true;
+                }
+
                 public String[] getTrait(Tree tree, NodeRef node) {
-                    getExpectedCountsForBranch(node); // Lazy simulation of complete histories
+                    double[] count = getExpectedCountsForBranch(node); // Lazy simulation of complete histories
                     List<String> events = new ArrayList<String>();
                     for (int i = 0; i < numCodons; i++) {
-                        if (completeHistoryPerNode[node.getNumber()][i] != null) {
-                            events.add(completeHistoryPerNode[node.getNumber()][i]);
+                        String eventString = completeHistoryPerNode[node.getNumber()][i];
+                        if (eventString != null) {
+                            if (eventString.contains("},{")) { // There are multiple events
+                                String[] elements = eventString.split("(?<=\\}),(?=\\{)");
+                                for (String e : elements) {
+                                    events.add(e);
+                                }
+                            } else {
+                                events.add(eventString);
+                            }
+                        }
+                    }
+                    if (DEBUG) {
+                        double sum = 0.0;
+                        for (double d : count) {
+                            if (d > 0.0) {
+                                sum += 1;
+                            }
+                        }
+                        System.err.println(events.size() + " " + sum);
+                        if (Math.abs(events.size() - sum) > 0.5) {
+                            System.err.println("Error");
+                            for (int i = 0; i < count.length; ++i) {
+                                if (count[i] != 0.0) {
+                                    System.err.println(i + ": " + count[i] + completeHistoryPerNode[node.getNumber()][i]);
+                                }
+                            }
+                            System.err.println("Error");
+                            int c = 0;
+                            for (String s : events) {
+                                c++;
+                                System.err.println(c + ":" + s);
+                            }
+                            System.exit(-1);
                         }
                     }
                     String[] array = new String[events.size()];
