@@ -1,13 +1,37 @@
+/*
+ * ComplexSubstitutionModel.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beagle.evomodel.substmodel;
 
 import dr.evolution.datatype.DataType;
-import dr.inference.model.Parameter;
-import dr.inference.model.Likelihood;
-import dr.inference.model.BayesianStochasticSearchVariableSelection;
-import dr.inference.model.Model;
 import dr.inference.loggers.LogColumn;
-import dr.inference.loggers.MatrixEntryColumn;
 import dr.inference.loggers.NumberColumn;
+import dr.inference.model.BayesianStochasticSearchVariableSelection;
+import dr.inference.model.Likelihood;
+import dr.inference.model.Model;
+import dr.inference.model.Parameter;
 import dr.math.matrixAlgebra.Vector;
 
 import java.util.Arrays;
@@ -19,6 +43,7 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
 
     public ComplexSubstitutionModel(String name, DataType dataType, FrequencyModel freqModel, Parameter parameter) {
         super(name, dataType, freqModel, parameter, -1);
+        probability = new double[stateCount * stateCount];
     }
 
     protected EigenSystem getDefaultEigenSystem(int stateCount) {
@@ -44,7 +69,7 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
         double[] Evec = eigen.getEigenVectors();
         double[] Eval = eigen.getEigenValues();
         double[] EvalImag = new double[stateCount];
-        System.arraycopy(Eval,stateCount,EvalImag,0,stateCount);
+        System.arraycopy(Eval, stateCount, EvalImag, 0, stateCount);
         double[] Ievc = eigen.getInverseEigenVectors();
 
         double[][] iexp = new double[stateCount][stateCount];
@@ -81,10 +106,10 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
                 double expatsinbt = expat * Math.sin(distance * b);
 
                 for (int j = 0; j < stateCount; j++) {
-                    iexp[i ][j] = expatcosbt * Ievc[i  * stateCount + j] +
-                                  expatsinbt * Ievc[i2 * stateCount + j];
+                    iexp[i][j] = expatcosbt * Ievc[i * stateCount + j] +
+                            expatsinbt * Ievc[i2 * stateCount + j];
                     iexp[i2][j] = expatcosbt * Ievc[i2 * stateCount + j] -
-                                  expatsinbt * Ievc[i  * stateCount + j];
+                            expatsinbt * Ievc[i * stateCount + j];
                 }
                 i++; // processed two conjugate rows
             }
@@ -116,13 +141,17 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
         int i, j, k = 0;
         for (i = 0; i < stateCount; i++) {
             for (j = i + 1; j < stateCount; j++) {
-                matrix[i][j] = rates[k++] * pi[j];
+                double thisRate = rates[k++];
+                if (thisRate < 0.0) thisRate = 0.0;
+                matrix[i][j] = thisRate * pi[j];
             }
         }
         // Copy lower triangle in column-order form (transposed)
         for (j = 0; j < stateCount; j++) {
             for (i = j + 1; i < stateCount; i++) {
-                matrix[i][j] = rates[k++] * pi[j];
+                double thisRate = rates[k++];
+                if (thisRate < 0.0) thisRate = 0.0;
+                matrix[i][j] = thisRate * pi[j];
             }
         }
     }
@@ -132,19 +161,28 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
     }
 
     protected double getNormalizationValue(double[][] matrix, double[] pi) {
-        if (doNormalization)
-            return super.getNormalizationValue(matrix, pi);
-        return 1.0;
+        double norm = 1.0;
+        if (doNormalization) {
+            norm = super.getNormalizationValue(matrix, pi);
+        }
+//            return super.getNormalizationValue(matrix, pi);
+//        } else {
+//            return 1.0;
+//        }
+//        System.err.println("norm = " + doNormalization + " " + norm);
+//        System.err.println(new Matrix(matrix));
+        return norm;
     }
 
     public double getLogLikelihood() {
-        if (BayesianStochasticSearchVariableSelection.Utils.connectedAndWellConditioned(probability,this))
+        if (BayesianStochasticSearchVariableSelection.Utils.connectedAndWellConditioned(probability, this))
             return 0;
         return Double.NEGATIVE_INFINITY;
     }
 
     /**
      * Needs to be evaluated before the corresponding data likelihood.
+     *
      * @return
      */
     public boolean evaluateEarly() {
@@ -163,6 +201,17 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
 
     }
 
+    public void printLastProbabilityMatrix() {
+        getLogLikelihood();
+        System.err.println((probability == null) ? "Null probability vector" : "Not null probability vector");
+        if (probability == null) {
+            boolean test = BayesianStochasticSearchVariableSelection.Utils.connectedAndWellConditioned(probability, this);
+            System.err.println("BSSVS valid = " + test);
+        }
+        System.err.println(new Vector(probability));
+    }
+
+
     @Override
     public boolean isUsed() {
         return super.isUsed() && isUsed;
@@ -195,5 +244,9 @@ public class ComplexSubstitutionModel extends GeneralSubstitutionModel implement
         }
     }
 
+    void setDoNormalization(boolean normalize) {
+        this.doNormalization = normalize;
+    }
+
     private boolean doNormalization = true;
 }
