@@ -91,7 +91,7 @@ public class UniformizedSubstitutionModel extends MarkovJumpsSubstitutionModel {
         if (model == substModel) {
             updateSubordinator = true;
         }
-        super.handleModelChangedEvent(model, object, index);       
+        super.handleModelChangedEvent(model, object, index);
     }
 
     public void setSaveCompleteHistory(boolean in) {
@@ -130,12 +130,16 @@ public class UniformizedSubstitutionModel extends MarkovJumpsSubstitutionModel {
         return getCompleteHistory(null, null);
     }
 
-    public String getCompleteHistory(Double newStartTime, Double newEndTime) {
+   public String getCompleteHistory(Double newStartTime, Double newEndTime) {
+        return getCompleteHistory(-1, newStartTime, newEndTime);
+   }
+
+    public String getCompleteHistory(int site, Double newStartTime, Double newEndTime) {
         if (newStartTime != null && newEndTime != null) {
             // Rescale time of events
             completeHistory.rescaleTimesOfEvents(newStartTime, newEndTime);
         }
-        return completeHistory.toStringChanges(dataType); //, 0.0);
+        return completeHistory.toStringChanges(site, dataType); //, 0.0);
     }
 
     public int getNumberOfJumpsInCompleteHistory() {
@@ -144,7 +148,7 @@ public class UniformizedSubstitutionModel extends MarkovJumpsSubstitutionModel {
 
     public double computeCondStatMarkovJumps(int startingState,
                                              int endingState,
-                                             double time,                                             
+                                             double time,
                                              double transitionProbability) {
 
         if (updateSubordinator) {
@@ -153,16 +157,16 @@ public class UniformizedSubstitutionModel extends MarkovJumpsSubstitutionModel {
 
         double total = 0;
         for (int i = 0; i < numSimulants; i++) {
-            StateHistory history;
+            StateHistory history = null;
             try {
                 history = UniformizedStateHistory.simulateConditionalOnEndingState(
-                    0.0,
-                    startingState,
-                    time,
-                    endingState,
-                    transitionProbability,
-                    stateCount,
-                    subordinator
+                        0.0,
+                        startingState,
+                        time,
+                        endingState,
+                        transitionProbability,
+                        stateCount,
+                        subordinator
                 );
             } catch (SubordinatedProcess.Exception e) {
 
@@ -192,7 +196,7 @@ public class UniformizedSubstitutionModel extends MarkovJumpsSubstitutionModel {
                     if (history.getEndingState() == endingState) {
                         success = true;
                     }
-                    
+
                     attempts++;
                 }
             }
