@@ -1,8 +1,32 @@
+/*
+ * AncestralStateBeagleTreeLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beagle.evomodel.treelikelihood;
 
-import dr.app.beagle.evomodel.sitemodel.BranchSubstitutionModel;
+import dr.app.beagle.evomodel.branchmodel.BranchModel;
 import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
-import dr.app.beagle.evomodel.substmodel.SubstitutionModel;
 import dr.evolution.alignment.PatternList;
 import dr.evolution.datatype.Codons;
 import dr.evolution.datatype.DataType;
@@ -26,7 +50,7 @@ import java.util.Set;
  * @author Andrew Rambaut
  */
 
-public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood implements TreeTraitProvider {
+public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood implements TreeTraitProvider, AncestralStateTraitProvider {
 
 //    public AncestralStateBeagleTreeLikelihood(PatternList patternList, TreeModel treeModel,
 //                                              BranchSubstitutionModel branchSubstitutionModel, SiteRateModel siteRateModel,
@@ -40,7 +64,8 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
 //    }
 
     public AncestralStateBeagleTreeLikelihood(PatternList patternList, TreeModel treeModel,
-                                              BranchSubstitutionModel branchSubstitutionModel, SiteRateModel siteRateModel,
+                                              BranchModel branchModel,
+                                              SiteRateModel siteRateModel,
                                               BranchRateModel branchRateModel,
                                               TipStatesModel tipStatesModel,
                                               boolean useAmbiguities,
@@ -48,12 +73,12 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
                                               Map<Set<String>, Parameter> partialsRestrictions,
                                               final DataType dataType,
                                               final String tag,
-                                              SubstitutionModel substModel,
+//                                              SubstitutionModel substModel,
                                               boolean useMAP,
                                               boolean returnML) {
 
-        super(patternList, treeModel, branchSubstitutionModel, siteRateModel, branchRateModel, tipStatesModel, useAmbiguities, scalingScheme,
-              partialsRestrictions);
+        super(patternList, treeModel, branchModel, siteRateModel, branchRateModel, tipStatesModel, useAmbiguities, scalingScheme,
+                partialsRestrictions);
 
         this.dataType = dataType;
 //        this.tag = tag;
@@ -72,11 +97,9 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
             // Find the id of tip i in the patternList
             String id = treeModel.getTaxonId(i);
             int index = patternList.getTaxonIndex(id);
-            tipStates[i] = getStates(patternList,index);
+            tipStates[i] = getStates(patternList, index);
         }
 
-        substitutionModel = substModel;
-
         reconstructedStates = new int[treeModel.getNodeCount()][patternCount];
         storedReconstructedStates = new int[treeModel.getNodeCount()][patternCount];
 
@@ -97,20 +120,16 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
             }
 
             public int[] getTrait(Tree tree, NodeRef node) {
-                return getStatesForNode(tree,node);
+                return getStatesForNode(tree, node);
             }
 
             public String getTraitString(Tree tree, NodeRef node) {
-                return formattedState(getStatesForNode(tree,node), dataType);
+                return formattedState(getStatesForNode(tree, node), dataType);
             }
         });
 
     }
 
-    public SubstitutionModel getSubstitutionModel() {
-        return substitutionModel;
-    }
-
     private int[] getStates(PatternList patternList,
                             int sequenceIndex) {
 
@@ -121,6 +140,10 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
         return states;
     }
 
+    public BranchModel getBranchModel() {
+        return branchModel;
+    }
+
     protected Helper treeTraits = new Helper();
 
     public TreeTrait[] getTreeTraits() {
@@ -137,9 +160,9 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
     }
 
     public int[] getStatesForNode(Tree tree, NodeRef node) {
-         if (tree != treeModel) {
-             throw new RuntimeException("Can only reconstruct states on treeModel given to constructor");
-         }
+        if (tree != treeModel) {
+            throw new RuntimeException("Can only reconstruct states on treeModel given to constructor");
+        }
 
         if (!likelihoodKnown) {
             calculateLogLikelihood();
@@ -153,7 +176,7 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
     }
 
     @Override
-     protected int getScaleBufferCount() {
+    protected int getScaleBufferCount() {
         return internalNodeCount + 2;
     }
 
@@ -173,17 +196,17 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
         }
     }
 
-     public void makeDirty() {
-         super.makeDirty();
-         areStatesRedrawn = false;
-     }
+    public void makeDirty() {
+        super.makeDirty();
+        areStatesRedrawn = false;
+    }
 
-     public void redrawAncestralStates() {
-         // Sample states
-         jointLogLikelihood = 0;
-         traverseSample(treeModel, treeModel.getRoot(), null, null);
-         areStatesRedrawn = true;
-     }
+    public void redrawAncestralStates() {
+        // Sample states
+        jointLogLikelihood = 0;
+        traverseSample(treeModel, treeModel.getRoot(), null, null);
+        areStatesRedrawn = true;
+    }
 
 //    protected double calculateLogLikelihood() {
 //        areStatesRedrawn = false;
@@ -233,8 +256,8 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
         return sb.toString();
     }
 
-    protected void getMatrix(int matrixNum, double[] probabilities) {
-        beagle.getTransitionMatrix(matrixBufferHelper.getOffsetIndex(matrixNum),probabilities);
+    protected void getMatrix(int branchIndex, double[] probabilities) {
+        beagle.getTransitionMatrix(substitutionModelDelegate.getMatrixIndex(branchIndex), probabilities);
         // NB: It may be faster to compute matrices in BEAST via substitutionModel
     }
 
@@ -244,7 +267,7 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
         makeDirty();
     }
 
-    public void getStates(int tipNum, int[] states)  {
+    public void getStates(int tipNum, int[] states) {
         // Saved locally to reduce BEAGLE library access
         System.arraycopy(tipStates[tipNum], 0, states, 0, states.length);
     }
@@ -324,8 +347,8 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
 
             if (parent == null) {
 
-                 // This is the root node
-                getPartials(nodeNum,partials);
+                // This is the root node
+                getPartials(nodeNum, partials);
 
                 boolean sampleCategory = categoryCount > 1;
                 double[] posteriorWeightedCategory = null;
@@ -356,7 +379,7 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
                     int partialsIndex = (rateCategory == null ? 0 : rateCategory[j]) * stateCount * patternCount;
                     System.arraycopy(partials, partialsIndex + j * stateCount, conditionalProbabilities, 0, stateCount);
 
-                    double[] frequencies = branchSubstitutionModel.getStateFrequencies(0); // TODO May have more than one set of frequencies
+                    double[] frequencies = substitutionModelDelegate.getRootStateFrequencies(); // TODO May have more than one set of frequencies
                     for (int i = 0; i < stateCount; i++) {
                         conditionalProbabilities[i] *= frequencies[i];
                     }
@@ -385,13 +408,13 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
 
                 // This is an internal node, but not the root
                 double[] partialLikelihood = new double[stateCount * patternCount * categoryCount];
-                getPartials(nodeNum,partialLikelihood);
-                
+                getPartials(nodeNum, partialLikelihood);
+
                 // Sibon says that this actually works now
 //                if (categoryCount > 1)
 //                    throw new RuntimeException("Reconstruction not implemented for multiple categories yet.");
 
-                getMatrix(nodeNum,probabilities);
+                getMatrix(nodeNum, probabilities);
 
                 for (int j = 0; j < patternCount; j++) {
 
@@ -442,10 +465,10 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
                     int category = rateCategory == null ? 0 : rateCategory[j];
                     int matrixIndex = category * stateCount * stateCount;
 
-                    getMatrix(nodeNum,probabilities);
+                    getMatrix(nodeNum, probabilities);
                     System.arraycopy(probabilities, parentIndex + matrixIndex, conditionalProbabilities, 0, stateCount);
 
-                    if (!dataType.isUnknownState(thisState)) { // Not completely unknown
+                    if (useAmbiguities && !dataType.isUnknownState(thisState)) { // Not completely unknown
                         boolean[] stateSet = dataType.getStateSet(thisState);
 
                         for (int k = 0; k < stateCount; k++) {
@@ -481,7 +504,7 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
     private int[][] reconstructedStates;
     private int[][] storedReconstructedStates;
 
-//    private final String tag;
+    //    private final String tag;
     protected boolean areStatesRedrawn = false;
     protected boolean storedAreStatesRedrawn = false;
 
@@ -493,8 +516,6 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
 
     private int[][] tipStates;
 
-    protected SubstitutionModel substitutionModel;
-
     private double[] probabilities;
     private double[] partials;
 
@@ -502,4 +523,5 @@ public class AncestralStateBeagleTreeLikelihood extends BeagleTreeLikelihood imp
 //    private double[] rootPartials;
 //    private int[][] cumulativeScaleBuffers;
 //    private int scaleBufferIndex;
+
 }
