@@ -1,6 +1,31 @@
+/*
+ * MarkovJumpsBeagleTreeLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beagle.evomodel.treelikelihood;
 
-import dr.app.beagle.evomodel.sitemodel.BranchSubstitutionModel;
+import dr.app.beagle.evomodel.branchmodel.BranchModel;
 import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
 import dr.app.beagle.evomodel.substmodel.MarkovJumpsSubstitutionModel;
 import dr.app.beagle.evomodel.substmodel.SubstitutionModel;
@@ -33,25 +58,25 @@ import java.util.*;
  *         Journal of Mathematical Biology, 56, 391-412.
  */
 public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLikelihood
-        implements MarkovJumpsRegisterAcceptor {
+        implements MarkovJumpsRegisterAcceptor, MarkovJumpsTraitProvider {
 
     public MarkovJumpsBeagleTreeLikelihood(PatternList patternList, TreeModel treeModel,
-                                           BranchSubstitutionModel branchSubstitutionModel, SiteRateModel siteRateModel,
+                                           BranchModel branchModel,
+                                           SiteRateModel siteRateModel,
                                            BranchRateModel branchRateModel,
                                            TipStatesModel tipStatesModel,
                                            boolean useAmbiguities,
                                            PartialsRescalingScheme scalingScheme,
                                            Map<Set<String>, Parameter> partialsRestrictions,
                                            DataType dataType, String stateTag,
-                                           SubstitutionModel substModel,
                                            boolean useMAP,
                                            boolean returnMarginalLikelihood,
                                            boolean useUniformization,
                                            boolean reportUnconditionedColumns,
                                            int nSimulants) {
 
-        super(patternList, treeModel, branchSubstitutionModel, siteRateModel, branchRateModel, tipStatesModel, useAmbiguities,
-                scalingScheme, partialsRestrictions, dataType, stateTag, substModel, useMAP, returnMarginalLikelihood);
+        super(patternList, treeModel, branchModel, siteRateModel, branchRateModel, tipStatesModel, useAmbiguities,
+                scalingScheme, partialsRestrictions, dataType, stateTag, useMAP, returnMarginalLikelihood);
 
         this.useUniformization = useUniformization;
         this.reportUnconditionedColumns = reportUnconditionedColumns;
@@ -82,11 +107,11 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
         addVariable(addRegisterParameter);
         final String tag = addRegisterParameter.getId();
 
-        for (int i = 0; i < branchSubstitutionModel.getEigenCount(); ++i) {
+        for (int i = 0; i < substitutionModelDelegate.getSubstitutionModelCount(); ++i) {
 
             registerParameter.add(addRegisterParameter);
             MarkovJumpsSubstitutionModel mjModel;
-            SubstitutionModel substitutionModel = branchSubstitutionModel.getSubstitutionModel(i, 0);
+            SubstitutionModel substitutionModel = substitutionModelDelegate.getSubstitutionModel(i);
 
             if (useUniformization) {
                 mjModel = new UniformizedSubstitutionModel(substitutionModel, type, nSimulants);
@@ -103,14 +128,14 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
 
             String traitName;
 
-            if (branchSubstitutionModel.getEigenCount() == 1) {
+            if (substitutionModelDelegate.getSubstitutionModelCount() == 1) {
                 traitName = tag;
             } else {
                 traitName = tag + i;
             }
 
             jumpTag.add(traitName);
-            
+
             expectedJumps.add(new double[treeModel.getNodeCount()][patternCount]);
 //        storedExpectedJumps.add(new double[treeModel.getNodeCount()][patternCount]);
 
@@ -172,6 +197,49 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
                 historyRegisterNumber = numRegisters; // Record the complete history for this register
                 ((UniformizedSubstitutionModel) mjModel).setSaveCompleteHistory(true);
 
+                if (useCompactHistory && logHistory) {
+
+                    treeTraits.addTrait(ALL_HISTORY, new TreeTrait.SA() {
+                        public String getTraitName() {
+                            return ALL_HISTORY;
+                        }
+
+                        public Intent getIntent() {
+                            return Intent.BRANCH;
+                        }
+
+                        public boolean getFormatAsArray() {
+                            return true;
+                        }
+
+                        public String[] getTrait(Tree tree, NodeRef node) {
+
+                            List<String> events = new ArrayList<String>();
+                            for (int i = 0; i < patternCount; i++) {
+                                String eventString = getHistoryForNode(tree, node, i);
+                                if (eventString != null && eventString.compareTo("{}") != 0) {
+                                    eventString = eventString.substring(1, eventString.length() - 1);
+                                    if (eventString.contains("},{")) { // There are multiple events
+                                        String[] elements = eventString.split("(?<=\\}),(?=\\{)");
+                                        for (String e : elements) {
+                                            events.add(e);
+                                        }
+                                    } else {
+                                        events.add(eventString);
+                                    }
+                                }
+                            }
+                            String[] array = new String[events.size()];
+                            events.toArray(array);
+                            return array;
+                        }
+
+                        public boolean getLoggable() {
+                            return true;
+                        }
+                    });
+                }
+
                 for (int site = 0; site < patternCount; ++site) {
 
                     final String anonName = (patternCount == 1) ? HISTORY : HISTORY + "_" + (site + 1);
@@ -188,11 +256,12 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
                         }
 
                         public String getTrait(Tree tree, NodeRef node) {
-                            return getHistoryForNode(tree, node, anonSite);
+                            String history = getHistoryForNode(tree, node, anonSite);
+                            return (history.compareTo("{}") != 0) ? history : null; // Return null if empty
                         }
 
                         public boolean getLoggable() {
-                            return logHistory;
+                            return logHistory && !useCompactHistory;
                         }
                     });
                 }
@@ -207,6 +276,10 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
         logHistory = in;
     }
 
+    public void setUseCompactHistory(boolean in) {
+        useCompactHistory = in;
+    }
+
 //    public double[] getRewardsForNodeAndPattern(Tree tree, NodeRef node, int pattern) {
 //        double[] rtn = new double[numRegisters];
 //        for (int r = 0; r < numRegisters; r++) {
@@ -238,7 +311,7 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
             calculateLogLikelihood();
             likelihoodKnown = true;
         }
-        
+
         if (!areStatesRedrawn) {
             redrawAncestralStates();
         }
@@ -304,13 +377,17 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
             MarkovJumpsSubstitutionModel thisMarkovJumps = markovjumps.get(r);
 
             final int modelNumberFromrRegistry = branchModelNumber.get(r);
-            int dummy = 0;
-            final int modelNumberFromTree = branchSubstitutionModel.getBranchIndex(tree, childNode, dummy);
+//            int dummy = 0;
+//            final int modelNumberFromTree = branchSubstitutionModel.getBranchIndex(tree, childNode, dummy);
+            // @todo AR - not sure about this - if this is an epoch this is just going to get the most
+            // @todo tipward model for the branch. I think this was what was happening before (in comment,
+            // @todo above).
+            BranchModel.Mapping mapping = branchModel.getBranchModelMapping(childNode);
 
-            if (modelNumberFromrRegistry == modelNumberFromTree) {
+            if (modelNumberFromrRegistry == mapping.getOrder()[0]) {
                 if (useUniformization) {
                     computeSampledMarkovJumpsForBranch(((UniformizedSubstitutionModel) thisMarkovJumps), substTime,
-                            branchRate, childNum, parentStates, childStates, parentTime, childTime,probabilities, scaleByTime[r],
+                            branchRate, childNum, parentStates, childStates, parentTime, childTime, probabilities, scaleByTime[r],
                             expectedJumps.get(r), rateCategory, r == historyRegisterNumber);
                 } else {
                     computeIntegratedMarkovJumpsForBranch(thisMarkovJumps, substTime, branchRate, childNum, parentStates,
@@ -355,7 +432,8 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
             }
             thisExpectedJumps[childNum][j] = value;
             if (saveHistory) {
-                histories[childNum][j] = thisMarkovJumps.getCompleteHistory(parentTime, childTime);
+                int site = (useCompactHistory) ? j + 1 : -1;
+                histories[childNum][j] = thisMarkovJumps.getCompleteHistory(site, parentTime, childTime);
             }
         }
     }
@@ -536,6 +614,7 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
         }
     }
 
+    public static final String ALL_HISTORY = "history_all";
     public static final String HISTORY = "history";
     public static final String TOTAL_COUNTS = "allTransitions";
 
@@ -544,8 +623,9 @@ public class MarkovJumpsBeagleTreeLikelihood extends AncestralStateBeagleTreeLik
     private List<Parameter> registerParameter;
     private List<String> jumpTag;
     private List<double[][]> expectedJumps;
-//    private List<double[][]> storedExpectedJumps;
+    //    private List<double[][]> storedExpectedJumps;
     private boolean logHistory = false;
+    private boolean useCompactHistory = false;
     private String[][] histories = null;
     private boolean[] scaleByTime;
     private double[] tmpProbabilities;
