@@ -44,6 +44,10 @@ import java.util.Iterator;
 import java.util.Locale;
 import java.util.logging.*;
 
+/**
+ * MCMCMC functionality now rolled into the BEAST Main class.
+ * @deprecated
+ */
 public class BeastMC3 {
 
     public final static int HOT_CHAIN_COUNT = 2;
@@ -82,9 +86,7 @@ public class BeastMC3 {
 
         int chainCount = chainTemperatures.length;
         MCMC[] chains = new MCMC[chainCount];
-        MCMCMCOptions options = new MCMCMCOptions();
-        options.setChainTemperatures(chainTemperatures);
-        options.setSwapChainsEvery(swapChainsEvery);
+        MCMCMCOptions options = new MCMCMCOptions(chainTemperatures, swapChainsEvery);
 
         // Add a handler to handle warnings and errors. This is a ConsoleHandler
         // so the messages will go to StdOut..
