@@ -38,19 +38,21 @@ import dr.util.Version;
  *
  * @author Alexei Drummond
  * @author Andrew Rambaut
+ *
+ * $Id: BeastVersion.java 5972 2013-10-16 12:30:40Z rambaut $
  */
 public class BeastVersion implements Version {
 
     /**
      * Version string: assumed to be in format x.x.x
      */
-    private static final String VERSION = "1.7.5";
+    private static final String VERSION = "1.8.0";
 
     private static final String DATE_STRING = "2002-2013";
 
     private static final boolean IS_PRERELEASE = false;
 
-    private static final String REVISION = "$Rev: 5415 $";
+    private static final String REVISION = "$Rev: 5972 $";
 
     public String getVersion() {
         return VERSION;
@@ -89,9 +91,9 @@ public class BeastVersion implements Version {
                 "\thttp://code.google.com/p/beast-mcmc",
                 "",
                 "BEAST developers:",
-                "\tAlex Alekseyenko, Guy Baele, Trevor Bedford, Filip Bielejec, Erik Bloomquist, ",
-                "\tJoseph Heled, Sebastian Hoehna, Denise Kuehnert, Philippe Lemey, Wai Lok Sibon Li, ",
-                "\tGerton Lunter, Sidney Markowitz, Vladimir Minin, Michael Defoin Platel, ",
+                "\tAlex Alekseyenko, Guy Baele, Trevor Bedford, Filip Bielejec, Erik Bloomquist, Matthew Hall,",
+                "\tJoseph Heled, Sebastian Hoehna, Denise Kuehnert, Philippe Lemey, Wai Lok Sibon Li,",
+                "\tGerton Lunter, Sidney Markowitz, Vladimir Minin, Michael Defoin Platel,",
                 "\tOliver Pybus, Chieh-Hsi Wu, Walter Xie",
                 "",
                 "Thanks to:",
@@ -112,7 +114,7 @@ public class BeastVersion implements Version {
                         "<p>Source code distributed under the GNU LGPL:<br>" +
                         "<a href=\"http://code.google.com/p/beast-mcmc\">http://code.google.com/p/beast-mcmc</a></p>" +
                         "<p>BEAST developers:<br>" +
-                        "Alex Alekseyenko, Guy Baele, Trevor Bedford, Filip Bielejec, Erik Bloomquist, <br>"+
+                        "Alex Alekseyenko, Guy Baele, Trevor Bedford, Filip Bielejec, Erik Bloomquist, Matthew Hall,<br>"+
                         "Joseph Heled, Sebastian Hoehna, Denise Kuehnert, Philippe Lemey, Wai Lok Sibon Li,<br>"+
                         "Gerton Lunter, Sidney Markowitz, Vladimir Minin, Michael Defoin Platel,<br>"+
                         "Oliver Pybus, Chieh-Hsi Wu, Walter Xie</p>" +
