@@ -76,7 +76,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
     }// END: usesInsertionPoint
 
-    protected void generate(InsertionPoint point, Object item, XMLWriter writer) {
+    protected void generate(final InsertionPoint point, final Object item, final String prefix, final XMLWriter writer) {
 
         AncestralStatesComponentOptions component = (AncestralStatesComponentOptions) options
                 .getComponentOptions(AncestralStatesComponentOptions.class);
@@ -86,7 +86,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
                 writeCodonPartitionedRobustCounting(writer, component);
                 break;
             case IN_TREE_LIKELIHOOD:
-                writeCountingParameter(writer, (AbstractPartitionData)item);
+                writeCountingParameter(writer, (AbstractPartitionData)item, prefix);
                 break;
             case IN_FILE_LOG_PARAMETERS:
                 writeLogs(writer, component);
@@ -108,7 +108,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
     }// END: generate
 
-    private void writeCountingParameter(XMLWriter writer, AbstractPartitionData partition) {
+    private void writeCountingParameter(XMLWriter writer, AbstractPartitionData partition, String prefix) {
         AncestralStatesComponentOptions component = (AncestralStatesComponentOptions) options
                 .getComponentOptions(AncestralStatesComponentOptions.class);
         if (!component.isCountingStates(partition)) {
@@ -121,7 +121,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
         DataType dataType = partition.getDataType();
         int stateCount = dataType.getStateCount();
 
-        if (dataType == GeneralDataType.INSTANCE) {
+        if (dataType.getType() == DataType.GENERAL) {
             PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
             stateCount = substModel.getDiscreteStateSet().size();
         }
@@ -137,7 +137,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
         }
         writer.writeTag("parameter",
                 new Attribute[] {
-                        new Attribute.Default<String>("id", partition.getPrefix() + "count"),  // TODO Pass codon partition number, so can construct unique name
+                        new Attribute.Default<String>("id", prefix + "count"),
                         new Attribute.Default<String>("value", matrix.toString()) },
                 true);
 
@@ -274,7 +274,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
             }
 
             if (component.isCountingStates(partition)) {
-                if (partition.getDataType() == ContinuousDataType.INSTANCE)  {
+                if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
                     throw new RuntimeException("Can't do counting on Continuous data partition");
                 }
 
@@ -299,7 +299,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
     private void writeTrait(XMLWriter writer, AbstractPartitionData partition, String prefix, String tag, String name) {
         String traitName = prefix + tag;
 
-        if (partition.getDataType() == ContinuousDataType.INSTANCE)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
             traitName = partition.getName();
         }
 
@@ -309,7 +309,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
                         new Attribute.Default<String>("tag", name)
                 }
         );
-        if (partition.getDataType() == ContinuousDataType.INSTANCE)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
             writer.writeIDref("multivariateTraitLikelihood", prefix + "traitLikelihood");
         } else {
             writer.writeIDref("ancestralTreeLikelihood", prefix + "treeLikelihood");
@@ -362,7 +362,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
     private void writeAncestralTrait(XMLWriter writer, AbstractPartitionData partition, String mrcaId, String prefix, String nameString) {
         String traitName = prefix + AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG;
-        if (partition.getDataType() == ContinuousDataType.INSTANCE)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
             traitName = partition.getName();
         }
 
@@ -374,7 +374,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
         );
         writer.writeIDref("treeModel", partition.getPartitionTreeModel().getPrefix() + "treeModel");
 
-        if (partition.getDataType() == ContinuousDataType.INSTANCE)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
             writer.writeIDref("multivariateTraitLikelihood", prefix + "traitLikelihood");
         } else {
             writer.writeIDref("ancestralTreeLikelihood", prefix + "treeLikelihood");
