@@ -2,6 +2,7 @@ package dr.app.beauti.components.continuous;
 
 import dr.app.beauti.generator.BaseComponentGenerator;
 import dr.app.beauti.options.*;
+import dr.app.beauti.types.OperatorType;
 import dr.app.beauti.util.XMLWriter;
 import dr.evolution.datatype.ContinuousDataType;
 import dr.evolution.util.Taxon;
@@ -44,7 +45,7 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
 
     }// END: usesInsertionPoint
 
-    protected void generate(InsertionPoint point, Object item, XMLWriter writer) {
+    protected void generate(final InsertionPoint point, final Object item, final String prefix, final XMLWriter writer) {
 
         ContinuousComponentOptions component = (ContinuousComponentOptions) options
                 .getComponentOptions(ContinuousComponentOptions.class);
@@ -61,7 +62,8 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
                 writeMultivariateTreeLikelihoods(writer, component);
                 break;
             case IN_OPERATORS:
-                writeRRWOperators(writer, component);
+                // the RRW operators are added to the operator list
+//                writeRRWOperators(writer, component);
                 writePrecisionGibbsOperators(writer, component);
                 break;
             case IN_MCMC_PRIOR:
@@ -71,7 +73,7 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
                 writeMultivariateTreeLikelihoodIdRefs(writer, component);
                 break;
             case IN_FILE_LOG_PARAMETERS:
-                writePrecisionMatrixIdRefs(writer, component);
+                writeParameterIdRefs(writer, component);
                 break;
             case IN_FILE_LOG_LIKELIHOODS:
                 writeMultivariateTreeLikelihoodIdRefs(writer, component);
@@ -93,30 +95,25 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
 
     private void writeTaxonTraits(Taxon taxon, XMLWriter writer) {
         for (AbstractPartitionData partition : options.getDataPartitions(ContinuousDataType.INSTANCE)) {
-            // If an attribute already exists with this name then we don't need to write it again
-            // (it is likely to be one of the original loaded traits).
-            if (!taxon.containsAttribute(partition.getName())) {
+            writer.writeOpenTag(AttributeParser.ATTRIBUTE, new Attribute[]{
+                    new Attribute.Default<String>(Attribute.NAME, partition.getName())});
 
-                writer.writeOpenTag(AttributeParser.ATTRIBUTE, new Attribute[]{
-                        new Attribute.Default<String>(Attribute.NAME, partition.getName())});
-
-                StringBuilder sb = new StringBuilder();
-                boolean first = true;
-                for (TraitData trait : partition.getTraits()) {
-                    if (!first) {
-                        sb.append(" ");
-                    }
-
-                    if (taxon.containsAttribute(trait.getName())) {
-                        sb.append(taxon.getAttribute(trait.getName()).toString());
-                    } else {
-                        sb.append("?");
-                    }
-                    first = false;
+            StringBuilder sb = new StringBuilder();
+            boolean first = true;
+            for (TraitData trait : partition.getTraits()) {
+                if (!first) {
+                    sb.append(" ");
                 }
-                writer.writeText(sb.toString());
-                writer.writeCloseTag(AttributeParser.ATTRIBUTE);
+
+                if (taxon.containsAttribute(trait.getName())) {
+                    sb.append(taxon.getAttribute(trait.getName()).toString());
+                } else {
+                    sb.append("?");
+                }
+                first = false;
             }
+            writer.writeText(sb.toString());
+            writer.writeCloseTag(AttributeParser.ATTRIBUTE);
         }
     }
 
@@ -241,13 +238,13 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
 
             writeMultivariateTreeLikelihood(writer, partitionData, diffusionModelId, treeModelId);
 
-            if (partitionData.getTraits().size() == 2) {
-                // if we are analysing bivariate traits we can add these special statistics...
-                String precisionMatrixId = model.getName() + ".precision";
-                write2DStatistics(writer, partitionData, precisionMatrixId, treeModelId);
+            String precisionMatrixId = model.getName() + ".precision";
+
+            writeDiffusionStatistics(writer, partitionData, treeModelId, precisionMatrixId,
+                    partitionData.getName() + ".traitLikelihood");
         }
     }
-    }
+
 
 
     private void writeRelaxedBranchRateModel(XMLWriter writer,
@@ -265,39 +262,59 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         writer.writeIDref("treeModel", treeModelId);
 
         writer.writeOpenTag("distribution");
-        writer.writeOpenTag("onePGammaDistributionModel");
-        // don't think this needs an id
-//                new Attribute[] {
-//                        new Attribute.Default<String>("id", prefix + "gamma"),
-//                });
 
-        writer.writeOpenTag("shape");
-        switch (partitionData.getPartitionSubstitutionModel().getContinuousSubstModelType()) {
+        if (partitionData.getPartitionSubstitutionModel().getContinuousSubstModelType() == ContinuousSubstModelType.LOGNORMAL_RRW) {
+            writer.writeOpenTag("logNormalDistributionModel",
+                    new Attribute[]{ new Attribute.Default<String>("meanInRealSpace", "true") });
 
-            case HOMOGENOUS:
-                throw new IllegalArgumentException("Shouldn't be here");
-            case CAUCHY_RRW:
-                writer.writeComment("half DF (i.e., df = 1)");
-                writer.writeTag("parameter",
-                        new Attribute[]{
-                                // don't think this needs an id
+            writer.writeOpenTag("mean");
+            writer.writeTag("parameter",
+                    new Attribute[]{
+                            new Attribute.Default<String>("value", "1.0")
+                    }, true);
+
+            writer.writeCloseTag("mean");
+
+            writer.writeOpenTag("stdev");
+            writer.writeTag("parameter",
+                    new Attribute[]{
+                            new Attribute.Default<String>("id", prefix + ContinuousComponentOptions.STDEV),
+                            new Attribute.Default<String>("value", "1.0"),
+                            new Attribute.Default<String>("lower", "0.0")
+                    }, true);
+
+            writer.writeCloseTag("stdev");
+            writer.writeCloseTag("logNormalDistributionModel");
+        } else {
+            writer.writeOpenTag("onePGammaDistributionModel");
+            writer.writeOpenTag("shape");
+            switch (partitionData.getPartitionSubstitutionModel().getContinuousSubstModelType()) {
+                case CAUCHY_RRW:
+                    writer.writeComment("half DF (i.e., df = 1)");
+                    writer.writeTag("parameter",
+                            new Attribute[]{
+                                    // don't think this needs an id
 //                        new Attribute.Default<String>("id", "halfDF"),
-                                new Attribute.Default<String>("value", "0.5")
-                        }, true);
-                break;
-            case GAMMA_RRW:
-                writer.writeComment("half DF");
-                writer.writeTag("parameter",
-                        new Attribute[]{
-                                new Attribute.Default<String>("id", prefix + "halfDF"),
-                                new Attribute.Default<String>("value", "0.5")
-                        }, true);
-                break;
-            default:
-                throw new IllegalArgumentException("Unknown continuous substitution type");
+                                    new Attribute.Default<String>("value", "0.5")
+                            }, true);
+                    break;
+                case GAMMA_RRW:
+                    writer.writeComment("half DF");
+                    writer.writeTag("parameter",
+                            new Attribute[]{
+                                    new Attribute.Default<String>("id", prefix + ContinuousComponentOptions.HALF_DF),
+                                    new Attribute.Default<String>("value", "0.5")
+                            }, true);
+                    break;
+                case LOGNORMAL_RRW:
+                case HOMOGENOUS:
+                    throw new IllegalArgumentException("Shouldn't be here");
+                default:
+                    throw new IllegalArgumentException("Unknown continuous substitution type");
+            }
+            writer.writeCloseTag("shape");
+            writer.writeCloseTag("onePGammaDistributionModel");
         }
-        writer.writeCloseTag("shape");
-        writer.writeCloseTag("onePGammaDistributionModel");
         writer.writeCloseTag("distribution");
 
 
@@ -325,7 +342,24 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
                 });
 
         writer.writeIDref("multivariateDiffusionModel", diffusionModelId);
-        writer.writeIDref("treeModel", treeModelId);
+
+
+        ContinuousComponentOptions component = (ContinuousComponentOptions) options
+                .getComponentOptions(ContinuousComponentOptions.class);
+
+        if (component.useLambda(partitionData.getPartitionSubstitutionModel())) {
+            writer.writeOpenTag("transformedTreeModel");
+            writer.writeIDref("treeModel", treeModelId);
+            writer.writeTag("parameter", new Attribute[] {
+                    new Attribute.Default<String>("id", partitionData.getName() + "." + ContinuousComponentOptions.LAMBDA),
+                    new Attribute.Default<String>("value", "0.5"),
+                    new Attribute.Default<String>("lower", "0.0"),
+                    new Attribute.Default<String>("upper", "1.0")
+            }, true);
+            writer.writeCloseTag("transformedTreeModel");
+        } else {
+            writer.writeIDref("treeModel", treeModelId);
+        }
 
         writer.writeOpenTag("traitParameter");
         writer.writeTag("parameter", new Attribute.Default<String>("id", "leaf." + partitionData.getName()), true);
@@ -358,18 +392,22 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         writer.writeCloseTag("multivariateTraitLikelihood");
     }
 
-    private void write2DStatistics(XMLWriter writer, AbstractPartitionData partitionData, String precisionMatrixId, String treeModelId) {
+    private void writeDiffusionStatistics(XMLWriter writer, AbstractPartitionData partitionData,
+                                          String treeModelId, String precisionMatrixId, String traitLikelihoodId) {
         String prefix = partitionData.getName() + ".";
 
-        writer.writeOpenTag("correlation",
-                new Attribute[] {
-                        new Attribute.Default<String>("id", prefix + "correlation"),
-                        new Attribute.Default<Integer>("dimension1", 1),
-                        new Attribute.Default<Integer>("dimension2", 2)
-                });
-        writer.writeIDref("matrixParameter", precisionMatrixId);
-        writer.writeCloseTag("correlation");
+        if (partitionData.getTraits().size() == 2) {
+            writer.writeOpenTag("correlation",
+                    new Attribute[] {
+                            new Attribute.Default<String>("id", prefix + "correlation"),
+                            new Attribute.Default<Integer>("dimension1", 1),
+                            new Attribute.Default<Integer>("dimension2", 2)
+                    });
+            writer.writeIDref("matrixParameter", precisionMatrixId);
+            writer.writeCloseTag("correlation");
+        }
 
+        /*
         writer.writeOpenTag("treeLengthStatistic",
                 new Attribute[] {
                         new Attribute.Default<String>("id", prefix + "treeLength")
@@ -405,94 +443,25 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         writer.writeIDref("parameter", prefix + "precision.col2");
         writer.writeCloseTag("subStatistic");
         writer.writeCloseTag("productStatistic");
+        */
 
-    }
+        writer.writeOpenTag("matrixInverse",
+                new Attribute[] {
+                        new Attribute.Default<String>("id", prefix + "varCovar")
+                });
+        writer.writeIDref("matrixParameter", precisionMatrixId);
+        writer.writeCloseTag("matrixInverse");
 
-    private void write2DStatisticsIDrefs(XMLWriter writer, AbstractPartitionData partitionData) {
-        String prefix = partitionData.getName() + ".";
-        writer.writeIDref("correlation", prefix + "correlation");
-
-        writer.writeIDref("treeLengthStatistic", prefix + "treeLength");
-
-        writer.writeIDref("productStatistic", prefix + "treeLengthPrecision1");
-
-        writer.writeIDref("productStatistic", prefix + "treeLengthPrecision2");
-    }
-
-    /*
- <correlation id="locationCorrelation" dimension1="1" dimension2="2">
- <matrixParameter idref="precisionMatrix"/>
- </correlation>
-
- <treeLengthStatistic id="treeLength">
- <treeModel idref="treeModel"/>
- </treeLengthStatistic>
-
- <productStatistic id="treeLengthPrecision1">
- <treeLengthStatistic idref="treeLength"/>
- <subStatistic id="precision1" dimension="0">  <!-- I do not  know why Joseph programmed these to start counting from 0 -->
- <parameter idref="col1"/>
- </subStatistic>
- </productStatistic>
-
- <productStatistic id="treeLengthPrecision2">
- <treeLengthStatistic idref="treeLength"/>
- <subStatistic id="precision2" dimension="1">
- <parameter idref="col2"/>
- </subStatistic>
- </productStatistic>
-
- <treeDispersionStatistic id="dispersionRate" greatCircleDistance="true">
- <treeModel idref="treeModel"/>
- <multivariateTraitLikelihood idref="traitLikelihood"/>
- </treeDispersionStatistic>
-    */
-
-    private void writeRRWOperators(XMLWriter writer,
-                                   ContinuousComponentOptions component) {
-
-        for (AbstractPartitionData partitionData : component.getOptions().getDataPartitions(ContinuousDataType.INSTANCE)) {
-            ContinuousSubstModelType type = partitionData.getPartitionSubstitutionModel().getContinuousSubstModelType();
-
-            if (type != ContinuousSubstModelType.HOMOGENOUS) {
-                // this is now in the parameter table...
-//                if (type == ContinuousSubstModelType.GAMMA_RRW) {
-//                    writer.writeOpenTag("scaleOperator",
-//                            new Attribute[] {
-//                                    new Attribute.Default<String>("scaleFactor", "0.75"),
-//                                    new Attribute.Default<String>("weight", "1")
-//                            });
-//                    writer.writeIDref("parameter", partitionData.getName() + ".halfDF");
-//                    writer.writeCloseTag("scaleOperator");
-//                }
-
-                writer.writeOpenTag("swapOperator",
-                        new Attribute[] {
-                                new Attribute.Default<String>("size", "1"),
-                                new Attribute.Default<String>("weight", "30"),
-                                new Attribute.Default<String>("autoOptimize", "false")
-                        });
-                writer.writeIDref("parameter", partitionData.getName() + ".rrwCategories");
-                writer.writeCloseTag("swapOperator");
-
-                // See Issue 500:
-                // http://code.google.com/p/beast-mcmc/issues/detail?id=500&can=1&start=400
-//                writer.writeOpenTag("randomWalkIntegerOperator",
-//                        new Attribute[] {
-//                                new Attribute.Default<String>("windowSize", "2"),
-//                                new Attribute.Default<String>("weight", "10")
-//                        });
-//                writer.writeIDref("parameter", partitionData.getName() + ".rrwCategories");
-//                writer.writeCloseTag("randomWalkIntegerOperator");
-
-                writer.writeOpenTag("uniformIntegerOperator",
-                        new Attribute[] {
-                                new Attribute.Default<String>("weight", "10")
-                        });
-                writer.writeIDref("parameter", partitionData.getName() + ".rrwCategories");
-                writer.writeCloseTag("uniformIntegerOperator");
-            }
-        }
+        writer.writeOpenTag("diffusionRateStatistic", (partitionData.getPartitionSubstitutionModel().isLatitudeLongitude() ?
+                new Attribute[] {
+                        new Attribute.Default<String>("id", prefix + "diffusionRate"),
+                        new Attribute.Default<String>("greatCircleDistance", "true")
+                } :
+                new Attribute[] {
+                        new Attribute.Default<String>("id", prefix + "diffusionRate"),
+                }));
+        writer.writeIDref("multivariateTraitLikelihood", traitLikelihoodId);
+        writer.writeCloseTag("diffusionRateStatistic");
     }
 
     private void writePrecisionGibbsOperators(XMLWriter writer,
@@ -507,24 +476,33 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
                                              final ContinuousComponentOptions component,
                                              AbstractPartitionData partitionData
     ) {
-        writer.writeOpenTag("precisionGibbsOperator",
+        writer.writeOpenTag(ContinuousComponentOptions.PRECISION_GIBBS_OPERATOR,
                 new Attribute[] {
                         new Attribute.Default<String>("weight", "" + partitionData.getTraits().size())
                 });
         writer.writeIDref("multivariateTraitLikelihood", partitionData.getName() + ".traitLikelihood");
         writer.writeIDref("multivariateWishartPrior", partitionData.getPartitionSubstitutionModel().getName() + ".precisionPrior");
-        writer.writeCloseTag("precisionGibbsOperator");
+        writer.writeCloseTag(ContinuousComponentOptions.PRECISION_GIBBS_OPERATOR);
 
     }
 
-    private void writePrecisionMatrixIdRefs(final XMLWriter writer, final ContinuousComponentOptions component) {
+    private void writeParameterIdRefs(final XMLWriter writer, final ContinuousComponentOptions component) {
         for (AbstractPartitionData partitionData : component.getOptions().getDataPartitions(ContinuousDataType.INSTANCE)) {
             PartitionSubstitutionModel model = partitionData.getPartitionSubstitutionModel();
             writer.writeIDref("matrixParameter", model.getName() + ".precision");
 
-            if (model.getContinuousTraitCount() == 2) {
-                // if we are analysing bivariate traits we can add these special statistics...
-                write2DStatisticsIDrefs(writer, partitionData);
+            String prefix = partitionData.getName() + ".";
+            if (partitionData.getTraits().size() == 2) {
+                writer.writeIDref("correlation", prefix + "correlation");
+//            writer.writeIDref("treeLengthStatistic", prefix + "treeLength");
+//            writer.writeIDref("productStatistic", prefix + "treeLengthPrecision1");
+//            writer.writeIDref("productStatistic", prefix + "treeLengthPrecision2");
+            }
+            writer.writeIDref("matrixInverse", prefix + "varCovar");
+            writer.writeIDref("diffusionRateStatistic", prefix + "diffusionRate");
+
+            if (component.useLambda(model)) {
+                writer.writeIDref("parameter", model.getName() + "." + ContinuousComponentOptions.LAMBDA);
             }
         }
     }
