@@ -84,7 +84,6 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
             case IN_FILE_LOG_PARAMETERS:
             case IN_FILE_LOG_LIKELIHOODS:
             case AFTER_FILE_LOG:
-//            case IN_TREES_LOG:
                 return true;
             case IN_MCMC_PRIOR:
                 return hasBSSVS();
@@ -93,7 +92,7 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
         }
     }
 
-    protected void generate(final InsertionPoint point, final Object item, final XMLWriter writer) {
+    protected void generate(final InsertionPoint point, final Object item, String prefix, final XMLWriter writer) {
         DiscreteTraitsComponentOptions comp = (DiscreteTraitsComponentOptions)options.getComponentOptions(DiscreteTraitsComponentOptions.class);
 
         switch (point) {
@@ -129,9 +128,6 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
             case AFTER_FILE_LOG:
                 writeDiscreteTraitFileLoggers(writer);
 
-                // This was for ancestral state writing which is now handled by the ancestral state component
-//            case IN_TREES_LOG:
-//                writeTreeLogEntries((PartitionTreeModel)item, writer);
                 break;
             default:
                 throw new IllegalArgumentException("This insertion point is not implemented for " + this.getClass().getName());
@@ -149,10 +145,10 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
         boolean first = true;
         for (PartitionSubstitutionModel model : options.getPartitionSubstitutionModels(GeneralDataType.INSTANCE)) {
             if (!first) {
-                writer.writeBlankLine();
+            writer.writeBlankLine();
             } else {
                 first = false;
-            }
+        }
             writeGeneralDataType(model, writer);
         }
 
@@ -229,7 +225,7 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
 
             writer.writeOpenTag(GeneralSubstitutionModelParser.FREQUENCIES);
 
-            writeDiscreteFrequencyModel(model, stateCount, true, writer);
+            writeDiscreteFrequencyModel(model, "", stateCount, true, writer);
 
             writer.writeCloseTag(GeneralSubstitutionModelParser.FREQUENCIES);
 
@@ -248,7 +244,7 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
 
             writer.writeOpenTag(GeneralSubstitutionModelParser.FREQUENCIES);
 
-            writeDiscreteFrequencyModel(model, stateCount, true, writer);
+            writeDiscreteFrequencyModel(model, "", stateCount, true, writer);
 
             writer.writeCloseTag(GeneralSubstitutionModelParser.FREQUENCIES);
 
@@ -278,21 +274,21 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
         return false;
     }
 
-    private void writeDiscreteFrequencyModel(PartitionSubstitutionModel model, int stateCount, Boolean normalize, XMLWriter writer) {
+    private void writeDiscreteFrequencyModel(PartitionSubstitutionModel model, String name, int stateCount, Boolean normalize, XMLWriter writer) {
         String prefix = model.getName() + ".";
         if (normalize == null) {
             writer.writeOpenTag(FrequencyModelParser.FREQUENCY_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, prefix + FrequencyModelParser.FREQUENCY_MODEL)});
+                    new Attribute.Default<String>(XMLParser.ID, prefix + name + FrequencyModelParser.FREQUENCY_MODEL)});
         } else {
             writer.writeOpenTag(FrequencyModelParser.FREQUENCY_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, prefix + FrequencyModelParser.FREQUENCY_MODEL),
+                    new Attribute.Default<String>(XMLParser.ID, prefix + name + FrequencyModelParser.FREQUENCY_MODEL),
                     new Attribute.Default<Boolean>(FrequencyModelParser.NORMALIZE, normalize)});
         }
 
         writer.writeIDref(GeneralDataTypeParser.GENERAL_DATA_TYPE, prefix + "dataType");
 
         writer.writeOpenTag(FrequencyModelParser.FREQUENCIES);
-        writeParameter(prefix + "frequencies", stateCount, Double.NaN, Double.NaN, Double.NaN, writer);
+        writeParameter(prefix + name + "frequencies", stateCount, Double.NaN, Double.NaN, Double.NaN, writer);
         writer.writeCloseTag(FrequencyModelParser.FREQUENCIES);
 
         writer.writeCloseTag(FrequencyModelParser.FREQUENCY_MODEL);
@@ -356,7 +352,7 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
     }
 
     private void writeTreeLikelihoods(XMLWriter writer,
-                                      DiscreteTraitsComponentOptions component) {
+                                               DiscreteTraitsComponentOptions component) {
         // generate tree likelihoods for discrete trait partitions
         if (options.hasDiscreteTraitPartition()) {
             writer.writeComment("Likelihood for tree given discrete trait data");
@@ -422,6 +418,12 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
                 throw new IllegalArgumentException("Unknown clock model");
         }
 
+        if (substModel.getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
+            int stateCount = options.getStatesForDiscreteModel(substModel).size();
+            writer.writeComment("The root state frequencies");
+            writeDiscreteFrequencyModel(substModel, "root.", stateCount, true, writer);
+        }
+
         getCallingGenerator().generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREE_LIKELIHOOD, partition, writer);
 
         writer.writeCloseTag(treeLikelihoodTag);
@@ -504,7 +506,7 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
     private void writeLogEntries(PartitionSubstitutionModel model, XMLWriter writer) {
         String prefix = model.getName() + ".";
 
-        writer.writeIDref(ParameterParser.PARAMETER, prefix + "rates");
+           writer.writeIDref(ParameterParser.PARAMETER, prefix + "rates");
 
         if (model.isActivateBSSVS()) { //If "BSSVS" is not activated, rateIndicator should not be there.
             writer.writeIDref(ParameterParser.PARAMETER, prefix + "indicators");
@@ -512,16 +514,4 @@ public class DiscreteTraitsComponentGenerator extends BaseComponentGenerator {
         }
     }
 
-    // This was for ancestral state writing which is now handled by the ancestral state component
-//    private void writeTreeLogEntries(PartitionTreeModel treeModel, XMLWriter writer) {
-//        for (AbstractPartitionData partitionData : options.getDataPartitions(GeneralDataType.INSTANCE)) {
-//            if (partitionData.getPartitionTreeModel() == treeModel) {
-//                String prefix = partitionData.getName() + ".";
-//                writer.writeIDref(AncestralStateTreeLikelihoodParser.RECONSTRUCTING_TREE_LIKELIHOOD,
-//                        prefix + TreeLikelihoodParser.TREE_LIKELIHOOD);
-//            }
-//        }
-//    }
-
-
 }
\ No newline at end of file
