@@ -15,22 +15,13 @@ public class TipDateSamplingComponentFactory implements ComponentFactory {
         // singleton pattern - private constructor
     }
 
-    public ComponentGenerator getGenerator(final BeautiOptions beautiOptions) {
-        if (generator == null) {
-            generator = new TipDateSamplingComponentGenerator(beautiOptions);
-        }
-        return generator;
+    public ComponentGenerator createGenerator(final BeautiOptions beautiOptions) {
+        return new TipDateSamplingComponentGenerator(beautiOptions);
     }
 
-    public ComponentOptions getOptions(final BeautiOptions beautiOptions) {
-        if (options == null) {
-            options = new TipDateSamplingComponentOptions(beautiOptions);
-        }
-        return options;
+    public ComponentOptions createOptions(final BeautiOptions beautiOptions) {
+        return new TipDateSamplingComponentOptions(beautiOptions);
     }
 
-    private TipDateSamplingComponentGenerator generator = null;
-    private TipDateSamplingComponentOptions options = null;
-
     public static ComponentFactory INSTANCE = new TipDateSamplingComponentFactory();
 }
\ No newline at end of file
