@@ -1,7 +1,7 @@
 /*
  * BeastGenerator.java
  *
- * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -28,14 +28,14 @@ package dr.app.beauti.generator;
 import dr.app.beast.BeastVersion;
 import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.components.ComponentFactory;
+import dr.app.beauti.components.marginalLikelihoodEstimation.MarginalLikelihoodEstimationOptions;
 import dr.app.beauti.options.*;
 import dr.app.beauti.types.*;
 import dr.app.beauti.util.XMLWriter;
 import dr.app.util.Arguments;
 import dr.evolution.alignment.Alignment;
-import dr.evolution.datatype.ContinuousDataType;
+import dr.evolution.alignment.Patterns;
 import dr.evolution.datatype.DataType;
-import dr.evolution.datatype.GeneralDataType;
 import dr.evolution.datatype.Microsatellite;
 import dr.evolution.util.Taxa;
 import dr.evolution.util.Taxon;
@@ -125,6 +125,40 @@ public class BeastGenerator extends Generator {
      * @throws IllegalArgumentException if there is a problem with the current settings
      */
     public void checkOptions() throws GeneratorException {
+        //++++++++++++++ Microsatellite +++++++++++++++
+        // this has to execute before all checking below
+        // mask all ? from microsatellite data for whose tree only has 1 data partition
+
+        try{
+            if (options.contains(Microsatellite.INSTANCE)) {
+                // clear all masks
+                for (PartitionPattern partitionPattern : options.getPartitionPattern()) {
+                    partitionPattern.getPatterns().clearMask();
+                }
+
+                // set mask
+                for (PartitionTreeModel model : options.getPartitionTreeModels()) {
+                    // if a tree only has 1 data partition, which mostly mean unlinked trees
+                    if (options.getDataPartitions(model).size() == 1) {
+                        PartitionPattern partition = (PartitionPattern) options.getDataPartitions(model).get(0);
+                        Patterns patterns = partition.getPatterns();
+
+                        for (int i = 0; i < patterns.getTaxonCount(); i++) {
+                            int state = patterns.getPatternState(i, 0);
+                            // mask ? from data
+                            if (state < 0) {
+                                patterns.addMask(i);
+                            }
+                        }
+
+//                        System.out.println("mask set = " + patterns.getMaskSet() + " in partition " + partition.getName());
+                    }
+                }
+            }
+        } catch (Exception e) {
+            throw new GeneratorException(e.getMessage());
+        }
+
         //++++++++++++++++ Taxon List ++++++++++++++++++
         TaxonList taxonList = options.taxonList;
         Set<String> ids = new HashSet<String>();
@@ -228,8 +262,8 @@ public class BeastGenerator extends Generator {
         if (options.getPartitionTreeModels().size() > 1) { //TODO not allowed multi-prior yet
             for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
                 if (prior.getNodeHeightPrior() == TreePriorType.GMRF_SKYRIDE) {
-                    throw new GeneratorException("For GMRF, tree model/tree prior combination not implemented by BEAST yet" +
-                            "\nIt is only available for single tree model partition for this release.", BeautiFrame.TREES);
+                    throw new GeneratorException("For the Skyride, tree model/tree prior combination not implemented by BEAST." +
+                            "\nThe Skyride is only available for a single tree model partition in this release.", BeautiFrame.TREES);
                 }
             }
         }
@@ -298,6 +332,18 @@ public class BeastGenerator extends Generator {
             }
         }
 
+        MarginalLikelihoodEstimationOptions mleOptions = (MarginalLikelihoodEstimationOptions)options.getComponentOptions(MarginalLikelihoodEstimationOptions.class);
+        if (mleOptions.performMLE) {
+            for (Parameter param : options.selectParameters()) {
+                    if (param.isPriorImproper() || param.priorType == PriorType.ONE_OVER_X_PRIOR) {
+                        throw new GeneratorException("Parameter \"" + param.getName() + "\":" +
+                                "\nhas an improper prior and will not sample correctly when estimating " +
+                                "the marginal likelihood. " +
+                                "\nPlease check the Prior panel.", BeautiFrame.PRIORS);
+                    }
+            }
+        }
+
         // add other tests and warnings here
         // Speciation model with dated tips
         // Sampling rates without dated tips or priors on rate or nodes
@@ -330,6 +376,15 @@ public class BeastGenerator extends Generator {
         // chance to generate XML at this point in the BEAST file.
         generateInsertionPoint(ComponentGenerator.InsertionPoint.BEFORE_TAXA, writer);
 
+        if (options.originDate != null) {
+            // Create a dummy taxon whose job is to specify the origin date
+            Taxon originTaxon = new Taxon("originTaxon");
+            options.originDate.setUnits(options.units);
+            originTaxon.setDate(options.originDate);
+            writeTaxon(originTaxon, true, false, writer);
+        }
+
+
         //++++++++++++++++ Taxon List ++++++++++++++++++
         try {
             // write complete taxon list
@@ -344,6 +399,13 @@ public class BeastGenerator extends Generator {
                         writeDifferentTaxa(partition, writer);
                     }
                 }
+            } else {
+                // microsat
+                for (PartitionPattern partitionPattern : options.getPartitionPattern()) {
+                    if (partitionPattern.getTaxonList() != null && partitionPattern.getPatterns().hasMask()) {
+                        writeDifferentTaxa(partitionPattern, writer);
+                    }
+                }
             }
         } catch (Exception e) {
             System.err.println(e);
@@ -476,7 +538,7 @@ public class BeastGenerator extends Generator {
             }
 
             for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
-                treePriorGenerator.writeEBSPVariableDemographic(prior, writer);
+                treePriorGenerator.writeMultiLociTreePriors(prior, writer);
             }
 
             generateInsertionPoint(ComponentGenerator.InsertionPoint.AFTER_TREE_PRIOR, writer);
@@ -538,8 +600,8 @@ public class BeastGenerator extends Generator {
                 // generate tree likelihoods for alignment data partitions
                 if (partition.getTaxonList() != null) {
                     if (partition instanceof PartitionData) {
-                        if (partition.getDataType() != GeneralDataType.INSTANCE &&
-                                partition.getDataType() != ContinuousDataType.INSTANCE) {
+                        if (partition.getDataType().getType() != DataType.GENERAL &&
+                                partition.getDataType().getType() != DataType.CONTINUOUS) {
                             treeLikelihoodGenerator.writeTreeLikelihood((PartitionData) partition, writer);
                             writer.writeText("");
                         }
@@ -676,10 +738,6 @@ public class BeastGenerator extends Generator {
 
         boolean hasAttr = options.traits.size() > 0;
 
-        // why was this here?
-//        boolean hasAttr = (options.traits.size() > 0 && !options.useStarBEAST)
-//                || (options.traits.size() > 1 && options.useStarBEAST);
-
         boolean firstDate = true;
         for (int i = 0; i < taxonList.getTaxonCount(); i++) {
             Taxon taxon = taxonList.getTaxon(i);
@@ -689,11 +747,6 @@ public class BeastGenerator extends Generator {
                 hasDate = TaxonList.Utils.hasAttribute(taxonList, i, dr.evolution.util.Date.DATE);
             }
 
-            writer.writeTag(TaxonParser.TAXON, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, taxon.getId())},
-                    !(hasDate || hasAttr)); // false if any of hasDate or hasAttr is true
-
-
             if (hasDate) {
                 dr.evolution.util.Date date = (dr.evolution.util.Date) taxon.getAttribute(dr.evolution.util.Date.DATE);
 
@@ -705,36 +758,69 @@ public class BeastGenerator extends Generator {
                         System.err.println("Error: Units in dates do not match.");
                     }
                 }
+            }
 
-                Attribute[] attributes = {
+            writeTaxon(taxon, hasDate, hasAttr, writer);
+
+        }
+
+        writer.writeCloseTag(TaxaParser.TAXA);
+    }
+
+    /**
+     * Generate a taxa block from these beast options
+     *
+     * @param writer    the writer
+     * @param taxon the taxon to write
+     * @throws dr.app.util.Arguments.ArgumentException
+     *          ArgumentException
+     */
+    private void writeTaxon(Taxon taxon, boolean hasDate, boolean hasAttr, XMLWriter writer) throws Arguments.ArgumentException {
+
+        writer.writeTag(TaxonParser.TAXON, new Attribute[]{
+                new Attribute.Default<String>(XMLParser.ID, taxon.getId())},
+                !(hasDate || hasAttr)); // false if any of hasDate or hasAttr is true
+
+
+        if (hasDate) {
+            dr.evolution.util.Date date = (dr.evolution.util.Date) taxon.getAttribute(dr.evolution.util.Date.DATE);
+
+            Attribute[] attributes;
+            if (date.getPrecision() > 0.0) {
+                attributes = new Attribute[] {
+                        new Attribute.Default<Double>(DateParser.VALUE, date.getTimeValue()),
+                        new Attribute.Default<String>(DateParser.DIRECTION, date.isBackwards() ? DateParser.BACKWARDS : DateParser.FORWARDS),
+                        new Attribute.Default<String>(DateParser.UNITS, Units.Utils.getDefaultUnitName(options.units)),
+                        new Attribute.Default<Double>(DateParser.PRECISION, date.getPrecision())
+                };
+            } else {
+                attributes = new Attribute[] {
                         new Attribute.Default<Double>(DateParser.VALUE, date.getTimeValue()),
                         new Attribute.Default<String>(DateParser.DIRECTION, date.isBackwards() ? DateParser.BACKWARDS : DateParser.FORWARDS),
                         new Attribute.Default<String>(DateParser.UNITS, Units.Utils.getDefaultUnitName(options.units))
                         //new Attribute.Default("origin", date.getOrigin()+"")
                 };
-
-                writer.writeTag(dr.evolution.util.Date.DATE, attributes, true);
             }
 
-            for (TraitData trait : options.traits) {
-                // there is no harm in allowing the species trait to be listed in the taxa
-//                if (!trait.getName().equalsIgnoreCase(TraitData.TRAIT_SPECIES)) {
-                writer.writeOpenTag(AttributeParser.ATTRIBUTE, new Attribute[]{
-                        new Attribute.Default<String>(Attribute.NAME, trait.getName())});
-
-                // denotes missing data using '?'
-                writer.writeText(taxon.containsAttribute(trait.getName()) ? taxon.getAttribute(trait.getName()).toString() : "?");
-                writer.writeCloseTag(AttributeParser.ATTRIBUTE);
-//                }
-            }
-
-            generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TAXON, taxon, writer);
-
-            if (hasDate || hasAttr) writer.writeCloseTag(TaxonParser.TAXON);
-
+            writer.writeTag(dr.evolution.util.Date.DATE, attributes, true);
         }
 
-        writer.writeCloseTag(TaxaParser.TAXA);
+        for (TraitData trait : options.traits) {
+            // there is no harm in allowing the species trait to be listed in the taxa
+//                if (!trait.getName().equalsIgnoreCase(TraitData.TRAIT_SPECIES)) {
+            writer.writeOpenTag(AttributeParser.ATTRIBUTE, new Attribute[]{
+                    new Attribute.Default<String>(Attribute.NAME, trait.getName())});
+
+            // denotes missing data using '?'
+            writer.writeText(taxon.containsAttribute(trait.getName()) ? taxon.getAttribute(trait.getName()).toString() : "?");
+            writer.writeCloseTag(AttributeParser.ATTRIBUTE);
+//                }
+        }
+
+        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TAXON, taxon, writer);
+
+        if (hasDate || hasAttr) writer.writeCloseTag(TaxonParser.TAXON);
+
     }
 
     public void writeDifferentTaxa(AbstractPartitionData dataPartition, XMLWriter writer) {
@@ -746,8 +832,10 @@ public class BeastGenerator extends Generator {
         writer.writeOpenTag(TaxaParser.TAXA, new Attribute[]{new Attribute.Default<String>(XMLParser.ID, name + "." + TaxaParser.TAXA)});
 
         for (int i = 0; i < taxonList.getTaxonCount(); i++) {
-            final Taxon taxon = taxonList.getTaxon(i);
-            writer.writeIDref(TaxonParser.TAXON, taxon.getId());
+            if ( !(dataPartition instanceof PartitionPattern && ((PartitionPattern) dataPartition).getPatterns().isMasked(i) ) ) {
+                final Taxon taxon = taxonList.getTaxon(i);
+                writer.writeIDref(TaxonParser.TAXON, taxon.getId());
+            }
         }
 
         writer.writeCloseTag(TaxaParser.TAXA);
@@ -829,7 +917,7 @@ public class BeastGenerator extends Generator {
         }
 
         for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
-            treePriorGenerator.writeEBSPVariableDemographicReference(prior, writer);
+            treePriorGenerator.writeMultiLociLikelihoodReference(prior, writer);
             writer.writeText("");
         }
 
