@@ -1,24 +1,46 @@
+/*
+ * InitialTreeGenerator.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
 import dr.app.beauti.options.*;
-import dr.app.beauti.types.FixRateType;
 import dr.app.beauti.types.PriorType;
 import dr.app.beauti.types.TreePriorType;
 import dr.app.beauti.util.XMLWriter;
-import dr.evolution.datatype.DataType;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evolution.util.Taxa;
 import dr.evomodelxml.coalescent.CoalescentSimulatorParser;
+import dr.evomodelxml.coalescent.OldCoalescentSimulatorParser;
 import dr.evomodelxml.coalescent.ConstantPopulationModelParser;
 import dr.evomodelxml.coalescent.ExponentialGrowthModelParser;
-import dr.evomodelxml.coalescent.NewCoalescentSimulatorParser;
 import dr.evoxml.*;
 import dr.util.Attribute;
 import dr.xml.XMLParser;
 
-import javax.swing.*;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -44,61 +66,50 @@ public class InitialTreeGenerator extends Generator {
 
         setModelPrefix(model.getPrefix()); // only has prefix, if (options.getPartitionTreeModels().size() > 1)
 
-        Parameter rootHeight = model.getParameter("treeModel.rootHeight");
-
         switch (model.getStartingTreeType()) {
             case USER:
-                if (model.isNewick()) {
-                    writeNewickTree(model.getUserStartingTree(), writer);
-                } else {
-                    writeSimpleTree(model.getUserStartingTree(), writer);
-                }
-                break;
-
             case UPGMA:
-                // generate a upgma starting tree
-                writer.writeComment("Construct a rough-and-ready UPGMA tree as an starting tree");
-                if (rootHeight.priorType != PriorType.NONE_TREE_PRIOR) {
-                    writer.writeOpenTag(
-                            UPGMATreeParser.UPGMA_TREE,
-                            new Attribute[]{
-                                    new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE),
-                                    new Attribute.Default<String>(UPGMATreeParser.ROOT_HEIGHT, "" + rootHeight.initial)
-                            }
-                    );
-                } else {
-                    writer.writeOpenTag(
-                            UPGMATreeParser.UPGMA_TREE,
-                            new Attribute[]{
-                                    new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE)
-                            }
-                    );
-                }
-                writer.writeOpenTag(
-                        DistanceMatrixParser.DISTANCE_MATRIX,
-                        new Attribute[]{
-                                new Attribute.Default<String>(DistanceMatrixParser.CORRECTION, "JC")
+                Parameter rootHeight = model.getParameter("treeModel.rootHeight");
+
+                // generate a rescaled starting tree
+                writer.writeComment("Construct a starting tree that is compatible with specified clade heights");
+                Attribute[] attributes = (rootHeight.priorType != PriorType.NONE_TREE_PRIOR ?
+                        new Attribute[] {
+                                new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE),
+                                new Attribute.Default<String>(RescaledTreeParser.HEIGHT, "" + rootHeight.initial)
+                        } :
+                        new Attribute[] {
+                                new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE)
+                        });
+                writer.writeOpenTag(RescaledTreeParser.RESCALED_TREE, attributes);
+                writeSourceTree(model, writer);
+                if (options.taxonSets != null && options.taxonSets.size() > 0 && !options.useStarBEAST) {
+                    for (Taxa taxa : options.taxonSets) {
+                        Double height = options.taxonSetsHeights.get(taxa);
+                        if (height != null) {
+                            writer.writeOpenTag(RescaledTreeParser.CLADE, new Attribute.Default<String>(RescaledTreeParser.HEIGHT, height.toString()));
+                        } else {
+                            writer.writeOpenTag(RescaledTreeParser.CLADE);
                         }
-                );
-                writer.writeOpenTag(SitePatternsParser.PATTERNS);
-                writer.writeComment("To generate UPGMA starting tree, only use the 1st aligment, "
-                        + "which may be 1 of many aligments using this tree.");
-                writer.writeIDref(AlignmentParser.ALIGNMENT, options.getDataPartitions(model).get(0).getTaxonList().getId());
-                // alignment has no gene prefix
-                writer.writeCloseTag(SitePatternsParser.PATTERNS);
-                writer.writeCloseTag(DistanceMatrixParser.DISTANCE_MATRIX);
-                writer.writeCloseTag(UPGMATreeParser.UPGMA_TREE);
+                        writer.writeTag("taxa", new Attribute.Default<String>(XMLParser.IDREF, taxa.getId()), true);
+                        writer.writeCloseTag(RescaledTreeParser.CLADE);
+                    }
+                }
+
+                writer.writeCloseTag(RescaledTreeParser.RESCALED_TREE);
                 break;
 
             case RANDOM:
                 // generate a coalescent tree
                 String simulatorId = modelPrefix + STARTING_TREE;
 
-                String taxaId;
-                if (options.hasIdenticalTaxa() && options.getPartitionPattern().size() < 1) {
-                    taxaId = TaxaParser.TAXA;
-                } else { // Microsatellite always uses code below
-                    taxaId = options.getDataPartitions(model).get(0).getPrefix() + TaxaParser.TAXA;
+                String taxaId = TaxaParser.TAXA;
+                AbstractPartitionData abstractPartitionData = options.getDataPartitions(model).get(0);
+                if (!options.hasIdenticalTaxa()) {
+                    taxaId = abstractPartitionData.getPrefix() + TaxaParser.TAXA;
+                }
+                if (abstractPartitionData instanceof PartitionPattern && ((PartitionPattern) abstractPartitionData).getPatterns().hasMask()) {
+                    taxaId = abstractPartitionData.getPrefix() + TaxaParser.TAXA;
                 }
 
                 writer.writeComment("Generate a random starting tree under the coalescent process");
@@ -106,7 +117,7 @@ public class InitialTreeGenerator extends Generator {
                     writeSubTree(simulatorId, taxaId, options.taxonList, model, writer);
                 } else {
                     writer.writeOpenTag(
-                            NewCoalescentSimulatorParser.COALESCENT_SIMULATOR,
+                            CoalescentSimulatorParser.COALESCENT_SIMULATOR,
                             new Attribute[]{
                                     new Attribute.Default<String>(XMLParser.ID, simulatorId)
                             }
@@ -124,22 +135,62 @@ public class InitialTreeGenerator extends Generator {
         }
     }
 
+    public void writeSourceTree(PartitionTreeModel model, XMLWriter writer) {
+
+        switch (model.getStartingTreeType()) {
+            case USER:
+                if (model.isNewick()) {
+                    writeNewickTree(model.getUserStartingTree(), writer);
+                } else {
+                    writeSimpleTree(model.getUserStartingTree(), writer);
+                }
+                break;
+
+            case UPGMA:
+                // generate a upgma starting tree
+                writer.writeComment("Construct a rough-and-ready UPGMA tree as an starting tree");
+                writer.writeOpenTag(UPGMATreeParser.UPGMA_TREE);
+                writer.writeOpenTag(
+                        DistanceMatrixParser.DISTANCE_MATRIX,
+                        new Attribute[]{
+                                new Attribute.Default<String>(DistanceMatrixParser.CORRECTION, "JC")
+                        }
+                );
+                writer.writeOpenTag(SitePatternsParser.PATTERNS);
+                writer.writeComment("To generate UPGMA starting tree, only use the 1st aligment, "
+                        + "which may be 1 of many aligments using this tree.");
+                writer.writeIDref(AlignmentParser.ALIGNMENT, options.getDataPartitions(model).get(0).getTaxonList().getId());
+                // alignment has no gene prefix
+                writer.writeCloseTag(SitePatternsParser.PATTERNS);
+                writer.writeCloseTag(DistanceMatrixParser.DISTANCE_MATRIX);
+                writer.writeCloseTag(UPGMATreeParser.UPGMA_TREE);
+                break;
+
+            case RANDOM:
+                throw new IllegalArgumentException("Shouldn't be here");
+
+            default:
+                throw new IllegalArgumentException("Unknown StartingTreeType");
+
+        }
+    }
+
     private void writeTaxaRef(String taxaId, PartitionTreeModel model, XMLWriter writer) {
 
         Attribute[] taxaAttribute = {new Attribute.Default<String>(XMLParser.IDREF, taxaId)};
 
         if (options.taxonSets != null && options.taxonSets.size() > 0 && !options.useStarBEAST) { // need !options.useStarBEAST,
             // *BEAST case is in STARBEASTGenerator.writeStartingTreeForCalibration(XMLWriter writer)
-            writer.writeOpenTag(CoalescentSimulatorParser.CONSTRAINED_TAXA);
+            writer.writeOpenTag(OldCoalescentSimulatorParser.CONSTRAINED_TAXA);
             writer.writeTag(TaxaParser.TAXA, taxaAttribute, true);
             for (Taxa taxa : options.taxonSets) {
                 if (options.taxonSetsTreeModel.get(taxa).equals(model)) {
                     Parameter statistic = options.getStatistic(taxa);
 
                     Attribute mono = new Attribute.Default<Boolean>(
-                            CoalescentSimulatorParser.IS_MONOPHYLETIC, options.taxonSetsMono.get(taxa));
+                            OldCoalescentSimulatorParser.IS_MONOPHYLETIC, options.taxonSetsMono.get(taxa));
 
-                    writer.writeOpenTag(CoalescentSimulatorParser.TMRCA_CONSTRAINT, mono);
+                    writer.writeOpenTag(OldCoalescentSimulatorParser.TMRCA_CONSTRAINT, mono);
 
                     writer.writeIDref(TaxaParser.TAXA, taxa.getId());
 
@@ -148,10 +199,10 @@ public class InitialTreeGenerator extends Generator {
                         writeDistribution(statistic, false, writer);
                     }
 
-                    writer.writeCloseTag(CoalescentSimulatorParser.TMRCA_CONSTRAINT);
+                    writer.writeCloseTag(OldCoalescentSimulatorParser.TMRCA_CONSTRAINT);
                 }
             }
-            writer.writeCloseTag(CoalescentSimulatorParser.CONSTRAINED_TAXA);
+            writer.writeCloseTag(OldCoalescentSimulatorParser.CONSTRAINED_TAXA);
         } else {
             writer.writeTag(TaxaParser.TAXA, taxaAttribute, true);
         }
@@ -173,14 +224,14 @@ public class InitialTreeGenerator extends Generator {
             } else {
                 attributes = new Attribute[] {
                         new Attribute.Default<String>(XMLParser.ID, treeId),
-                        new Attribute.Default<String>(NewCoalescentSimulatorParser.HEIGHT, "" + height)
+                        new Attribute.Default<String>(CoalescentSimulatorParser.HEIGHT, "" + height)
                 };
 
             }
         } else {
             if (!Double.isNaN(height)) {
                 attributes = new Attribute[] {
-                        new Attribute.Default<String>(NewCoalescentSimulatorParser.HEIGHT, "" + height)
+                        new Attribute.Default<String>(CoalescentSimulatorParser.HEIGHT, "" + height)
                 };
 
             }
@@ -189,7 +240,7 @@ public class InitialTreeGenerator extends Generator {
         // construct a subtree
 
         writer.writeOpenTag(
-                NewCoalescentSimulatorParser.COALESCENT_SIMULATOR,
+                CoalescentSimulatorParser.COALESCENT_SIMULATOR,
                 attributes
         );
 
@@ -231,7 +282,7 @@ public class InitialTreeGenerator extends Generator {
             writer.writeIDref(TaxaParser.TAXA, taxaId);
         }
         writeInitialDemoModelRef(model, writer);
-        writer.writeCloseTag(NewCoalescentSimulatorParser.COALESCENT_SIMULATOR);
+        writer.writeCloseTag(CoalescentSimulatorParser.COALESCENT_SIMULATOR);
     }
 
     private void writeInitialDemoModelRef(PartitionTreeModel model, XMLWriter writer) {
@@ -253,7 +304,7 @@ public class InitialTreeGenerator extends Generator {
         writer.writeOpenTag(
                 NewickParser.NEWICK,
                 new Attribute[]{
-                        new Attribute.Default<String>(XMLParser.ID, STARTING_TREE),
+//                        new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE),
 //                        new Attribute.Default<String>(DateParser.UNITS, options.datesUnits.getAttribute()),
                         new Attribute.Default<Boolean>(SimpleTreeParser.USING_DATES, options.clockModelOptions.isTipCalibrated())
                 }
@@ -289,7 +340,7 @@ public class InitialTreeGenerator extends Generator {
         writer.writeOpenTag(
                 SimpleTreeParser.SIMPLE_TREE,
                 new Attribute[]{
-                        new Attribute.Default<String>(XMLParser.ID, STARTING_TREE),
+//                        new Attribute.Default<String>(XMLParser.ID, modelPrefix + STARTING_TREE),
 //                        new Attribute.Default<String>(DateParser.UNITS, options.datesUnits.getAttribute()),
                         new Attribute.Default<Object>(DateParser.UNITS, options.units.toString()),
                         new Attribute.Default<Boolean>(SimpleTreeParser.USING_DATES, options.clockModelOptions.isTipCalibrated())
