@@ -1,3 +1,28 @@
+/*
+ * OperatorsGenerator.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
@@ -7,6 +32,7 @@ import dr.app.beauti.types.FixRateType;
 import dr.app.beauti.types.RelativeRatesType;
 import dr.app.beauti.types.TreePriorType;
 import dr.app.beauti.util.XMLWriter;
+import dr.evolution.datatype.DataType;
 import dr.evomodel.operators.BitFlipInSubstitutionModelOperator;
 import dr.evomodel.substmodel.AbstractSubstitutionModel;
 import dr.evomodel.tree.TreeModel;
@@ -48,19 +74,29 @@ public class OperatorsGenerator extends Generator {
      */
     public void writeOperatorSchedule(List<Operator> operators, XMLWriter writer) {
         Attribute[] operatorAttributes;
-//		switch (options.coolingSchedule) {
-//			case SimpleOperatorSchedule.LOG_SCHEDULE:
-//        if (options.nodeHeightPrior == TreePriorType.GMRF_SKYRIDE) {
-        // TODO: multi-prior, currently simplify to share same prior case
-        if (options.isShareSameTreePrior() && options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.GMRF_SKYRIDE) {
-            operatorAttributes = new Attribute[2];
-            operatorAttributes[1] = new Attribute.Default<String>(SimpleOperatorScheduleParser.OPTIMIZATION_SCHEDULE, SimpleOperatorSchedule.LOG_STRING);
-        } else {
-//				break;
-//			default:
-            operatorAttributes = new Attribute[1];
+
+        // certain models would benefit from a logarithm operator optimization
+        boolean shouldLogCool = false;
+        for (PartitionTreePrior partition : options.getPartitionTreePriors()) {
+            if (partition.getNodeHeightPrior() == TreePriorType.SKYGRID ||
+                    partition.getNodeHeightPrior() == TreePriorType.GMRF_SKYRIDE) {
+                shouldLogCool = true;
+                break;
+            }
         }
-        operatorAttributes[0] = new Attribute.Default<String>(XMLParser.ID, "operators");
+        for (PartitionSubstitutionModel model : options.getPartitionSubstitutionModels()) {
+            if (model.getDataType().getType() == DataType.GENERAL ||
+                    model.getDataType().getType() == DataType.CONTINUOUS) {
+                shouldLogCool = true;
+                break;
+            }
+        }
+
+        operatorAttributes = new Attribute[] {
+                new Attribute.Default<String>(XMLParser.ID, "operators"),
+                new Attribute.Default<String>(SimpleOperatorScheduleParser.OPTIMIZATION_SCHEDULE,
+                        (shouldLogCool ? SimpleOperatorSchedule.LOG_STRING : SimpleOperatorSchedule.DEFAULT_STRING))
+        };
 
         writer.writeComment("Define operators");
         writer.writeOpenTag(
@@ -71,9 +107,9 @@ public class OperatorsGenerator extends Generator {
 
         for (Operator operator : operators) {
             if (operator.weight > 0. && operator.inUse) {
-            	setModelPrefix(operator.getPrefix());
+                setModelPrefix(operator.getPrefix());
 
-            	writeOperator(operator, writer);
+                writeOperator(operator, writer);
             }
         }
 
@@ -111,7 +147,7 @@ public class OperatorsGenerator extends Generator {
                 writeUpDownOperator(MicrosatUpDownOperatorParser.MICROSAT_UP_DOWN_OPERATOR, operator, writer);
                 break;
             case UP_DOWN_ALL_RATES_HEIGHTS:
-            	writeUpDownOperatorAllRatesTrees(operator, writer);
+                writeUpDownOperatorAllRatesTrees(operator, writer);
                 break;
             case SCALE_ALL:
                 writeScaleAllOperator(operator, writer);
@@ -170,9 +206,12 @@ public class OperatorsGenerator extends Generator {
             case GMRF_GIBBS_OPERATOR:
                 writeGMRFGibbsOperator(operator, writer);
                 break;
+            case SKY_GRID_GIBBS_OPERATOR:
+                writeSkyGridGibbsOperator(operator, writer);
+                break;
             case NODE_REHIGHT:
-            	writeSpeciesTreeOperator(operator, writer);
-            	break;
+                writeSpeciesTreeOperator(operator, writer);
+                break;
             default:
                 throw new IllegalArgumentException("Unknown operator type");
         }
@@ -327,7 +366,7 @@ public class OperatorsGenerator extends Generator {
 
         } else if (operator.getBaseName().startsWith(RelativeRatesType.CLOCK_RELATIVE_RATES.toString())) {
 
-        	int[] parameterWeights = options.clockModelOptions.getPartitionClockWeights(operator.getClockModelGroup());
+            int[] parameterWeights = options.clockModelOptions.getPartitionClockWeights(operator.getClockModelGroup());
 
             if (parameterWeights != null && parameterWeights.length > 1) {
                 String pw = "" + parameterWeights[0];
@@ -420,8 +459,8 @@ public class OperatorsGenerator extends Generator {
 
     private void writeTreeBitMoveOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(TreeBitMoveOperatorParser.BIT_MOVE_OPERATOR,
-                        getWeightAttribute(operator.weight));
-        writer.writeIDref(TreeModel.TREE_MODEL,  modelPrefix + TreeModel.TREE_MODEL);
+                getWeightAttribute(operator.weight));
+        writer.writeIDref(TreeModel.TREE_MODEL, modelPrefix + TreeModel.TREE_MODEL);
         writer.writeCloseTag(TreeBitMoveOperatorParser.BIT_MOVE_OPERATOR);
     }
 
@@ -444,21 +483,21 @@ public class OperatorsGenerator extends Generator {
     private void writeNarrowExchangeOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(ExchangeOperatorParser.NARROW_EXCHANGE,
                 getWeightAttribute(operator.weight));
-        writer.writeIDref(TreeModel.TREE_MODEL,  modelPrefix + TreeModel.TREE_MODEL);
+        writer.writeIDref(TreeModel.TREE_MODEL, modelPrefix + TreeModel.TREE_MODEL);
         writer.writeCloseTag(ExchangeOperatorParser.NARROW_EXCHANGE);
     }
 
     private void writeWideExchangeOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(ExchangeOperatorParser.WIDE_EXCHANGE,
                 getWeightAttribute(operator.weight));
-        writer.writeIDref(TreeModel.TREE_MODEL,  modelPrefix + TreeModel.TREE_MODEL);
+        writer.writeIDref(TreeModel.TREE_MODEL, modelPrefix + TreeModel.TREE_MODEL);
         writer.writeCloseTag(ExchangeOperatorParser.WIDE_EXCHANGE);
     }
 
     private void writeWilsonBaldingOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(WilsonBaldingParser.WILSON_BALDING,
                 getWeightAttribute(operator.weight));
-        writer.writeIDref(TreeModel.TREE_MODEL,  modelPrefix + TreeModel.TREE_MODEL);
+        writer.writeIDref(TreeModel.TREE_MODEL, modelPrefix + TreeModel.TREE_MODEL);
         // not supported anymore. probably never worked. (todo) get it out of GUI too
 //        if (options.nodeHeightPrior == TreePriorType.CONSTANT) {
 //            treePriorGenerator.writeNodeHeightPriorModelRef(writer);
@@ -485,6 +524,18 @@ public class OperatorsGenerator extends Generator {
         writer.writeCloseTag(SampleNonActiveGibbsOperatorParser.SAMPLE_NONACTIVE_GIBBS_OPERATOR);
     }
 
+    private void writeSkyGridGibbsOperator(Operator operator, XMLWriter writer) {
+        writer.writeOpenTag(
+                GMRFSkyrideBlockUpdateOperatorParser.GRID_BLOCK_UPDATE_OPERATOR,
+                new Attribute[]{
+                        new Attribute.Default<Double>(GMRFSkyrideBlockUpdateOperatorParser.SCALE_FACTOR, operator.tuning),
+                        getWeightAttribute(operator.weight)
+                }
+        );
+        writer.writeIDref(GMRFSkyrideLikelihoodParser.SKYLINE_LIKELIHOOD, modelPrefix + "skygrid");
+        writer.writeCloseTag(GMRFSkyrideBlockUpdateOperatorParser.GRID_BLOCK_UPDATE_OPERATOR);
+    }
+
     private void writeGMRFGibbsOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(
                 GMRFSkyrideBlockUpdateOperatorParser.BLOCK_UPDATE_OPERATOR,
@@ -493,7 +544,7 @@ public class OperatorsGenerator extends Generator {
                         getWeightAttribute(operator.weight)
                 }
         );
-        writer.writeIDref(GMRFSkyrideLikelihoodParser.SKYLINE_LIKELIHOOD,  modelPrefix + "skyride");
+        writer.writeIDref(GMRFSkyrideLikelihoodParser.SKYLINE_LIKELIHOOD, modelPrefix + "skyride");
         writer.writeCloseTag(GMRFSkyrideBlockUpdateOperatorParser.BLOCK_UPDATE_OPERATOR);
     }
 
@@ -519,16 +570,16 @@ public class OperatorsGenerator extends Generator {
                         getWeightAttribute(operator.weight)
                 }
         );
-        writer.writeIDref(TreeModel.TREE_MODEL,  modelPrefix + TreeModel.TREE_MODEL);
+        writer.writeIDref(TreeModel.TREE_MODEL, modelPrefix + TreeModel.TREE_MODEL);
         writer.writeCloseTag(SubtreeSlideOperatorParser.SUBTREE_SLIDE);
     }
 
     private void writeSpeciesTreeOperator(Operator operator, XMLWriter writer) {
         writer.writeOpenTag(TreeNodeSlideParser.TREE_NODE_REHEIGHT,
-                new Attribute[]{ getWeightAttribute(operator.weight) }
+                new Attribute[]{getWeightAttribute(operator.weight)}
         );
-        writer.writeIDref(TraitData.TRAIT_SPECIES,  TraitData.TRAIT_SPECIES);
-        writer.writeIDref(SpeciesTreeModelParser.SPECIES_TREE,  Generator.SP_TREE);
+        writer.writeIDref(TraitData.TRAIT_SPECIES, TraitData.TRAIT_SPECIES);
+        writer.writeIDref(SpeciesTreeModelParser.SPECIES_TREE, Generator.SP_TREE);
         writer.writeCloseTag(TreeNodeSlideParser.TREE_NODE_REHEIGHT);
     }
 
@@ -543,7 +594,7 @@ public class OperatorsGenerator extends Generator {
         writer.writeOpenTag(UpDownOperatorParser.UP);
         // for isEstimatedRate() = false, write nothing on up part of upDownOp
         if (!operator.parameter1.isFixed && operator.getClockModelGroup().getRateTypeOption() != FixRateType.FIX_MEAN) {
-        	writeParameter1Ref(writer, operator);
+            writeParameter1Ref(writer, operator);
         }
         writer.writeCloseTag(UpDownOperatorParser.UP);
 
@@ -552,7 +603,7 @@ public class OperatorsGenerator extends Generator {
 //	        writer.writeIDref(ParameterParser.PARAMETER,  operator.parameter2.getName());
             writeParameter2Ref(writer, operator);
         } else {
-        	writer.writeIDref(operator.tag,  operator.idref);
+            writer.writeIDref(operator.tag, operator.idref);
         }
         writer.writeCloseTag(UpDownOperatorParser.DOWN);
 
@@ -560,7 +611,7 @@ public class OperatorsGenerator extends Generator {
     }
 
     private void writeUpDownOperatorAllRatesTrees(Operator operator, XMLWriter writer) {
-    	writer.writeOpenTag(UpDownOperatorParser.UP_DOWN_OPERATOR,
+        writer.writeOpenTag(UpDownOperatorParser.UP_DOWN_OPERATOR,
                 new Attribute[]{
                         new Attribute.Default<Double>(ScaleOperatorParser.SCALE_FACTOR, operator.tuning),
                         getWeightAttribute(operator.weight)
@@ -570,45 +621,45 @@ public class OperatorsGenerator extends Generator {
         writer.writeOpenTag(UpDownOperatorParser.UP);
 
         for (PartitionClockModel model : options.getPartitionClockModels()) {
-			if (model.isEstimatedRate()) {
-				switch (model.getClockType()) {
-	            case STRICT_CLOCK:
-	            case RANDOM_LOCAL_CLOCK:
-	            	writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "clock.rate");
-	                break;
+            if (model.isEstimatedRate()) {
+                switch (model.getClockType()) {
+                    case STRICT_CLOCK:
+                    case RANDOM_LOCAL_CLOCK:
+                        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "clock.rate");
+                        break;
 
-	            case UNCORRELATED:
-                    switch (model.getClockDistributionType()) {
-                        case LOGNORMAL:
-                            writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_MEAN);
-                            break;
-                        case GAMMA:
-                            throw new UnsupportedOperationException("Uncorrelated gamma relaxed clock model not implemented yet");
+                    case UNCORRELATED:
+                        switch (model.getClockDistributionType()) {
+                            case LOGNORMAL:
+                                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_MEAN);
+                                break;
+                            case GAMMA:
+                                throw new UnsupportedOperationException("Uncorrelated gamma relaxed clock model not implemented yet");
 //                            break;
-                        case CAUCHY:
-                            throw new UnsupportedOperationException("Uncorrelated Cauchy relaxed clock model not implemented yet");
+                            case CAUCHY:
+                                throw new UnsupportedOperationException("Uncorrelated Cauchy relaxed clock model not implemented yet");
 //                            break;
-                        case EXPONENTIAL:
-                            writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCED_MEAN);
-                            break;
-                    }
-	            	break;
+                            case EXPONENTIAL:
+                                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCED_MEAN);
+                                break;
+                        }
+                        break;
 
-	            case AUTOCORRELATED:
-                    throw new UnsupportedOperationException("Autocorrelated relaxed clock model not implemented yet");
+                    case AUTOCORRELATED:
+                        throw new UnsupportedOperationException("Autocorrelated relaxed clock model not implemented yet");
 //	                break;
 
-	            default:
-	                throw new IllegalArgumentException("Unknown clock model");
-				}
-			}
+                    default:
+                        throw new IllegalArgumentException("Unknown clock model");
+                }
+            }
         }
         if (options.useStarBEAST) {
-	        if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_BIRTH_DEATH) {
-	        	writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME);
-	        } else if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_YULE) {
-	        	writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + YuleModelParser.YULE + "." + YuleModelParser.BIRTH_RATE);
-	        }
+            if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_BIRTH_DEATH) {
+                writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME);
+            } else if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_YULE) {
+                writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + YuleModelParser.YULE + "." + YuleModelParser.BIRTH_RATE);
+            }
         }// nothing for EBSP
 
         writer.writeCloseTag(UpDownOperatorParser.UP);
@@ -616,16 +667,16 @@ public class OperatorsGenerator extends Generator {
         writer.writeOpenTag(UpDownOperatorParser.DOWN);
 
         if (options.useStarBEAST) {
-	        writer.writeIDref(SpeciesTreeModelParser.SPECIES_TREE, SP_TREE); // <speciesTree idref="sptree" /> has to be the 1st always
-	        writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + options.starBEASTOptions.POP_MEAN);
-	        writer.writeIDref(ParameterParser.PARAMETER, SpeciesTreeModelParser.SPECIES_TREE + "." + SPLIT_POPS);
+            writer.writeIDref(SpeciesTreeModelParser.SPECIES_TREE, SP_TREE); // <speciesTree idref="sptree" /> has to be the 1st always
+            writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + options.starBEASTOptions.POP_MEAN);
+            writer.writeIDref(ParameterParser.PARAMETER, SpeciesTreeModelParser.SPECIES_TREE + "." + SPLIT_POPS);
         } else if (options.isEBSPSharingSamePrior()) {
-        	writer.writeIDref(ParameterParser.PARAMETER, VariableDemographicModelParser.demoElementName + ".populationMean");
-	        writer.writeIDref(ParameterParser.PARAMETER, VariableDemographicModelParser.demoElementName + ".popSize");
+            writer.writeIDref(ParameterParser.PARAMETER, VariableDemographicModelParser.demoElementName + ".populationMean");
+            writer.writeIDref(ParameterParser.PARAMETER, VariableDemographicModelParser.demoElementName + ".popSize");
         }
 
         for (PartitionTreeModel tree : options.getPartitionTreeModels()) {
-        	writer.writeIDref(ParameterParser.PARAMETER, tree.getPrefix() + "treeModel.allInternalNodeHeights");
+            writer.writeIDref(ParameterParser.PARAMETER, tree.getPrefix() + "treeModel.allInternalNodeHeights");
         }
 
         writer.writeCloseTag(UpDownOperatorParser.DOWN);
@@ -634,8 +685,8 @@ public class OperatorsGenerator extends Generator {
     }
 
     private Attribute getWeightAttribute(double weight) {
-        if (weight == (int)weight) {
-            return new Attribute.Default<Integer>("weight", (int)weight);
+        if (weight == (int) weight) {
+            return new Attribute.Default<Integer>("weight", (int) weight);
         } else {
             return new Attribute.Default<Double>("weight", weight);
         }
