@@ -31,6 +31,7 @@ import dr.app.beauti.components.ancestralstates.AncestralStatesComponentOptions;
 import dr.app.beauti.options.*;
 import dr.app.beauti.types.MicroSatModelType;
 import dr.app.beauti.util.XMLWriter;
+import dr.evolution.datatype.DataType;
 import dr.evolution.datatype.Nucleotides;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.sitemodel.GammaSiteModel;
@@ -91,7 +92,7 @@ public class TreeLikelihoodGenerator extends Generator {
             }
         }
 
-        if (model.getDataType() == Nucleotides.INSTANCE && model.getCodonHeteroPattern() != null) {
+        if (model.getDataType().getType() == DataType.NUCLEOTIDES && model.getCodonHeteroPattern() != null) {
 
             for (int i = 1; i <= model.getCodonPartitionCount(); i++) {
                 writeTreeLikelihood(treeLikelihoodTag, TreeLikelihoodParser.TREE_LIKELIHOOD, i, partition, writer);
@@ -137,12 +138,16 @@ public class TreeLikelihoodGenerator extends Generator {
                     new Attribute.Default<Integer>(MarkovJumpsTreeLikelihoodParser.NUMBER_OF_SIMULANTS, 1),
                     new Attribute.Default<String>(AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG_NAME, prefix + AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG),
             };
-        } else {
+        } else if (tag.equals(TreeLikelihoodParser.ANCESTRAL_TREE_LIKELIHOOD)) {
             attributes = new Attribute[]{
                     new Attribute.Default<String>(XMLParser.ID, idString),
                     new Attribute.Default<Boolean>(TreeLikelihoodParser.USE_AMBIGUITIES, substModel.isUseAmbiguitiesTreeLikelihood()),
                     new Attribute.Default<String>(AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG_NAME, prefix + AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG),
-
+            };
+        } else {
+            attributes = new Attribute[]{
+                    new Attribute.Default<String>(XMLParser.ID, idString),
+                    new Attribute.Default<Boolean>(TreeLikelihoodParser.USE_AMBIGUITIES, substModel.isUseAmbiguitiesTreeLikelihood())
             };
         }
 
@@ -191,7 +196,7 @@ public class TreeLikelihoodGenerator extends Generator {
                 throw new IllegalArgumentException("Unknown clock model");
         }
 
-        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREE_LIKELIHOOD, partition, writer);
+        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREE_LIKELIHOOD, partition, prefix, writer);
 
         writer.writeCloseTag(tag);
     }
@@ -217,7 +222,7 @@ public class TreeLikelihoodGenerator extends Generator {
                 if (partition instanceof PartitionData && partition.getTraits() == null) {
                     // is an alignment data partition
                     PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
-                    if (substModel.getDataType() == Nucleotides.INSTANCE && substModel.getCodonHeteroPattern() != null) {
+                    if (substModel.getDataType().getType() == DataType.NUCLEOTIDES && substModel.getCodonHeteroPattern() != null) {
                         for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
                             writer.writeIDref(treeLikelihoodTag, partition.getPrefix() + substModel.getPrefix(i) + TreeLikelihoodParser.TREE_LIKELIHOOD);
                         }
@@ -240,7 +245,7 @@ public class TreeLikelihoodGenerator extends Generator {
      */
     public void writeTreeLikelihood(PartitionPattern partition, XMLWriter writer) {
         PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
-        PartitionTreeModel treeModel = partition.getPartitionTreeModel();
+//        PartitionTreeModel treeModel = partition.getPartitionTreeModel();
         PartitionClockModel clockModel = partition.getPartitionClockModel();
 
         writer.writeComment("Microsatellite Sampler Tree Likelihood");
@@ -252,7 +257,7 @@ public class TreeLikelihoodGenerator extends Generator {
         writeMicrosatSubstModelRef(substModel, writer);
 
         writer.writeIDref(MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL,
-                treeModel.getPrefix() + TreeModel.TREE_MODEL + ".microsatellite");
+                partition.getName() + "." + MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL);
 
         switch (clockModel.getClockType()) {
             case STRICT_CLOCK:
