@@ -32,7 +32,7 @@ import dr.app.beauti.options.PartitionTreeModel;
 import dr.app.beauti.util.XMLWriter;
 import dr.evolution.datatype.DataType;
 import dr.evomodel.tree.TreeModel;
-import dr.evomodelxml.coalescent.CoalescentSimulatorParser;
+import dr.evomodelxml.coalescent.OldCoalescentSimulatorParser;
 import dr.evomodelxml.tree.MicrosatelliteSamplerTreeModelParser;
 import dr.evomodelxml.tree.TreeModelParser;
 import dr.evoxml.MicrosatellitePatternParser;
@@ -77,7 +77,7 @@ public class TreeModelGenerator extends Generator {
                 writer.writeIDref(UPGMATreeParser.UPGMA_TREE, modelPrefix + STARTING_TREE);
                 break;
             case RANDOM:
-                writer.writeIDref(CoalescentSimulatorParser.COALESCENT_TREE, modelPrefix + STARTING_TREE);
+                writer.writeIDref(OldCoalescentSimulatorParser.COALESCENT_TREE, modelPrefix + STARTING_TREE);
                 break;
             default:
                 throw new IllegalArgumentException("Unknown StartingTreeType");
@@ -85,7 +85,7 @@ public class TreeModelGenerator extends Generator {
 
         writer.writeOpenTag(TreeModelParser.ROOT_HEIGHT);
         writer.writeTag(ParameterParser.PARAMETER,
-                new Attribute.Default<String>(XMLParser.ID, treeModelName + "." + CoalescentSimulatorParser.ROOT_HEIGHT), true);
+                new Attribute.Default<String>(XMLParser.ID, treeModelName + "." + OldCoalescentSimulatorParser.ROOT_HEIGHT), true);
         writer.writeCloseTag(TreeModelParser.ROOT_HEIGHT);
 
 
@@ -168,7 +168,7 @@ public class TreeModelGenerator extends Generator {
 //            writer.writeCloseTag(TreeModelParser.NODE_TRAITS);
 //        }
 
-        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREE_MODEL, writer);
+        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREE_MODEL, model, writer);
 
         writer.writeCloseTag(TreeModel.TREE_MODEL);
 
@@ -183,27 +183,31 @@ public class TreeModelGenerator extends Generator {
 //        }
 
         if (model.getDataType().getType() == DataType.MICRO_SAT) {
-            writer.writeComment("Generate a microsatellite tree model");
-            writer.writeTag(MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL,
-                    new Attribute.Default<String>(XMLParser.ID, treeModelName + ".microsatellite"), false);
+            for (AbstractPartitionData partitionData : options.getDataPartitions(model)) {
+                writer.writeComment("Generate a microsatellite tree model");
+                writer.writeTag(MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL,
+                        new Attribute.Default<String>(XMLParser.ID, partitionData.getName() + "." +
+                                MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL), false);
 
-            writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.TREE);
-            writer.writeIDref(TreeModel.TREE_MODEL, treeModelName);
-            writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.TREE);
+                writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.TREE);
+                writer.writeIDref(TreeModel.TREE_MODEL, treeModelName);
+                writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.TREE);
 
-            writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.INTERNAL_VALUES);
-            writer.writeTag(ParameterParser.PARAMETER, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, treeModelName + ".microsatellite.internalNodesParameter"),
-                    new Attribute.Default<Integer>(ParameterParser.DIMENSION, model.getDimension())}, true);
-            writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.INTERNAL_VALUES);
+                writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.INTERNAL_VALUES);
+                writer.writeTag(ParameterParser.PARAMETER, new Attribute[]{
+                        new Attribute.Default<String>(XMLParser.ID, partitionData.getName() + "." +
+                               MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL + ".internalNodesParameter"),
+                        new Attribute.Default<Integer>(ParameterParser.DIMENSION, model.getDimension())}, true);
+                writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.INTERNAL_VALUES);
 
-            writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.EXTERNAL_VALUES);
-            for (AbstractPartitionData pattern : options.getDataPartitions(model)) {
-                writer.writeIDref(MicrosatellitePatternParser.MICROSATPATTERN, pattern.getName());
+                writer.writeOpenTag(MicrosatelliteSamplerTreeModelParser.EXTERNAL_VALUES);
+
+                writer.writeIDref(MicrosatellitePatternParser.MICROSATPATTERN, partitionData.getName());
+
+                writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.EXTERNAL_VALUES);
+
+                writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL);
             }
-            writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.EXTERNAL_VALUES);
-
-            writer.writeCloseTag(MicrosatelliteSamplerTreeModelParser.TREE_MICROSATELLITE_SAMPLER_MODEL);
         }
     }
 }
