@@ -28,9 +28,13 @@ package dr.app.beauti.mcmcpanel;
 
 import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.BeautiPanel;
+import dr.app.beauti.components.marginalLikelihoodEstimation.MLEDialog;
+import dr.app.beauti.components.marginalLikelihoodEstimation.MarginalLikelihoodEstimationOptions;
+import dr.app.beauti.components.sequenceerror.SequenceErrorModelComponentOptions;
 import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.PartitionTreeModel;
 import dr.app.beauti.options.STARBEASTOptions;
+import dr.app.beauti.util.PanelUtils;
 import dr.app.gui.components.WholeNumberField;
 import dr.app.util.OSType;
 import dr.evolution.datatype.Microsatellite;
@@ -58,29 +62,34 @@ public class MCMCPanel extends BeautiPanel {
     WholeNumberField logEveryField = new WholeNumberField(1, Integer.MAX_VALUE);
 
     JCheckBox samplePriorCheckBox = new JCheckBox("Sample from prior only - create empty alignment");
+    JCheckBox performMLE = new JCheckBox("Perform marginal likelihood estimation (MLE) using path sampling/stepping-stone sampling");
+    JButton buttonMLE = new JButton("Settings");
 
     public static final String DEFAULT_FILE_NAME_STEM = "untitled";
     JTextField fileNameStemField = new JTextField(DEFAULT_FILE_NAME_STEM);
 
     private JCheckBox addTxt = new JCheckBox("Add .txt suffix");
 
-    JTextField logFileNameField = new JTextField(DEFAULT_FILE_NAME_STEM + ".log");
-    JTextField treeFileNameField = new JTextField(DEFAULT_FILE_NAME_STEM + "." + STARBEASTOptions.TREE_FILE_NAME);
+    JTextArea logFileNameField = new JTextArea(DEFAULT_FILE_NAME_STEM + ".log");
+    JTextArea treeFileNameField = new JTextArea(DEFAULT_FILE_NAME_STEM + "." + STARBEASTOptions.TREE_FILE_NAME);
 //    JCheckBox allowOverwriteLogCheck = new JCheckBox("Allow to overwrite the existing log file");
 
 //    JCheckBox mapTreeLogCheck = new JCheckBox("Create tree file containing the MAP tree:");
 //    JTextField mapTreeFileNameField = new JTextField("untitled.MAP.tree");
 
     JCheckBox substTreeLogCheck = new JCheckBox("Create tree log file with branch length in substitutions:");
-    JTextField substTreeFileNameField = new JTextField("untitled(subst).trees");
+    JTextArea substTreeFileNameField = new JTextArea("untitled(subst).trees");
 
     JCheckBox operatorAnalaysisCheck = new JCheckBox("Create operator analysis file:");
-    JTextField operatorAnalaysisFileNameField = new JTextField(DEFAULT_FILE_NAME_STEM + ".ops");
+    JTextArea operatorAnalaysisFileNameField = new JTextArea(DEFAULT_FILE_NAME_STEM + ".ops");
 
     BeautiFrame frame = null;
     private final OptionsPanel optionsPanel;
     private BeautiOptions options;
 
+    private MLEDialog mleDialog = null;
+    private MarginalLikelihoodEstimationOptions mleOptions;
+
     public MCMCPanel(BeautiFrame parent) {
         setLayout(new BorderLayout());
 
@@ -252,6 +261,45 @@ public class MCMCPanel extends BeautiPanel {
             }
         });
 
+        optionsPanel.addSeparator();
+
+        JTextArea mleInfo = new JTextArea("Select the option below to perform marginal likelihoood " +
+                "estimation (MLE) using path sampling (PS) / stepping-stone sampling (SS) " +
+                "which performs an additional analysis after the standard MCMC chain has finished.");
+        mleInfo.setColumns(50);
+        PanelUtils.setupComponent(mleInfo);
+        optionsPanel.addSpanningComponent(mleInfo);
+
+        optionsPanel.addComponent(performMLE);
+        //will be false by default
+        //options.performMLE = false;
+        optionsPanel.addComponent(buttonMLE);
+        buttonMLE.setEnabled(false);
+        performMLE.addActionListener(new java.awt.event.ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                if (performMLE.isSelected()) {
+                    mleOptions.performMLE = true;
+                    buttonMLE.setEnabled(true);
+                    updateMLEFileNameStem();
+                } else {
+                    mleOptions.performMLE = false;
+                    buttonMLE.setEnabled(false);
+                }
+            }
+        });
+        buttonMLE.addActionListener(new java.awt.event.ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                updateMLEFileNameStem();
+
+                int result = mleDialog.showDialog();
+
+                if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                    return;
+                }
+
+            }
+        });
+
 //        logFileNameField.addKeyListener(listener);
 //        treeFileNameField.addKeyListener(listener);
         //mapTreeFileNameField.addKeyListener(listener);
@@ -274,28 +322,28 @@ public class MCMCPanel extends BeautiPanel {
     private void updateTreeFileNameList() {
         options.treeFileName.clear();
         options.substTreeFileName.clear();
-        String treeFN;
+        String treeFileName;
 
         for (PartitionTreeModel tree : options.getPartitionTreeModels()) {
             if (options.substTreeLog) {
-                treeFN = options.fileNameStem + "." + tree.getPrefix() + "(time)." + STARBEASTOptions.TREE_FILE_NAME;
+                treeFileName = options.fileNameStem + "." + tree.getPrefix() + "(time)." + STARBEASTOptions.TREE_FILE_NAME;
             } else {
-                treeFN = options.fileNameStem + "." + tree.getPrefix() + STARBEASTOptions.TREE_FILE_NAME; // stem.partitionName.tree
+                treeFileName = options.fileNameStem + "." + tree.getPrefix() + STARBEASTOptions.TREE_FILE_NAME; // stem.partitionName.tree
             }
-            if (addTxt.isSelected()) treeFN = treeFN + ".txt";
-            options.treeFileName.add(treeFN);
+            if (addTxt.isSelected()) treeFileName = treeFileName + ".txt";
+            options.treeFileName.add(treeFileName);
 
             if (options.substTreeLog) {
-                treeFN = options.fileNameStem + "." + tree.getPrefix() + "(subst)." + STARBEASTOptions.TREE_FILE_NAME;
-                if (addTxt.isSelected()) treeFN = treeFN + ".txt";
-                options.substTreeFileName.add(treeFN);
+                treeFileName = options.fileNameStem + "." + tree.getPrefix() + "(subst)." + STARBEASTOptions.TREE_FILE_NAME;
+                if (addTxt.isSelected()) treeFileName = treeFileName + ".txt";
+                options.substTreeFileName.add(treeFileName);
             }
         }
 
         if (options.useStarBEAST) {
-            treeFN = options.fileNameStem + "." + options.starBEASTOptions.SPECIES_TREE_FILE_NAME;
-            if (addTxt.isSelected()) treeFN = treeFN + ".txt";
-            options.treeFileName.add(treeFN);
+            treeFileName = options.fileNameStem + "." + options.starBEASTOptions.SPECIES_TREE_FILE_NAME;
+            if (addTxt.isSelected()) treeFileName = treeFileName + ".txt";
+            options.treeFileName.add(treeFileName);
             //TODO: species sub tree
         }
     }
@@ -316,6 +364,13 @@ public class MCMCPanel extends BeautiPanel {
     public void setOptions(BeautiOptions options) {
         this.options = options;
 
+        // get the MLE options
+        mleOptions = (MarginalLikelihoodEstimationOptions)options.getComponentOptions(MarginalLikelihoodEstimationOptions.class);
+
+        if (mleDialog != null) {
+            mleDialog.setOptions(mleOptions);
+        }
+
         chainLengthField.setValue(options.chainLength);
         echoEveryField.setValue(options.echoEvery);
         logEveryField.setValue(options.logEvery);
@@ -384,6 +439,7 @@ public class MCMCPanel extends BeautiPanel {
             substTreeLogCheck.setEnabled(true);
             substTreeLogCheck.setSelected(options.substTreeLog);
 
+            updateMLEFileNameStem();
         } else {
 //            fileNameStemField.setText(fileNameStem);
 //            fileNameStemField.setEnabled(false);
@@ -400,6 +456,13 @@ public class MCMCPanel extends BeautiPanel {
         }
     }
 
+    private void updateMLEFileNameStem() {
+        if (mleDialog == null) {
+            mleDialog = new MLEDialog(frame, mleOptions);
+        }
+        mleDialog.setFilenameStem(options.fileNameStem, addTxt.isSelected());
+    }
+
     public void getOptions(BeautiOptions options) {
         options.fileNameStem = fileNameStemField.getText();
         options.logFileName = logFileNameField.getText();
@@ -414,6 +477,11 @@ public class MCMCPanel extends BeautiPanel {
         options.operatorAnalysisFileName = operatorAnalaysisFileNameField.getText();
 
         options.samplePriorOnly = samplePriorCheckBox.isSelected();
+
+        if (mleDialog != null) {
+            mleDialog.getOptions(mleOptions);
+        }
+
     }
 
     public JComponent getExportableComponent() {
