@@ -25,6 +25,8 @@
 
 package dr.app.beauti.options;
 
+import java.util.*;
+
 import dr.app.beauti.components.ComponentFactory;
 import dr.app.beauti.components.ancestralstates.AncestralStatesComponentOptions;
 import dr.app.beauti.components.continuous.ContinuousComponentOptions;
@@ -33,9 +35,11 @@ import dr.app.beauti.mcmcpanel.MCMCPanel;
 import dr.app.beauti.types.TreePriorType;
 import dr.app.beauti.util.BeautiTemplate;
 import dr.evolution.alignment.Alignment;
+import dr.evolution.alignment.Patterns;
 import dr.evolution.datatype.DataType;
 import dr.evolution.datatype.Microsatellite;
 import dr.evolution.tree.Tree;
+import dr.evolution.util.Date;
 import dr.evolution.util.Taxa;
 import dr.evolution.util.Taxon;
 import dr.evolution.util.TaxonList;
@@ -43,8 +47,6 @@ import dr.evolution.util.Units;
 import dr.evoxml.util.DateUnitsType;
 import dr.inference.operators.OperatorSchedule;
 
-import java.util.*;
-
 /**
  * @author Andrew Rambaut
  * @author Alexei Drummond
@@ -52,6 +54,8 @@ import java.util.*;
  */
 public class BeautiOptions extends ModelOptions {
 
+    private static final long serialVersionUID = -3676802825545741012L;
+
     public BeautiOptions() {
         this(new ComponentFactory[]{});
     }
@@ -60,7 +64,7 @@ public class BeautiOptions extends ModelOptions {
 
         // Install all the component's options from the given list of factories:
         for (ComponentFactory component : components) {
-            addComponent(component.getOptions(this));
+            addComponent(component.createOptions(this));
         }
     }
 
@@ -136,6 +140,8 @@ public class BeautiOptions extends ModelOptions {
         speciesSetsMono.clear();
         starBEASTOptions = new STARBEASTOptions(this);
 
+        microsatelliteOptions = new MicrosatelliteOptions(this);
+
         beautiTemplate = new BeautiTemplate(this);
 
         parameters.clear();
@@ -143,6 +149,8 @@ public class BeautiOptions extends ModelOptions {
         statistics.clear();
 
         shareMicroSat = true;
+
+        clearDataPartitionCaches();
     }
 
     public void selectTaxonSetsStatistics(List<Parameter> params) {
@@ -256,6 +264,10 @@ public class BeautiOptions extends ModelOptions {
             starBEASTOptions.selectParameters(parameters);
         }
 
+        if (contains(Microsatellite.INSTANCE)) {
+            microsatelliteOptions.selectParameters(parameters);
+        }
+
 //        for (TraitData trait : getTraitsList()) { // all traits including locations
 //            if (!trait.getName().equalsIgnoreCase(TraitData.Traits.TRAIT_SPECIES.toString()))
 //        	   trait.gets.selectParameters(parameters);
@@ -310,6 +322,10 @@ public class BeautiOptions extends ModelOptions {
             starBEASTOptions.selectOperators(ops);
         }
 
+        if (contains(Microsatellite.INSTANCE)) {
+            microsatelliteOptions.selectOperators(ops);
+        }
+
 //        for (TraitData trait : getTraitsList()) { // all traits including locations
 //        	if (!trait.getName().equalsIgnoreCase(TraitData.Traits.TRAIT_SPECIES.toString()))
 //                trait.getTraitData().selectOperators(ops);
@@ -442,76 +458,152 @@ public class BeautiOptions extends ModelOptions {
         }
     }
 
+    private final Map<PartitionSubstitutionModel, List<AbstractPartitionData>> psmCache = new HashMap<PartitionSubstitutionModel, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionSubstitutionModel model) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionSubstitutionModel() == model) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = psmCache.get(model);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionSubstitutionModel() == model) {
+                    pdList.add(pd);
+                }
             }
+
+            psmCache.put(model, pdList);
         }
         return pdList;
     }
 
+    private final Map<PartitionClockModel, List<AbstractPartitionData>> pcmCache = new HashMap<PartitionClockModel, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionClockModel model) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionClockModel() == model) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = pcmCache.get(model);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionClockModel() == model) {
+                    pdList.add(pd);
+                }
             }
+
+            pcmCache.put(model, pdList);
         }
         return pdList;
     }
 
+    private final Map<PartitionTreeModel, List<AbstractPartitionData>> ptmCache = new HashMap<PartitionTreeModel, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionTreeModel model) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionTreeModel() == model) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = ptmCache.get(model);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionTreeModel() == model) {
+                    pdList.add(pd);
+                }
             }
+
+            ptmCache.put(model, pdList);
         }
         return pdList;
     }
 
+    private final Map<PartitionTreePrior, List<AbstractPartitionData>> ptpCache = new HashMap<PartitionTreePrior, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionTreePrior prior) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionTreeModel().getPartitionTreePrior() == prior) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = ptpCache.get(prior);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionTreeModel().getPartitionTreePrior() == prior) {
+                    pdList.add(pd);
+                }
             }
+
+            ptpCache.put(prior, pdList);
         }
         return pdList;
     }
 
+    private final Map<PartitionClockModelTreeModelLink, List<AbstractPartitionData>> pcmtmlCache = new HashMap<PartitionClockModelTreeModelLink, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionClockModelTreeModelLink link) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionClockModel() == link.getPartitionClockModel() && pd.getPartitionTreeModel() == link.getPartitionTreeTree()) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = pcmtmlCache.get(link);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionClockModel() == link.getPartitionClockModel() && pd.getPartitionTreeModel() == link.getPartitionTreeTree()) {
+                    pdList.add(pd);
+                }
             }
+
+            pcmtmlCache.put(link, pdList);
         }
         return pdList;
     }
 
+    private final Map<PartitionClockModelSubstModelLink, List<AbstractPartitionData>> pcmsmlCache = new HashMap<PartitionClockModelSubstModelLink, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(PartitionClockModelSubstModelLink link) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionClockModel() == link.getClockModel() && pd.getPartitionSubstitutionModel() == link.getSubstModel()) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = pcmsmlCache.get(link);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionClockModel() == link.getClockModel() && pd.getPartitionSubstitutionModel() == link.getSubstModel()) {
+                    pdList.add(pd);
+                }
             }
+
+            pcmsmlCache.put(link, pdList);
         }
         return pdList;
     }
 
+    private final Map<ClockModelGroup, List<AbstractPartitionData>> cmgCache = new HashMap<ClockModelGroup, List<AbstractPartitionData>>();
+
     public List<AbstractPartitionData> getDataPartitions(ClockModelGroup clockModelGroup) {
-        List<AbstractPartitionData> pdList = new ArrayList<AbstractPartitionData>();
-        for (AbstractPartitionData pd : dataPartitions) {
-            if (pd.getPartitionClockModel() != null && pd.getPartitionClockModel().getClockModelGroup() == clockModelGroup) {
-                pdList.add(pd);
+        List<AbstractPartitionData> pdList = pcmsmlCache.get(clockModelGroup);
+
+        if (pdList == null) {
+            pdList = new ArrayList<AbstractPartitionData>();
+
+            for (AbstractPartitionData pd : dataPartitions) {
+                if (pd.getPartitionClockModel() != null && pd.getPartitionClockModel().getClockModelGroup() == clockModelGroup) {
+                    pdList.add(pd);
+                }
             }
+
+            cmgCache.put(clockModelGroup, pdList);
         }
         return pdList;
     }
 
+    public void clearDataPartitionCaches() {
+        psmCache.clear();
+        pcmCache.clear();
+        ptmCache.clear();
+        ptpCache.clear();
+        pcmtmlCache.clear();
+        pcmsmlCache.clear();
+        cmgCache.clear();
+        psmlCache.clear();
+        ptmlCache.clear();
+        pcmlCache.clear();
+    }
+
     public boolean isEBSPSharingSamePrior() {
         return getPartitionTreePriors().size() >= 1 &&
                 (isShareSameTreePrior() && getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.EXTENDED_SKYLINE);
@@ -538,18 +630,27 @@ public class BeautiOptions extends ModelOptions {
         return models;
     }
 
+    private final Map<List<? extends AbstractPartitionData>, List<PartitionSubstitutionModel>> psmlCache = new HashMap<List<? extends AbstractPartitionData>, List<PartitionSubstitutionModel>>();
+
     public List<PartitionSubstitutionModel> getPartitionSubstitutionModels(List<? extends AbstractPartitionData> givenDataPartitions) {
+        List<PartitionSubstitutionModel> psmList = psmlCache.get(givenDataPartitions);
 
-        List<PartitionSubstitutionModel> activeModels = new ArrayList<PartitionSubstitutionModel>();
+        if (psmList == null) {
+            Set<PartitionSubstitutionModel> activeModels = new LinkedHashSet<PartitionSubstitutionModel>();
 
-        for (AbstractPartitionData partition : givenDataPartitions) {
-            PartitionSubstitutionModel model = partition.getPartitionSubstitutionModel();
-            if (model != null && (!activeModels.contains(model))) {
-                activeModels.add(model);
+            for (AbstractPartitionData partition : givenDataPartitions) {
+                PartitionSubstitutionModel model = partition.getPartitionSubstitutionModel();
+                if (model != null) {
+                    activeModels.add(model);
+                }
             }
+
+            psmList = new ArrayList<PartitionSubstitutionModel>(activeModels);
+
+            psmlCache.put(givenDataPartitions, psmList);
         }
 
-        return activeModels;
+        return psmList;
     }
 
     public List<PartitionSubstitutionModel> getPartitionSubstitutionModels() {
@@ -567,20 +668,27 @@ public class BeautiOptions extends ModelOptions {
 
     // ++++++++++++++ Partition Clock Model ++++++++++++++
 
+    private final Map<List<? extends AbstractPartitionData>, List<PartitionClockModel>> pcmlCache = new HashMap<List<? extends AbstractPartitionData>, List<PartitionClockModel>>();
+
     public List<PartitionClockModel> getPartitionClockModels(List<? extends AbstractPartitionData> givenDataPartitions) {
+        List<PartitionClockModel> pcmList = pcmlCache.get(givenDataPartitions);
 
-        List<PartitionClockModel> activeModels = new ArrayList<PartitionClockModel>();
+        if (pcmList == null) {
+            Set<PartitionClockModel> activeModels = new LinkedHashSet<PartitionClockModel>();
 
-        for (AbstractPartitionData partition : givenDataPartitions) {
-            PartitionClockModel model = partition.getPartitionClockModel();
-            if (model != null && (!activeModels.contains(model))
-                    // species excluded
-                    && (!partition.getDataType().equals(DataType.CONTINUOUS))) {
-                activeModels.add(model);
+            for (AbstractPartitionData partition : givenDataPartitions) {
+                PartitionClockModel model = partition.getPartitionClockModel();
+                if (model != null && !(partition.getDataType().getType() == DataType.CONTINUOUS)) {
+                    activeModels.add(model);
+                }
             }
+
+            pcmList = new ArrayList<PartitionClockModel>(activeModels);
+
+            pcmlCache.put(givenDataPartitions, pcmList);
         }
 
-        return activeModels;
+        return pcmList;
     }
 
     public List<PartitionClockModel> getPartitionClockModels(DataType dataType) {
@@ -627,18 +735,25 @@ public class BeautiOptions extends ModelOptions {
 //        return partitionTreeModels;
 //    }
 
+    private final Map<List<? extends AbstractPartitionData>, List<PartitionTreeModel>> ptmlCache = new HashMap<List<? extends AbstractPartitionData>, List<PartitionTreeModel>>();
+
     public List<PartitionTreeModel> getPartitionTreeModels(List<? extends AbstractPartitionData> givenDataPartitions) {
+        List<PartitionTreeModel> ptmList = ptmlCache.get(givenDataPartitions);
 
-        List<PartitionTreeModel> activeTrees = new ArrayList<PartitionTreeModel>();
+        if (ptmList == null) {
+            Set<PartitionTreeModel> activeTrees = new LinkedHashSet<PartitionTreeModel>();
 
-        for (AbstractPartitionData partition : givenDataPartitions) {
-            PartitionTreeModel tree = partition.getPartitionTreeModel();
-            if (tree != null && (!activeTrees.contains(tree))) {
-                activeTrees.add(tree);
+            for (AbstractPartitionData partition : givenDataPartitions) {
+                if (partition.getPartitionTreeModel() != null) {
+                    activeTrees.add(partition.getPartitionTreeModel());
+                }
             }
-        }
 
-        return activeTrees;
+            ptmList = new ArrayList<PartitionTreeModel>(activeTrees);
+
+            ptmlCache.put(givenDataPartitions, ptmList);
+        }
+        return ptmList;
     }
 
     public List<PartitionTreeModel> getPartitionTreeModels() {
@@ -698,6 +813,7 @@ public class BeautiOptions extends ModelOptions {
                 model.setPartitionTreePrior(ptp);
             }
         }
+        clearDataPartitionCaches();
     }
 
     // link all to given treePrior
@@ -706,6 +822,7 @@ public class BeautiOptions extends ModelOptions {
         for (PartitionTreeModel model : getPartitionTreeModels()) {
             model.setPartitionTreePrior(treePrior);
         }
+        clearDataPartitionCaches();
     }
 
     public boolean isShareSameTreePrior() {
@@ -733,6 +850,7 @@ public class BeautiOptions extends ModelOptions {
     }
 
     public void updatePartitionAllLinks() {
+        clearDataPartitionCaches();
         partitionClockTreeLinks.clear();
         partitionClockSubstLinks.clear();
 
@@ -885,14 +1003,27 @@ public class BeautiOptions extends ModelOptions {
         }
     }
 
+    /**
+     * given a list of BEAUti AbstractPartitionData, take the union of taxon list from them
+     * but if partition instanceof PartitionPattern and taxon is masked in Patterns class, then not count.
+     * @param partitionDataList    can be BEAUti PartitionData or PartitionPattern or both
+     * @return  num of taxon
+     */
     public int getTaxonCount(List<AbstractPartitionData> partitionDataList) {
         if (partitionDataList == null) return 0;
 
         List<String> taxonNameList = new ArrayList<String>();
         for (AbstractPartitionData partition : partitionDataList) {
             for (Taxon t : partition.getTaxonList()) {
-                if (!taxonNameList.contains(t.getId()))
-                    taxonNameList.add(t.getId());
+                if (!taxonNameList.contains(t.getId())) {
+                    if (partition instanceof PartitionPattern) {
+                        Patterns patterns = ((PartitionPattern) partition).getPatterns();
+                        if (!patterns.isMasked(patterns.getTaxonIndex(t)))
+                            taxonNameList.add(t.getId());
+                    } else {
+                        taxonNameList.add(t.getId());
+                    }
+                }
             }
         }
         return taxonNameList.size();
@@ -1188,6 +1319,9 @@ public class BeautiOptions extends ModelOptions {
     public double maximumTipHeight = 0.0;
     public int translation = 0;
 
+    public Date originDate = null;
+
+
     public DateGuesser dateGuesser = new DateGuesser();
 //    public TraitGuesser traitGuesser = new TraitGuesser();
 //
@@ -1226,6 +1360,11 @@ public class BeautiOptions extends ModelOptions {
 
     public String fileNameStem = MCMCPanel.DEFAULT_FILE_NAME_STEM;
     public String logFileName = null;
+
+    public boolean generateDemographicLogFile = false;
+    public String demographicModelName = null;
+    public String demographicLogFileName = null;
+
     public boolean allowOverwriteLog = false;
     //    public boolean mapTreeLog = false;
     //    public String mapTreeFileName = null;
@@ -1244,6 +1383,8 @@ public class BeautiOptions extends ModelOptions {
     public Map<Taxa, Boolean> speciesSetsMono = new HashMap<Taxa, Boolean>();
     public STARBEASTOptions starBEASTOptions = new STARBEASTOptions(this);
 
+    public MicrosatelliteOptions microsatelliteOptions = new MicrosatelliteOptions(this);
+
     public BeautiTemplate beautiTemplate = new BeautiTemplate(this);
 
     public boolean shareMicroSat = true;
