@@ -25,11 +25,15 @@ package dr.app.beauti.options;
 
 import dr.app.beauti.tipdatepanel.GuessDatesException;
 import dr.evolution.util.Date;
+import dr.evolution.util.Taxon;
 import dr.evolution.util.TaxonList;
 import dr.evolution.util.Units;
 
 import java.io.Serializable;
 import java.text.*;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.TimeZone;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -37,6 +41,8 @@ import java.util.regex.Pattern;
  * @author Andrew Rambaut
  */
 public class DateGuesser implements Serializable {
+    private static final long serialVersionUID = -9106689400887615213L;
+
     public enum GuessType {
         ORDER,
         PREFIX,
@@ -53,29 +59,44 @@ public class DateGuesser implements Serializable {
     public double unlessLessThan = 0.0;
     public double offset2 = 0.0;
     public boolean parseCalendarDates = false;
-    public String calendarDateFormat = "dd/MM/yyyy";
+    public boolean parseCalendarDatesAndPrecision = false;
+    public String calendarDateFormat = "yyyy-MM-dd";
 
-    private DateFormat dateFormat = new SimpleDateFormat(calendarDateFormat);
+    private DateFormat dateFormat;
 
     public void guessDates(TaxonList taxonList) {
+        // To avoid duplicating code, add all the taxa into a list and
+        // pass it to guessDates(List<Taxon> taxonList)
+        List<Taxon> taxa = new ArrayList<Taxon>();
+        for (Taxon taxon : taxonList) {
+            taxa.add(taxon);
+        }
+
+        guessDates(taxa);
+    }
+
+    public void guessDates(List<Taxon> taxonList) {
 
         dateFormat = new SimpleDateFormat(calendarDateFormat);
+        dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
 
-        for (int i = 0; i < taxonList.getTaxonCount(); i++) {
+        for (int i = 0; i < taxonList.size(); i++) {
+            // Allocates a Date object and initializes it to represent the specified number of milliseconds since the
+            // standard base time known as "the epoch", namely January 1, 1970, 00:00:00 GMT
             java.util.Date origin = new java.util.Date(0);
 
-            double d = 0.0;
+            double[] values = new double[2];
 
             try {
                 switch (guessType) {
                     case ORDER:
-                        d = guessDateFromOrder(taxonList.getTaxonId(i), order, fromLast);
+                        guessDateFromOrder(taxonList.get(i).getId(), order, fromLast, values);
                         break;
                     case PREFIX:
-                        d = guessDateFromPrefix(taxonList.getTaxonId(i), prefix, order, fromLast);
+                        guessDateFromPrefix(taxonList.get(i).getId(), prefix, order, fromLast, values);
                         break;
                     case REGEX:
-                        d = guessDateFromRegex(taxonList.getTaxonId(i), regex);
+                        guessDateFromRegex(taxonList.get(i).getId(), regex, values);
                         break;
                     default:
                         throw new IllegalArgumentException("unknown GuessType");
@@ -85,6 +106,7 @@ public class DateGuesser implements Serializable {
                 //
             }
 
+            double d = values[0];
             if (offset > 0) {
                 if (unlessLessThan > 0) {
                     if (d < unlessLessThan) {
@@ -98,11 +120,24 @@ public class DateGuesser implements Serializable {
             }
 
             Date date = Date.createTimeSinceOrigin(d, Units.Type.YEARS, origin);
-            taxonList.getTaxon(i).setAttribute("date", date);
+            date.setPrecision(values[1]);
+            taxonList.get(i).setAttribute("date", date);
         }
     }
 
-    private double guessDateFromOrder(String label, int order, boolean fromLast) throws GuessDatesException {
+    public Date parseDate(String value) throws GuessDatesException {
+        double[] values = new double[2];
+        parseDate("", value, values);
+
+        // Allocates a Date object and initializes it to represent the specified number of milliseconds since the
+        // standard base time known as "the epoch", namely January 1, 1970, 00:00:00 GMT
+        java.util.Date origin = new java.util.Date(0);
+
+        return Date.createTimeSinceOrigin(values[0], Units.Type.YEARS, origin);
+
+    }
+
+    private void guessDateFromOrder(String label, int order, boolean fromLast, double[] values) throws GuessDatesException {
 
         String field;
 
@@ -168,12 +203,23 @@ public class DateGuesser implements Serializable {
             } while (count <= order);
         }
 
-        return parseDate(label, field);
+        parseDate(label, field, values);
     }
 
-    private double guessDateFromPrefix(String label, String prefix, int order, boolean fromLast) throws GuessDatesException {
+    private static final String REGEX_CHARACTERS = "|[].*()-^$";
 
-        String[] fields = label.split(prefix);
+    private void guessDateFromPrefix(String label, String prefix, int order, boolean fromLast, double[] values) throws GuessDatesException {
+
+        StringBuilder sb = new StringBuilder();
+        for (int i = 0; i < prefix.length(); i++) {
+            if (REGEX_CHARACTERS.contains("" + prefix.charAt(i))) {
+                sb.append("\\").append(prefix.charAt(i));
+            } else {
+                sb.append(prefix.charAt(i));
+            }
+        }
+
+        String[] fields = label.split(sb.toString());
         int index;
         if (fromLast) {
             index = fields.length - order - 1;
@@ -187,10 +233,10 @@ public class DateGuesser implements Serializable {
             index = fields.length - 1;
         }
 
-        return parseDate(label, fields[index]);
+        parseDate(label, fields[index], values);
     }
 
-    private double guessDateFromPrefix(String label, String prefix) throws GuessDatesException {
+    private void guessDateFromPrefix(String label, String prefix, double[] values) throws GuessDatesException {
 
         int i = label.indexOf(prefix);
 
@@ -210,11 +256,10 @@ public class DateGuesser implements Serializable {
 
         String field = label.substring(j, i + 1);
 
-        return parseDate(label, field);
+        parseDate(label, field, values);
     }
 
-    private double guessDateFromRegex(String label, String regex) throws GuessDatesException {
-        double d;
+    private void guessDateFromRegex(String label, String regex, double[] values) throws GuessDatesException {
 
         if (!regex.contains("(")) {
             // if user hasn't specified a replace element, assume the whole regex should match
@@ -231,23 +276,74 @@ public class DateGuesser implements Serializable {
             throw new GuessDatesException("Date group not defined in regular expression");
         }
 
-        return parseDate(label, matcher.group(0));
+        parseDate(label, matcher.group(0), values);
     }
 
-    private double parseDate(String label, String value) throws GuessDatesException {
+
+    private DateFormat dateFormat1 = null;
+    private DateFormat dateFormat2 = null;
+    private DateFormat dateFormat3 = null;
+
+    private void parseDate(String label, String value, double[] values) throws GuessDatesException {
         double d;
-        try {
-            d = Double.parseDouble(value);
-        } catch (NumberFormatException nfe) {
+        double p = 0.0;
+
+        if (dateFormat1 == null) {
+            // set the timezones to GMT so they match the origin date...
+            dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
+            dateFormat1.setTimeZone(TimeZone.getTimeZone("GMT"));
+            dateFormat2 = new SimpleDateFormat("yyyy-MM");
+            dateFormat2.setTimeZone(TimeZone.getTimeZone("GMT"));
+            dateFormat3 = new SimpleDateFormat("yyyy");
+            dateFormat3.setTimeZone(TimeZone.getTimeZone("GMT"));
+        }
+
+        if (parseCalendarDatesAndPrecision) {
             try {
+                Date date = new Date(dateFormat1.parse(value));
+
+                d = date.getTimeValue();
+                p = 0.0;
+
+            } catch (ParseException pe) {
+                try {
+                    Date date = new Date(dateFormat2.parse(value));
+
+                    d = date.getTimeValue();
+                    p = 1.0 / 12.0;
+
+                } catch (ParseException pe2) {
+                    try {
+                        Date date = new Date(dateFormat3.parse(value));
+
+                        d = date.getTimeValue();
+                        p = 1.0;
+
+                    } catch (ParseException pe3) {
+                        throw new GuessDatesException("Badly formatted date in taxon label, " + label);
+                    }
+                }
+            }
+
+        } else if (parseCalendarDates) {
+            try {
+
                 Date date = new Date(dateFormat.parse(value));
 
                 d = date.getTimeValue();
             } catch (ParseException pe) {
-                throw new GuessDatesException("Badly formated date in taxon label, " + label);
+                throw new GuessDatesException("Badly formatted date in taxon label, " + label);
+            }
+
+        } else {
+            try {
+                d = Double.parseDouble(value);
+            } catch (NumberFormatException nfe) {
+                throw new GuessDatesException("Badly formatted date in taxon label, " + label);
             }
         }
 
-        return d;
+        values[0] = d;
+        values[1] = p;
     }
 }
