@@ -34,6 +34,8 @@ import java.util.List;
  * @version $Id$
  */
 public class PartitionClockModel extends PartitionOptions {
+    private static final long serialVersionUID = -6904595851602060488L;
+
     private static final boolean DEFAULT_CMTC_RATE_REFERENCE_PRIOR = false;
 
     private ClockType clockType = ClockType.STRICT_CLOCK;
@@ -42,8 +44,15 @@ public class PartitionClockModel extends PartitionOptions {
 
     private ClockModelGroup clockModelGroup = null;
 
-    public PartitionClockModel(BeautiOptions options, AbstractPartitionData partition) {
-        super(options, partition.getName());
+    private final int dataLength;
+
+    public PartitionClockModel(final BeautiOptions options, AbstractPartitionData partition) {
+        super(options);
+
+        this.partitionName = partition.getName();
+        dataLength = partition.getSiteCount();
+
+        initModelParametersAndOpererators();
     }
 
     /**
@@ -54,12 +63,19 @@ public class PartitionClockModel extends PartitionOptions {
      * @param source  the source model
      */
     public PartitionClockModel(BeautiOptions options, String name, PartitionClockModel source) {
-        super(options, name);
+        super(options);
+
+        this.partitionName = name;
+
         this.clockType = source.clockType;
         clockDistributionType = source.clockDistributionType;
         rate = source.rate;
 
+        dataLength = source.dataLength;
+
         clockModelGroup = source.clockModelGroup;
+
+        initModelParametersAndOpererators();
     }
 
 //    public PartitionClockModel(BeautiOptions options, String name) {
@@ -69,50 +85,43 @@ public class PartitionClockModel extends PartitionOptions {
 
     protected void initModelParametersAndOpererators() {
         rate = 1.0;
-        int dataLength = 0;
-        for (AbstractPartitionData partitionData : options.getDataPartitions(this)) {
-            dataLength += partitionData.getSiteCount();
-        }
 
         if (DEFAULT_CMTC_RATE_REFERENCE_PRIOR) {
             new Parameter.Builder("clock.rate", "substitution rate").
-                    prior(PriorType.CMTC_RATE_REFERENCE_PRIOR).initial(rate)
+                    prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                     .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
             new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
-                    prior(PriorType.CMTC_RATE_REFERENCE_PRIOR).initial(rate)
+                    prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                     .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
             new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
-                    prior(PriorType.CMTC_RATE_REFERENCE_PRIOR).initial(rate)
+                    prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                     .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
         } else {
-            if (dataLength <= 1) { // TODO Discuss threshold
-                new Parameter.Builder("clock.rate", "substitution rate").prior(PriorType.UNDEFINED).initial(rate)
+            if (dataLength <= 10) { // TODO Discuss threshold
+                new Parameter.Builder("clock.rate", "substitution rate").prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
                 new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
-                        prior(PriorType.UNDEFINED).initial(rate)
+                        prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
                 new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
-                        prior(PriorType.UNDEFINED).initial(rate)
+                        prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
             } else {
                 new Parameter.Builder("clock.rate", "substitution rate").
-                        prior(PriorType.UNIFORM_PRIOR).initial(rate)
-                        .truncationLower(0.0).truncationUpper(100)
+                        prior(PriorType.UNDEFINED).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
                 new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
-                        prior(PriorType.UNIFORM_PRIOR).initial(rate)
-                        .truncationLower(0.0).truncationUpper(100)
+                        prior(PriorType.UNDEFINED).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
                 new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
-                        prior(PriorType.UNIFORM_PRIOR).initial(rate)
-                        .truncationLower(0.0).truncationUpper(100)
+                        prior(PriorType.UNDEFINED).initial(rate)
                         .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
             }
         }
@@ -387,7 +396,7 @@ public class PartitionClockModel extends PartitionOptions {
     public void setUseReferencePrior(boolean useReferencePrior) {
         Parameter rateParam = getClockRateParam();
         if (useReferencePrior) {
-            rateParam.priorType = PriorType.CMTC_RATE_REFERENCE_PRIOR;
+            rateParam.priorType = PriorType.CTMC_RATE_REFERENCE_PRIOR;
         } else {
             rateParam.priorType = PriorType.UNDEFINED;
         }
@@ -409,6 +418,7 @@ public class PartitionClockModel extends PartitionOptions {
     }
 
     public void setClockModelGroup(ClockModelGroup clockModelGroup) {
+        options.clearDataPartitionCaches();
         this.clockModelGroup = clockModelGroup;
     }
 
