@@ -27,6 +27,7 @@ package dr.app.beauti.options;
 
 import dr.app.beauti.types.PriorScaleType;
 import dr.evolution.datatype.DataType;
+import dr.evolution.datatype.Nucleotides;
 import dr.math.MathUtils;
 
 import java.util.List;
@@ -72,7 +73,7 @@ public abstract class PartitionOptions extends ModelOptions {
 //
 //    protected void createParameterClockRateReferencePrior(PartitionOptions options, String name, String description, PriorScaleType scaleType,
 //                                                          double initial) { // it will change to Uniform
-//        new Parameter.Builder(name, description).scaleType(scaleType).prior(PriorType.CMTC_RATE_REFERENCE_PRIOR).initial(initial)
+//        new Parameter.Builder(name, description).scaleType(scaleType).prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(initial)
 //                .isCMTCRate(true).isNonNegative(true)
 //                .truncationLower(truncationLower).truncationUpper(truncationUpper).partitionOptions(options).build(parameters);
 //    }
@@ -148,6 +149,9 @@ public abstract class PartitionOptions extends ModelOptions {
     }
 
     public DataType getDataType() {
+        if (options.getDataPartitions(this).size() == 0) {
+            return Nucleotides.INSTANCE;
+        }
         return options.getDataPartitions(this).get(0).getDataType();
     }
 
