@@ -45,6 +45,7 @@ import java.util.Set;
  * @author Walter Xie
  */
 public class PartitionSubstitutionModel extends PartitionOptions {
+    private static final long serialVersionUID = -2570346396317131108L;
 
     // Instance variables
 
@@ -77,6 +78,7 @@ public class PartitionSubstitutionModel extends PartitionOptions {
     private MicroSatModelType.MutationalBias mutationBias = MicroSatModelType.MutationalBias.UNBIASED;
     private MicroSatModelType.Phase phase = MicroSatModelType.Phase.ONE_PHASE;
     private Microsatellite microsatellite = null;
+    private boolean isLatitudeLongitude = false;
 
     public PartitionSubstitutionModel(BeautiOptions options, AbstractPartitionData partition) {
 //        this(options, partition.getName(),(partition.getTrait() == null)
@@ -315,7 +317,7 @@ public class PartitionSubstitutionModel extends PartitionOptions {
                 0.0, 0.0, 10.0, 0.0);
         createParameterNormalPrior("biasLinear", "Linear bias", PriorScaleType.NONE,
                 0.0, 0.0, 10.0, 0.0);
-        createZeroOneParameterUniformPrior("geomDist", "The success probability of geometric distribution",  0.9);
+        createZeroOneParameterUniformPrior("geomDist", "The success probability of geometric distribution",  0.1);
         createZeroOneParameterUniformPrior("onePhaseProb", "A probability of geomDist being the last step of series", 1.0);
 
         createScaleOperator("propLinear", demoTuning, substWeights);
@@ -888,6 +890,14 @@ public class PartitionSubstitutionModel extends PartitionOptions {
         this.continuousSubstModelType = continuousSubstModelType;
     }
 
+    public void setIsLatitudeLongitude(boolean latitudeLongitude) {
+        isLatitudeLongitude = latitudeLongitude;
+    }
+
+    public boolean isLatitudeLongitude() {
+        return isLatitudeLongitude;
+    }
+
     public int getContinuousTraitCount() {
         return continuousTraitCount;
     }
@@ -1075,4 +1085,37 @@ public class PartitionSubstitutionModel extends PartitionOptions {
         return states;
     }
 
+    public void copyFrom(PartitionSubstitutionModel source) {
+        nucSubstitutionModel = source.nucSubstitutionModel;
+        aaSubstitutionModel = source.aaSubstitutionModel;
+        binarySubstitutionModel = source.binarySubstitutionModel;
+        discreteSubstType = source.discreteSubstType;
+        continuousSubstModelType = source.continuousSubstModelType;
+
+        activateBSSVS = source.activateBSSVS;
+        useAmbiguitiesTreeLikelihood = source.useAmbiguitiesTreeLikelihood;
+
+        frequencyPolicy = source.frequencyPolicy;
+        gammaHetero = source.gammaHetero;
+        gammaCategories = source.gammaCategories;
+        invarHetero = source.invarHetero;
+        codonHeteroPattern = source.codonHeteroPattern;
+        unlinkedSubstitutionModel = source.unlinkedSubstitutionModel;
+        unlinkedHeterogeneityModel = source.unlinkedHeterogeneityModel;
+        unlinkedFrequencyModel = source.unlinkedFrequencyModel;
+
+        dolloModel = source.dolloModel;
+
+        ratePorportion = source.ratePorportion;
+        mutationBias = source.mutationBias;
+        phase = source.phase;
+
+        microsatellite = source.microsatellite;
+    }
+
+    @Override
+    public String toString() {
+        return getName();
+    }
+
 }
\ No newline at end of file
