@@ -42,11 +42,12 @@ import java.util.List;
  * @version $Id$
  */
 public class STARBEASTOptions extends ModelOptions {
+    private static final long serialVersionUID = -668734758207432455L;
 
-	public static final String TREE_FILE_NAME = "trees";
+    public static final String TREE_FILE_NAME = "trees";
 
-    public final String POP_MEAN = "popMean";
-    public final String SPECIES_TREE_FILE_NAME = TraitData.TRAIT_SPECIES
+    public static final String POP_MEAN = "popMean";
+    public static final String SPECIES_TREE_FILE_NAME = TraitData.TRAIT_SPECIES
     							+ "." + STARBEASTOptions.TREE_FILE_NAME; // species.trees
 
     public static final String CITATION = "<html>Joseph Heled and Alexei J. Drummond,<br>" +
