@@ -37,6 +37,7 @@ import java.util.List;
  * @version $Id$
  */
 public class TreeModelOptions extends ModelOptions {
+    private static final long serialVersionUID = 5328826852511460749L;
 
     // Instance variables
     private final BeautiOptions options;
@@ -123,7 +124,7 @@ public class TreeModelOptions extends ModelOptions {
                 || para.priorType == PriorType.GAMMA_PRIOR
                 || para.priorType == PriorType.INVERSE_GAMMA_PRIOR
                 || para.priorType == PriorType.BETA_PRIOR
-                || para.priorType == PriorType.CMTC_RATE_REFERENCE_PRIOR
+                || para.priorType == PriorType.CTMC_RATE_REFERENCE_PRIOR
                 || para.priorType == PriorType.LOGNORMAL_HPM_PRIOR
                 || para.priorType == PriorType.POISSON_PRIOR;
     }
