@@ -25,6 +25,7 @@
 
 package dr.app.beauti.siteModelsPanel;
 
+import dr.app.beauti.components.continuous.ContinuousComponentOptions;
 import dr.app.beauti.components.continuous.ContinuousSubstModelType;
 import dr.app.beauti.components.discrete.DiscreteSubstModelType;
 import dr.app.beauti.components.dollo.DolloComponentOptions;
@@ -103,6 +104,12 @@ public class PartitionModelPanel extends OptionsPanel {
     private JComboBox continuousTraitSiteModelCombo = new JComboBox(
             ContinuousSubstModelType.values());
 
+    private JCheckBox latLongCheck = new JCheckBox(
+            "Bivariate trait represents latitude and longitude");
+
+    private JCheckBox useLambdaCheck = new JCheckBox(
+            "Estimate phylogenetic signal using tree transform");
+
     private JTextArea citationText;
 
     // =========== micro sat ===========
@@ -227,12 +234,6 @@ public class PartitionModelPanel extends OptionsPanel {
             }
         });
 
-        class ListenSetSRD06Button implements ActionListener {
-            public void actionPerformed(ActionEvent ev) {
-                setSRD06Model();
-            }
-        }
-
         setYang96Button = new JButton("Use Yang96 model");
         setYang96Button.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent ev) {
@@ -314,6 +315,9 @@ public class PartitionModelPanel extends OptionsPanel {
         });
 
         PanelUtils.setupComponent(continuousTraitSiteModelCombo);
+        continuousTraitSiteModelCombo
+                .setToolTipText("<html>Select the model of continuous random walk, either homogenous<br>" +
+                        "or relaxed random walk (RRW) with a choice of distributions.</html>");
         continuousTraitSiteModelCombo.addItemListener(new ItemListener() {
             public void itemStateChanged(ItemEvent ev) {
                 model
@@ -322,6 +326,30 @@ public class PartitionModelPanel extends OptionsPanel {
             }
         });
 
+        PanelUtils.setupComponent(latLongCheck);
+        latLongCheck
+                .setToolTipText("<html>Specify whether this is a geographical trait representing <br>"
+                        + "latitude and longitude. Provides appropriate statistics to log file.</html>");
+
+        latLongCheck.addItemListener(new ItemListener() {
+            public void itemStateChanged(ItemEvent ev) {
+                model.setIsLatitudeLongitude(latLongCheck.isSelected());
+            }
+        });
+        latLongCheck.setEnabled(false);
+
+        PanelUtils.setupComponent(useLambdaCheck);
+        useLambdaCheck.addItemListener(new ItemListener() {
+            public void itemStateChanged(ItemEvent ev) {
+                ContinuousComponentOptions component = (ContinuousComponentOptions) model.getOptions()
+                        .getComponentOptions(ContinuousComponentOptions.class);
+                component.setUseLambda(model, useLambdaCheck.isSelected());
+            }
+        });
+        useLambdaCheck
+                .setToolTipText("<html>Estimate degree of phylogenetic correlation in continuous traits using <br>"
+                        + "a tree transform. Inspired by Pagel (1999), described in Lemey et al (2013) <i>in prep</i></html>");
+
         PanelUtils.setupComponent(activateBSSVS);
         activateBSSVS.addItemListener(new ItemListener() {
             public void itemStateChanged(ItemEvent ev) {
@@ -329,7 +357,7 @@ public class PartitionModelPanel extends OptionsPanel {
             }
         });
         activateBSSVS
-                .setToolTipText("<html>Actives Bayesian stochastic search variable selection on the rates as decribed in<br>"
+                .setToolTipText("<html>Activates Bayesian stochastic search variable selection on the rates as described in<br>"
                         + "Lemey, Rambaut, Drummond & Suchard (2009) <i>PLoS Computational Biology</i> <b>5</b>: e1000520</html>");
 
         // ============ micro-sat ================
@@ -462,6 +490,13 @@ public class PartitionModelPanel extends OptionsPanel {
             case DataType.CONTINUOUS:
                 continuousTraitSiteModelCombo.setSelectedItem(model
                         .getContinuousSubstModelType());
+
+                ContinuousComponentOptions component = (ContinuousComponentOptions) model.getOptions()
+                        .getComponentOptions(ContinuousComponentOptions.class);
+
+                latLongCheck.setSelected(model.isLatitudeLongitude());
+                latLongCheck.setEnabled(model.getContinuousTraitCount() == 2);
+                useLambdaCheck.setSelected(component.useLambda(model));
                 break;
             case DataType.MICRO_SAT:
                 microsatName.setText(model.getMicrosatellite().getName());
@@ -615,6 +650,9 @@ public class PartitionModelPanel extends OptionsPanel {
             case DataType.CONTINUOUS:
                 addComponentWithLabel("Continuous Trait Model:",
                         continuousTraitSiteModelCombo);
+                addComponent(latLongCheck);
+                addSeparator();
+                addComponent(useLambdaCheck);
                 break;
 
             case DataType.MICRO_SAT:
