@@ -48,11 +48,11 @@ import javax.swing.table.AbstractTableModel;
 import javax.swing.table.TableColumn;
 import javax.swing.table.TableColumnModel;
 import java.awt.*;
+import java.awt.event.ActionEvent;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
-import java.util.EnumSet;
-import java.util.HashMap;
-import java.util.Map;
+import java.util.*;
+import java.util.List;
 import java.util.logging.Logger;
 
 /**
@@ -68,19 +68,22 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
 
     private static final int MINIMUM_TABLE_WIDTH = 140;
 
-    JTable modelTable = null;
-    ModelTableModel modelTableModel = null;
-    BeautiOptions options = null;
+    private JTable modelTable = null;
+    private ModelTableModel modelTableModel = null;
+    private BeautiOptions options = null;
 
-    JPanel modelPanelParent;
-    PartitionSubstitutionModel currentModel = null;
-    Map<PartitionSubstitutionModel, PartitionModelPanel> modelPanels = new HashMap<PartitionSubstitutionModel, PartitionModelPanel>();
-    TitledBorder modelBorder;
+    private JPanel modelPanelParent;
+    private PartitionSubstitutionModel currentModel = null;
+    private Map<PartitionSubstitutionModel, PartitionModelPanel> modelPanels = new HashMap<PartitionSubstitutionModel, PartitionModelPanel>();
+    private TitledBorder modelBorder;
+    private CloneModelDialog cloneModelDialog = null;
 
     BeautiFrame frame = null;
-//    CreateModelDialog createModelDialog = null;
+    //    CreateModelDialog createModelDialog = null;
     boolean settingOptions = false;
 
+    CloneModelsAction cloneModelsAction = new CloneModelsAction();
+
     public SiteModelsPanel(BeautiFrame parent, Action removeModelAction) {
 
         super();
@@ -101,7 +104,7 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
 
         TableEditorStopper.ensureEditingStopWhenTableLosesFocus(modelTable);
 
-        modelTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
+        modelTable.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
         modelTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
             public void valueChanged(ListSelectionEvent evt) {
                 selectionChanged();
@@ -126,6 +129,16 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
 //        panel.add(controlPanel1, BorderLayout.SOUTH);
         panel.setMinimumSize(new Dimension(MINIMUM_TABLE_WIDTH, 0));
 
+        JToolBar toolBar = new JToolBar();
+        toolBar.setFloatable(false);
+        toolBar.setOpaque(false);
+
+        toolBar.setLayout(new FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));
+        JButton button = new JButton(cloneModelsAction);
+        PanelUtils.setupComponent(button);
+        toolBar.add(button);
+        panel.add(toolBar, BorderLayout.SOUTH);
+
         modelPanelParent = new JPanel(new FlowLayout(FlowLayout.CENTER));
         modelPanelParent.setOpaque(false);
         modelBorder = new TitledBorder("Substitution Model");
@@ -151,17 +164,17 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
     }
 
     private void resetPanel() {
-    	if (!options.hasData()) {
-    		currentModel = null;
-    		modelPanels.clear();
-    		modelPanelParent.removeAll();
-    		modelBorder.setTitle("Substitution Model");
+        if (!options.hasData()) {
+            currentModel = null;
+            modelPanels.clear();
+            modelPanelParent.removeAll();
+            modelBorder.setTitle("Substitution Model");
 
 //            if (currentDiscreteTraitOption != null) {
 //                this.remove(d_splitPane);
 //                currentDiscreteTraitOption = null;
 //            }
-        	return;
+            return;
         }
     }
 
@@ -205,71 +218,134 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
     }
 
     private void selectionChanged() {
-        int selRow = modelTable.getSelectedRow();
+        if (modelTable.getSelectedRowCount() == 1) {
+            int selRow = modelTable.getSelectedRow();
 
-        if (selRow >= options.getPartitionSubstitutionModels().size()) {
-            selRow = 0;
-            modelTable.getSelectionModel().setSelectionInterval(selRow, selRow);
-        }
+            if (selRow >= options.getPartitionSubstitutionModels().size()) {
+                selRow = 0;
+                modelTable.getSelectionModel().setSelectionInterval(selRow, selRow);
+            }
 
-        if (selRow >= 0) {
-            setCurrentModel(options.getPartitionSubstitutionModels().get(selRow));
+            if (selRow >= 0) {
+                setCurrentModel(options.getPartitionSubstitutionModels().get(selRow));
 //            frame.modelSelectionChanged(!isUsed(selRow));
+            }
+        } else {
+            setCurrentModels(getSelectedModels());
         }
     }
 
+    private List<PartitionSubstitutionModel> getSelectedModels() {
+        java.util.List<PartitionSubstitutionModel> models = new ArrayList<PartitionSubstitutionModel>();
+
+        for (int row : modelTable.getSelectedRows()) {
+            models.add(options.getPartitionSubstitutionModels().get(row));
+        }
+        if (models.size() == 0) {
+            models.addAll(options.getPartitionSubstitutionModels());
+        }
+
+        return models;
+    }
+
     /**
      * Sets the current model that this model panel is displaying
      *
      * @param model the new model to display
      */
     private void setCurrentModel(PartitionSubstitutionModel model) {
-        if (model != null) {
-            if (currentModel != null) modelPanelParent.removeAll();
+        modelPanelParent.removeAll();
 
-            PartitionModelPanel panel = modelPanels.get(model);
+        currentModel = model;
+
+        if (currentModel != null) {
+            PartitionModelPanel panel = modelPanels.get(currentModel);
             if (panel == null) {
-                panel = new PartitionModelPanel(model);
-                modelPanels.put(model, panel);
+                panel = new PartitionModelPanel(currentModel);
+                modelPanels.put(currentModel, panel);
             }
 
-            currentModel = model;
             panel.setOptions();
             modelPanelParent.add(panel);
 
-            updateBorder();
+        } else {
+
         }
+
+        cloneModelsAction.setEnabled(true);
+
+        updateBorder();
+    }
+
+    private void setCurrentModels(List<PartitionSubstitutionModel> models) {
+        modelPanelParent.removeAll();
+
+        currentModel = null;
+
+        Set<DataType> dataTypes = new HashSet<DataType>();
+        for (PartitionSubstitutionModel model : models) {
+            dataTypes.add(model.getDataType());
+        }
+
+        if (dataTypes.size() == 1) {
+            DataType dataType = dataTypes.iterator().next();
+            modelBorder.setTitle("Multiple " + dataType.getName() + " substitution models selected");
+        } else {
+            modelBorder.setTitle("Multiple mixed type substitution models selected");
+        }
+
+        cloneModelsAction.setEnabled(dataTypes.size() == 1);
+
+        repaint();
     }
 
     private void updateBorder() {
 
-        String title;
+        if (currentModel != null) {
+            modelBorder.setTitle(currentModel.getDataType().getName() + " Substitution Model - " + currentModel.getName());
+        } else {
+            modelBorder.setTitle("Multiple substitution models selected");
+        }
+        repaint();
+    }
 
-        switch (currentModel.getDataType().getType()) {
-            case DataType.NUCLEOTIDES:
-                title = "Nucleotide";
-                break;
-            case DataType.AMINO_ACIDS:
-                title = "Amino Acid";
-                break;
-            case DataType.TWO_STATES:
-                title = "Binary";
-                break;
-            case DataType.GENERAL:
-                title = "Discrete Traits";
-                break;
-            case DataType.CONTINUOUS:
-                title = "Continuous Traits";
-                break;
-            case DataType.MICRO_SAT:
-                title = "Microsatellite";
-                break;
-            default:
-                throw new IllegalArgumentException("Unsupported data type");
+    private void cloneModelSettings() {
+        if (cloneModelDialog == null) {
+            cloneModelDialog = new CloneModelDialog(frame);
+        }
+
+        Set<DataType> dataTypes = new HashSet<DataType>();
+        for (PartitionSubstitutionModel model : getSelectedModels()) {
+            dataTypes.add(model.getDataType());
+        }
+
+        if (dataTypes.size() == 1) {
+            DataType dataType = dataTypes.iterator().next();
+
+            List<PartitionSubstitutionModel> sourceModels = new ArrayList<PartitionSubstitutionModel>();
+
+            for (PartitionSubstitutionModel model : options.getPartitionSubstitutionModels()) {
+                if (model.getDataType().equals(dataType)) {
+                    sourceModels.add(model);
+                }
+            }
+
+            int result = cloneModelDialog.showDialog(sourceModels);
+
+            if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+
+            PartitionSubstitutionModel sourceModel = cloneModelDialog.getSourceModel();
+            for (PartitionSubstitutionModel model : getSelectedModels()) {
+                if (!model.equals(sourceModel)) {
+                    model.copyFrom(sourceModel);
+                }
+            }
+
+            repaint();
 
         }
-        modelBorder.setTitle(title + " Substitution Model - " + currentModel.getName());
-        repaint();
     }
 
     private boolean isUsed(int row) {
@@ -393,9 +469,16 @@ public class SiteModelsPanel extends BeautiPanel implements Exportable {
 
     }
 
-//    Action addModelAction = new AbstractAction("+") {
-//        public void actionPerformed(ActionEvent ae) {
-//            createModel();
-//        }
-//    };
+    public class CloneModelsAction extends AbstractAction {
+        public CloneModelsAction() {
+            super("Clone Settings...");
+            setToolTipText("Use this tool to copy settings to selected models");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            cloneModelSettings();
+        }
+    }
+
+
 }
\ No newline at end of file
