@@ -63,6 +63,7 @@ public class GuessDatesDialog {
 
     private final JRadioButton numericalRadio = new JRadioButton("Parse as a number", true);
     private final JRadioButton calendarRadio = new JRadioButton("Parse as a calendar date", true);
+    private final JRadioButton calendar2Radio = new JRadioButton("Parse calendar dates with variable precision", true);
 
     private final JCheckBox offsetCheck = new JCheckBox("Add the following value to each: ", false);
     private final RealNumberField offsetText = new RealNumberField();
@@ -73,6 +74,7 @@ public class GuessDatesDialog {
     private final RealNumberField offset2Text = new RealNumberField();
 
     private final JTextField dateFormatText = new JTextField(16);
+    private String description = "Guess Dates for Taxa";
 
     public GuessDatesDialog(final JFrame frame) {
         this.frame = frame;
@@ -131,9 +133,15 @@ public class GuessDatesDialog {
         optionPanel.addComponents(dateFormatLabel, panel);
         dateFormatText.setText("yyyy-MM-dd");
 
+        optionPanel.addSpanningComponent(calendar2Radio);
+
         dateFormatLabel.setEnabled(false);
         dateFormatText.setEnabled(false);
 
+        numericalRadio.setToolTipText("Parse the date field as a decimal number");
+        calendarRadio.setToolTipText("Parse the date field using a standard date format specification");
+        calendar2Radio.setToolTipText("Parse the date field yyyy[-mm[-dd]] with possibly missing month or day");
+
         helpButton.addActionListener(new ActionListener() {
             public void actionPerformed(final ActionEvent actionEvent) {
                 JScrollPane scrollPane = TextUtil.createHTMLScrollPane(
@@ -182,6 +190,7 @@ public class GuessDatesDialog {
         group = new ButtonGroup();
         group.add(numericalRadio);
         group.add(calendarRadio);
+        group.add(calendar2Radio);
         listener = new ItemListener() {
             public void itemStateChanged(ItemEvent e) {
                 offsetCheck.setEnabled(numericalRadio.isSelected());
@@ -197,6 +206,7 @@ public class GuessDatesDialog {
         };
         numericalRadio.addItemListener(listener);
         calendarRadio.addItemListener(listener);
+        calendar2Radio.addItemListener(listener);
     }
 
     public int showDialog() {
@@ -209,7 +219,7 @@ public class GuessDatesDialog {
                 null);
         optionPane.setBorder(new EmptyBorder(12, 12, 12, 12));
 
-        final JDialog dialog = optionPane.createDialog(frame, "Guess Dates for Taxa");
+        final JDialog dialog = optionPane.createDialog(frame, description);
         dialog.pack();
 
         dialog.setVisible(true);
@@ -243,6 +253,7 @@ public class GuessDatesDialog {
             throw new IllegalArgumentException("unknown radio button selected");
         }
 
+        guesser.parseCalendarDatesAndPrecision = calendar2Radio.isSelected();
         guesser.parseCalendarDates = calendarRadio.isSelected();
         guesser.calendarDateFormat = dateFormatText.getText();
 
@@ -259,6 +270,10 @@ public class GuessDatesDialog {
 
     }
 
+    public void setDescription(String description) {
+        this.description = description;
+    }
+
     private static final String DATE_FORMAT_HELP =
             "<h4>Date and Time Patterns</h4>\n" +
             " <p>\n" +
@@ -432,4 +447,5 @@ public class GuessDatesDialog {
             "     interpreted as <a href=\"#text\">text</a>; otherwise,\n" +
             "     it is interpreted as a <a href=\"#number\">number</a>.\n" +
             " </ul>";
+
 }
\ No newline at end of file
