@@ -31,7 +31,6 @@ import dr.app.beauti.components.tipdatesampling.TipDateSamplingComponentOptions;
 import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.ClockModelGroup;
 import dr.app.beauti.options.DateGuesser;
-import dr.app.beauti.traitspanel.TraitValueDialog;
 import dr.app.beauti.types.TipDateSamplingType;
 import dr.app.beauti.util.PanelUtils;
 import dr.app.gui.table.DateCellEditor;
@@ -40,6 +39,7 @@ import dr.app.gui.table.TableSorter;
 import dr.evolution.util.*;
 import dr.evoxml.util.DateUnitsType;
 import jam.framework.Exportable;
+import jam.table.HeaderRenderer;
 import jam.table.TableRenderer;
 
 import javax.swing.*;
@@ -48,9 +48,7 @@ import javax.swing.event.ListSelectionListener;
 import javax.swing.plaf.BorderUIResource;
 import javax.swing.table.AbstractTableModel;
 import java.awt.*;
-import java.awt.event.ActionEvent;
-import java.awt.event.ItemEvent;
-import java.awt.event.ItemListener;
+import java.awt.event.*;
 import java.util.EnumSet;
 
 /**
@@ -73,13 +71,23 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
     ClearDatesAction clearDatesAction = new ClearDatesAction();
     GuessDatesAction guessDatesAction = new GuessDatesAction();
 
+    SetPrecisionAction setPrecisionAction = new SetPrecisionAction();
+
     JCheckBox usingTipDates = new JCheckBox("Use tip dates");
+    JCheckBox specifyOriginDate = new JCheckBox("Specify origin date:");
+    JTextField originDateText = new JTextField(20);
+    JLabel originDateLabel = new JLabel("");
 
     JComboBox unitsCombo = new JComboBox(EnumSet.range(DateUnitsType.YEARS, DateUnitsType.DAYS).toArray());
     JComboBox directionCombo = new JComboBox(EnumSet.range(DateUnitsType.FORWARDS, DateUnitsType.BACKWARDS).toArray());
 
-//    JComboBox tipDateSamplingCombo = new JComboBox( TipDateSamplingType.values() );
-    JComboBox tipDateSamplingCombo = new JComboBox(new TipDateSamplingType[] { TipDateSamplingType.NO_SAMPLING, TipDateSamplingType.SAMPLE_INDIVIDUALLY });
+    //    JComboBox tipDateSamplingCombo = new JComboBox( TipDateSamplingType.values() );
+    JComboBox tipDateSamplingCombo = new JComboBox(new TipDateSamplingType[] {
+            TipDateSamplingType.NO_SAMPLING,
+            TipDateSamplingType.SAMPLE_INDIVIDUALLY,
+//            TipDateSamplingType.SAMPLE_JOINT,
+            TipDateSamplingType.SAMPLE_PRECISION
+    });
     JComboBox tipDateTaxonSetCombo = new JComboBox();
 
     BeautiFrame frame = null;
@@ -89,7 +97,8 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
     double[] heights = null;
 
     private GuessDatesDialog guessDatesDialog = null;
-    private DateValueDialog dateValueDialog = null;
+    private SetValueDialog dateValueDialog = null;
+    private SetValueDialog precisionValueDialog = null;
 
     public TipDatesPanel(BeautiFrame parent) {
 
@@ -105,6 +114,10 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 //        dataTable.getTableHeader().setDefaultRenderer(
 //                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
+        dataTable.getTableHeader().setReorderingAllowed(false);
+        dataTable.getTableHeader().setDefaultRenderer(
+                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+
         dataTable.getColumnModel().getColumn(0).setCellRenderer(
                 new TableRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
         dataTable.getColumnModel().getColumn(0).setPreferredWidth(80);
@@ -118,6 +131,12 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         dataTable.getColumnModel().getColumn(2).setCellRenderer(
                 new TableRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
         dataTable.getColumnModel().getColumn(2).setPreferredWidth(80);
+        dataTable.getColumnModel().getColumn(2).setCellEditor(
+                new DateCellEditor());
+
+        dataTable.getColumnModel().getColumn(3).setCellRenderer(
+                new TableRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+        dataTable.getColumnModel().getColumn(3).setPreferredWidth(80);
 
         TableEditorStopper.ensureEditingStopWhenTableLosesFocus(dataTable);
 
@@ -152,15 +171,35 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         PanelUtils.setupComponent(button);
         toolBar1.add(button);
 
+        button = new JButton(setPrecisionAction);
+        PanelUtils.setupComponent(button);
+        toolBar1.add(button);
+
         toolBar1.add(new JToolBar.Separator(new Dimension(12, 12)));
         final JLabel unitsLabel = new JLabel("Dates specified as ");
         toolBar1.add(unitsLabel);
         toolBar1.add(unitsCombo);
         toolBar1.add(directionCombo);
 
+        JToolBar toolBar3 = new JToolBar();
+        toolBar3.setFloatable(false);
+        toolBar3.setOpaque(false);
+
+        toolBar3.setLayout(new FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));
+
+        toolBar3.add(specifyOriginDate);
+        toolBar3.add(originDateText);
+        toolBar3.add(originDateLabel);
+        JPanel panel2 = new JPanel();
+        panel2.setLayout(new BoxLayout(panel2, BoxLayout.PAGE_AXIS));
+        panel2.setOpaque(false);
+        panel2.add(toolBar1);
+        panel2.add(toolBar3);
+
+
         JPanel panel1 = new JPanel(new BorderLayout(0, 0));
         panel1.setOpaque(false);
-        panel1.add(toolBar1, "North");
+        panel1.add(panel2, "North");
         panel1.add(scrollPane, "Center");
 
         JToolBar toolBar2 = new JToolBar();
@@ -184,7 +223,6 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         toolBar2.add(tipDateTaxonSetLabel);
         toolBar2.add(tipDateTaxonSetCombo);
 
-
         setOpaque(false);
         setBorder(new BorderUIResource.EmptyBorderUIResource(new java.awt.Insets(12, 12, 12, 12)));
         setLayout(new BorderLayout(0, 0));
@@ -203,9 +241,30 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                 }
         );
 
+        specifyOriginDate.addItemListener(
+                new java.awt.event.ItemListener() {
+                    public void itemStateChanged(java.awt.event.ItemEvent ev) {
+                        boolean enabled = usingTipDates.isSelected();
+                        originDateText.setEnabled(enabled && specifyOriginDate.isSelected());
+                        originDateLabel.setEnabled(enabled && specifyOriginDate.isSelected());
+                        timeScaleChanged();
+                    }
+                }
+        );
+
+        originDateText.addFocusListener(
+                new FocusAdapter() {
+                    @Override
+                    public void focusLost(FocusEvent focusEvent) {
+                        timeScaleChanged();
+                    }
+                }
+        );
+
         clearDatesAction.setEnabled(false);
         guessDatesAction.setEnabled(false);
         setDatesAction.setEnabled(false);
+        setPrecisionAction.setEnabled(false);
         directionCombo.setEnabled(false);
         unitsLabel.setEnabled(false);
         unitsCombo.setEnabled(false);
@@ -215,6 +274,9 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         tipDateSamplingCombo.setEnabled(false);
         tipDateTaxonSetLabel.setEnabled(false);
         tipDateTaxonSetCombo.setEnabled(false);
+        specifyOriginDate.setEnabled(false);
+        originDateText.setEnabled(false);
+        originDateLabel.setEnabled(false);
 
         usingTipDates.addItemListener(new ItemListener() {
             public void itemStateChanged(ItemEvent ev) {
@@ -222,6 +284,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                 clearDatesAction.setEnabled(enabled);
                 guessDatesAction.setEnabled(enabled);
                 setDatesAction.setEnabled(enabled);
+                setPrecisionAction.setEnabled(enabled);
                 unitsLabel.setEnabled(enabled);
                 unitsCombo.setEnabled(enabled);
                 directionCombo.setEnabled(enabled);
@@ -229,6 +292,9 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                 dataTable.setEnabled(enabled);
                 tipDateSamplingCombo.setEnabled(enabled);
                 tipDateSamplingLabel.setEnabled(enabled);
+                specifyOriginDate.setEnabled(enabled);
+                originDateText.setEnabled(enabled && specifyOriginDate.isSelected());
+                originDateLabel.setEnabled(enabled && specifyOriginDate.isSelected());
 
                 if (options.taxonList != null) timeScaleChanged();
             }
@@ -265,9 +331,30 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
             Date newDate = createDate(d, units, backwards, 0.0);
 
+            newDate.setPrecision(date.getPrecision());
+
             options.taxonList.getTaxon(i).setDate(newDate);
         }
 
+        if (specifyOriginDate.isSelected()) {
+            String text = originDateText.getText();
+            DateGuesser guesser = options.dateGuesser;
+            guessDatesDialog.setupGuesser(guesser);
+
+            try {
+                options.originDate = guesser.parseDate(text);
+            } catch (GuessDatesException e) {
+                options.originDate = null;
+            }
+        } else {
+            options.originDate = null;
+        }
+        if ( options.originDate != null) {
+            originDateLabel.setText(" date value: " + Double.toString(options.originDate.getTimeValue()));
+        } else {
+            originDateLabel.setText(" unable to parse date");
+        }
+
         calculateHeights();
 
         if (options.clockModelOptions.isTipCalibrated()) { // todo correct?
@@ -351,12 +438,14 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         int result;
         do {
             if (dateValueDialog == null) {
-                dateValueDialog = new DateValueDialog(frame);
+                dateValueDialog = new SetValueDialog(frame, "Set Date for Taxa");
             }
 
             int[] selRows = dataTable.getSelectedRows();
 
-            if (selRows.length > 0) {
+            if (selRows.length == 1) {
+                precisionValueDialog.setDescription("Set date value for selected taxon");
+            } else if (selRows.length > 1) {
                 dateValueDialog.setDescription("Set date values for selected taxa");
             } else {
                 dateValueDialog.setDescription("Set date values for all taxa");
@@ -369,7 +458,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
             }
 
 //            currentTrait.guessTrait = true; // ?? no use?
-            String value = dateValueDialog.getDateValue();
+            String value = dateValueDialog.getValue();
 
             java.util.Date origin = new java.util.Date(0);
             double d = Double.parseDouble(value);
@@ -388,7 +477,58 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
             // adjust the dates to the current timescale...
             timeScaleChanged();
 
-           dataTableModel.fireTableDataChanged();
+            dataTableModel.fireTableDataChanged();
+        } while (result < 0);
+    }
+
+    public void setPrecisions() {
+        if (options.taxonList == null) { // validation of check empty taxonList
+            return;
+        }
+
+        int result;
+        do {
+            if (precisionValueDialog == null) {
+                precisionValueDialog = new SetValueDialog(frame, "Set Precision for Taxa");
+            }
+
+            int[] selRows = dataTable.getSelectedRows();
+
+            if (selRows.length == 1) {
+                precisionValueDialog.setDescription("Set precision value for selected taxon");
+            } else if (selRows.length > 1) {
+                precisionValueDialog.setDescription("Set precision values for selected taxa");
+            } else {
+                precisionValueDialog.setDescription("Set precision values for all taxa");
+            }
+
+            result = precisionValueDialog.showDialog();
+
+            if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+
+            double value;
+            try {
+                value = Double.parseDouble(precisionValueDialog.getValue());
+            } catch (NumberFormatException nfe) {
+                value = 0.0;
+            }
+            if (value < 0.0 || Double.isNaN(value) || Double.isInfinite(value)) {
+                value = 0.0;
+            }
+
+            if (selRows.length > 0) {
+                for (int row : selRows) {
+                    options.taxonList.getTaxon(row).getDate().setPrecision(value);
+                }
+            } else {
+                for (Taxon taxon : options.taxonList) {
+                    taxon.setAttribute("precision", value);
+                }
+            }
+
+            dataTableModel.fireTableDataChanged();
         } while (result < 0);
     }
 
@@ -414,6 +554,15 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
             guessDatesDialog = new GuessDatesDialog(frame);
         }
 
+        int[] selRows = dataTable.getSelectedRows();
+
+        if (selRows.length > 0) {
+            guessDatesDialog.setDescription("Guess date values for selected taxa");
+        } else {
+            guessDatesDialog.setDescription("Guess date values for all taxa");
+        }
+
+
         int result = guessDatesDialog.showDialog();
 
         if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
@@ -427,7 +576,17 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         String warningMessage = null;
 
-        guesser.guessDates(options.taxonList);
+            if (selRows.length > 0) {
+            Taxa selectedTaxa = new Taxa();
+
+            for (int row : selRows) {
+                Taxon taxon = (Taxon) dataTable.getValueAt(row, 0);
+                selectedTaxa.addTaxon(taxon);
+            }
+            guesser.guessDates(selectedTaxa);
+        } else {
+            guesser.guessDates(options.taxonList);
+        }
 
         if (warningMessage != null) {
             JOptionPane.showMessageDialog(this, "Warning: some dates may not be set correctly - \n" + warningMessage,
@@ -449,7 +608,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         public SetDatesAction() {
             super("Set Dates");
-            setToolTipText("Use this tool to set sampling date values from selected taxa");
+            setToolTipText("Use this tool to set sampling date values for the selected taxa");
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -489,6 +648,22 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         }
     }
 
+    public class SetPrecisionAction extends AbstractAction {
+        /**
+         *
+         */
+        private static final long serialVersionUID = -7281309694753868639L;
+
+        public SetPrecisionAction() {
+            super("Set Precision");
+            setToolTipText("Use this tool to set precision values for the selected taxa");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            setPrecisions();
+        }
+    }
+
     private void calculateHeights() {
 
         options.maximumTipHeight = 0.0;
@@ -496,7 +671,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         heights = null;
 
-        dr.evolution.util.Date mostRecent = null;
+        dr.evolution.util.Date mostRecent = options.originDate;
         for (int i = 0; i < options.taxonList.getTaxonCount(); i++) {
             Date date = options.taxonList.getTaxon(i).getDate();
             if ((date != null) && (mostRecent == null || date.after(mostRecent))) {
@@ -530,7 +705,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
          *
          */
         private static final long serialVersionUID = -6707994233020715574L;
-        String[] columnNames = {"Name", "Date", "Height"};
+        String[] columnNames = {"Name", "Date", "Precision", "Height"};
 
         public DataTableModel() {
         }
@@ -547,17 +722,24 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         }
 
         public Object getValueAt(int row, int col) {
+            Date date = options.taxonList.getTaxon(row).getDate();
+
             switch (col) {
                 case 0:
-                    return options.taxonList.getTaxonId(row);
+                    return options.taxonList.getTaxon(row);
                 case 1:
-                    Date date = options.taxonList.getTaxon(row).getDate();
                     if (date != null) {
                         return date.getTimeValue();
                     } else {
                         return "-";
                     }
                 case 2:
+                    if (date != null) {
+                        return date.getPrecision();
+                    } else {
+                        return "-";
+                    }
+                case 3:
                     if (heights != null) {
                         return heights[row];
                     } else {
@@ -577,6 +759,14 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                     Date newDate = createDate(d, date.getUnits(), date.isBackwards(), date.getOrigin());
                     options.taxonList.getTaxon(row).setDate(newDate);
                 }
+            } else if (col == 2) {
+                Date date = options.taxonList.getTaxon(row).getDate();
+                if (date != null) {
+                    double d = (Double) aValue;
+                    if (d >= 0.0) {
+                        date.setPrecision(d);
+                    }
+                }
             }
 
             timeScaleChanged();
@@ -584,7 +774,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         public boolean isCellEditable(int row, int col) {
             if (col == 0) return false;
-            if (col == 1) {
+            if (col == 1 || col == 2) {
                 Date date = options.taxonList.getTaxon(row).getDate();
                 return (date != null);
             }
