@@ -7,7 +7,8 @@ public enum TipDateSamplingType {
 
     NO_SAMPLING("Off"),
     SAMPLE_INDIVIDUALLY("Sampling with individual priors"),
-    SAMPLE_JOINT("Sampling with joint priors");
+    SAMPLE_JOINT("Sampling with joint priors"),
+    SAMPLE_PRECISION("Sampling uniformly from precision");
 
     TipDateSamplingType(String displayName) {
         this.displayName = displayName;
