@@ -69,13 +69,13 @@ public class TreesPanel extends JPanel implements Exportable {
     private Tree currentTree = null;
     private Tree bestFittingRootTree = null;
 
-    PathogenFrame frame = null;
-    JTabbedPane tabbedPane = new JTabbedPane();
-    JTextArea textArea = new JTextArea();
-    JCheckBox showMRCACheck = new JCheckBox("Show ancestor traces");
+    private final PathogenFrame frame;
+    private final JTabbedPane tabbedPane = new JTabbedPane();
+    private final JTextArea textArea = new JTextArea();
+    private final JCheckBox showMRCACheck = new JCheckBox("Show ancestor traces");
 
     //    JTreeDisplay treePanel;
-    FigTreePanel treePanel;
+    private final FigTreePanel treePanel;
 
     JChartPanel rootToTipPanel;
     JChart rootToTipChart;
@@ -90,6 +90,7 @@ public class TreesPanel extends JPanel implements Exportable {
     JChart residualChart;
     ScatterPlot residualPlot;
 
+    ErrorBarPlot errorBarPlot;
     ParentPlot mrcaPlot;
 
     Map<Node, Integer> pointMap = new HashMap<Node, Integer>();
@@ -101,6 +102,8 @@ public class TreesPanel extends JPanel implements Exportable {
     private TemporalRooting temporalRooting = null;
 
     public TreesPanel(PathogenFrame parent, Tree tree) {
+        frame = parent;
+
         statisticsModel = new StatisticsModel();
         statisticsTable = new JTable(statisticsModel);
 
@@ -128,13 +131,12 @@ public class TreesPanel extends JPanel implements Exportable {
         controlPanel1.add(panel3);
 
         final JComboBox rootingFunctionCombo = new JComboBox(TemporalRooting.RootingFunction.values());
-        /*
+
         JPanel panel4 = new JPanel(new BorderLayout(0,0));
         panel4.setOpaque(false);
         panel4.add(new JLabel("Function: "), BorderLayout.WEST);
         panel4.add(rootingFunctionCombo, BorderLayout.CENTER);
         controlPanel1.add(panel4);
-        */
 
         JPanel panel1 = new JPanel(new BorderLayout(0, 0));
 
@@ -184,13 +186,13 @@ public class TreesPanel extends JPanel implements Exportable {
         if (SHOW_NODE_DENSITY) {
             nodeDensityChart = new JChart(new LinearAxis(), new LinearAxis(Axis.AT_ZERO, Axis.AT_MINOR_TICK));
             nodeDensityPanel = new JChartPanel(nodeDensityChart, "", "time", "node density");
-            JPanel panel4 = new JPanel(new BorderLayout());
-            panel4.add(nodeDensityPanel, BorderLayout.CENTER);
-            panel4.setOpaque(false);
+            JPanel panel5 = new JPanel(new BorderLayout());
+            panel5.add(nodeDensityPanel, BorderLayout.CENTER);
+            panel5.setOpaque(false);
 
             ChartSelector selector3 = new ChartSelector(nodeDensityChart);
 
-            tabbedPane.add("Node density", panel4);
+            tabbedPane.add("Node density", panel5);
         }
 
 
@@ -228,23 +230,34 @@ public class TreesPanel extends JPanel implements Exportable {
         setTree(tree);
     }
 
+    public List<String> getSelectedTips() {
+        List<String> tips = new ArrayList<String>();
+        jebl.evolution.trees.Tree tree = treePanel.getTreeViewer().getTrees().get(0);
+
+        for (Node node : treePanel.getTreeViewer().getSelectedTips()) {
+            tips.add(tree.getTaxon(node).getName());
+        }
+        return tips;
+    }
 
     private void treeSelectionChanged() {
-        if (rootToTipPlot != null) {
-            Set<Node> selectedTips = treePanel.getTreeViewer().getSelectedTips();
-            selectedPoints = new HashSet<Integer>();
-            for (Node node : selectedTips) {
-                selectedPoints.add(pointMap.get(node));
-            }
-            rootToTipPlot.setSelectedPoints(selectedPoints);
-            residualPlot.setSelectedPoints(selectedPoints);
-
-            if (SHOW_NODE_DENSITY) {
-                nodeDensityPlot.setSelectedPoints(selectedPoints);
-            }
-
-            selectMRCA();
+        Set<Node> selectedTips = treePanel.getTreeViewer().getSelectedTips();
+        frame.getCopyAction().setEnabled(selectedTips != null && selectedTips.size() > 0);
+        selectedPoints = new HashSet<Integer>();
+        for (Node node : selectedTips) {
+            selectedPoints.add(pointMap.get(node));
         }
+        if (rootToTipPlot != null) {
+            rootToTipPlot.setSelectedPoints(selectedPoints);
+        }
+        if (residualPlot != null) {
+            residualPlot.setSelectedPoints(selectedPoints);
+        }
+        if (SHOW_NODE_DENSITY && nodeDensityPlot != null) {
+            nodeDensityPlot.setSelectedPoints(selectedPoints);
+        }
+
+        selectMRCA();
     }
 
     private void plotSelectionChanged(final Set<Integer> selectedPoints) {
@@ -397,6 +410,11 @@ public class TreesPanel extends JPanel implements Exportable {
                 rootToTipPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44, 44, 44), new Color(249, 202, 105));
                 rootToTipPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44, 44, 44), UIManager.getColor("List.selectionBackground"));
                 rootToTipPlot.addListener(new Plot.Adaptor() {
+                    @Override
+                    public void markClicked(int index, double x, double y, boolean isShiftDown) {
+                        rootToTipPlot.selectPoint(index, isShiftDown);
+                    }
+
                     public void selectionChanged(final Set<Integer> selectedPoints) {
                         plotSelectionChanged(selectedPoints);
                     }
@@ -450,6 +468,22 @@ public class TreesPanel extends JPanel implements Exportable {
                     rootToTipChart.addPlot(mrcaPlot);
                 }
 
+                if (true) {
+                    double[] datePrecisions = temporalRooting.getTipDatePrecisions(currentTree);
+
+                    Variate.D ed = new Variate.D();
+
+                    for (int i = 0; i < datePrecisions.length; i++) {
+                        ed.add(datePrecisions[i]);
+                    }
+
+                    errorBarPlot = new ErrorBarPlot(ErrorBarPlot.Orientation.HORIZONTAL, r.getXData(), r.getYData(), ed);
+                    errorBarPlot.setLineColor(new Color(44, 44, 44));
+                    errorBarPlot.setLineStroke(new BasicStroke(1.0F));
+
+                    rootToTipChart.addPlot(errorBarPlot);
+                }
+
                 rootToTipPlot = new ScatterPlot(r.getXData(), r.getYData());
                 rootToTipPlot.addListener(new Plot.Adaptor() {
                     public void selectionChanged(final Set<Integer> selectedPoints) {
@@ -482,6 +516,12 @@ public class TreesPanel extends JPanel implements Exportable {
                 Variate.D yOffsetValues = new Variate.D(dummyValues);
                 residualPlot = new ScatterPlot(values, yOffsetValues);
                 residualPlot.addListener(new Plot.Adaptor() {
+                    @Override
+                    public void markClicked(int index, double x, double y, boolean isShiftDown) {
+                        rootToTipPlot.selectPoint(index, isShiftDown);
+                    }
+
+                    @Override
                     public void selectionChanged(final Set<Integer> selectedPoints) {
                         plotSelectionChanged(selectedPoints);
                     }
