@@ -1,8 +1,8 @@
 package dr.app.tools;
 
 
+import dr.app.beagle.evomodel.branchmodel.HomogeneousBranchModel;
 import dr.app.beagle.evomodel.parsers.GammaSiteModelParser;
-import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
 import dr.app.beast.BeastVersion;
 import dr.app.util.Arguments;
 import dr.app.util.Utils;
@@ -13,10 +13,7 @@ import dr.evolution.alignment.SimpleAlignment;
 //import dr.evolution.datatype.AminoAcids;
 //import dr.evolution.datatype.GeneralDataType;
 import dr.evolution.datatype.*;
-import dr.evolution.io.Importer;
-import dr.evolution.io.NexusImporter;
-import dr.evolution.io.TreeExporter;
-import dr.evolution.io.TreeImporter;
+import dr.evolution.io.*;
 import dr.evolution.sequence.Sequence;
 import dr.evolution.tree.*;
 import dr.evolution.util.Taxon;
@@ -27,9 +24,9 @@ import dr.evomodel.branchratemodel.StrictClockBranchRates;
 //import dr.evomodel.sitemodel.SiteModel;
 //import dr.evomodel.substmodel.*;
 import dr.evomodel.substmodel.JTT;
+import dr.evomodel.substmodel.LG;
 import dr.evomodel.substmodel.WAG;
 import dr.evomodel.tree.TreeModel;
-import dr.evomodel.treelikelihood.AncestralStateTreeLikelihood;
 import dr.evomodelxml.substmodel.GeneralSubstitutionModelParser;
 import dr.inference.model.Parameter;
 import dr.stats.DiscreteStatistics;
@@ -77,12 +74,12 @@ public class AncestralSequenceAnnotator {
                                       String targetTreeFileName,
                                       String inputFileName,
                                       String outputFileName,
-                                      String clustalExecutable
+                                      String kalignExecutable
     ) throws IOException {
 
         this.posteriorLimit = posteriorLimit;
 
-        this.clustalExecutable = clustalExecutable;
+        this.kalignExecutable = kalignExecutable;
 
         attributeNames.add("height");
         attributeNames.add("length");
@@ -188,7 +185,6 @@ public class AncestralSequenceAnnotator {
 
         System.out.println("Processing ended.");
 
-//		System.exit(-1);
         System.out.println("Writing annotated tree....");
         if (outputFileName != null) {
             exporter = new NexusExporter(new PrintStream(new FileOutputStream(outputFileName)));
@@ -206,9 +202,8 @@ public class AncestralSequenceAnnotator {
     private abstract class SubstitutionModelLoader {
 
 
-        public final char[] AA_ORDER = AminoAcids.AMINOACID_CHARS; //"ACDEFGHIKLMNPQRSTVWY";
-        //public static final String AA_ORDER = "ACDEFGHIKLMNPQRSTVWY";
-        public final char[] NUCLEOTIDE_ORDER = Nucleotides.NUCLEOTIDE_CHARS; //"ACGU";
+        public final char[] AA_ORDER = AminoAcids.AMINOACID_CHARS;
+        public final char[] NUCLEOTIDE_ORDER = Nucleotides.NUCLEOTIDE_CHARS;
         public final String[] CODON_ORDER = Codons.CODON_TRIPLETS;
 
         protected SubstitutionModel substModel;
@@ -262,7 +257,6 @@ public class AncestralSequenceAnnotator {
             substModelName = modelType.replaceFirst("\\*.+","").replaceFirst("\\+.+","").trim();
 
             loadFrequencyModel(tree);
-            //System.out.println("Frequency model datatype: " + getDataType());  //dd
 
             modelSpecifics(tree, modelType);
             printLogLikelihood(tree);
@@ -532,22 +526,18 @@ public class AncestralSequenceAnnotator {
                 substModel = new EmpiricalAminoAcidModel(WAG.INSTANCE, freqModel);
             }
             if(substModelName.equals(LG_TEXT)) {
-                //todo Implement Le Gascuel (2008) model
-                System.err.println("Sorry, LG substitution model is not yet implemented in BEAST");
-                System.exit(0);
-                //substModel = new EmpiricalAminoAcidModel(JTT.INSTANCE, freqModel);
-
+                substModel = new EmpiricalAminoAcidModel(LG.INSTANCE, freqModel);
             }
             //todo Allow proper file input of Empirical amino-acid models
             if(substModelName.matches(Empirical_TEXT)) {
                 String empiricalModelFileName = substModelName.replaceFirst("Empirical\\(", "").replaceFirst("\\).*","");
-                if(empiricalModelFileName.equals("wag.dat")) {
+                if (empiricalModelFileName.equals("wag.dat")) {
                     substModel = new EmpiricalAminoAcidModel(WAG.INSTANCE, freqModel);
-                }
-                else if(empiricalModelFileName.equals("jtt.dat")) {
+                } else if(empiricalModelFileName.equals("jtt.dat")) {
                     substModel = new EmpiricalAminoAcidModel(JTT.INSTANCE, freqModel);
-                }
-                else {
+                } else if(empiricalModelFileName.equals("lg.dat")) {
+                    substModel = new EmpiricalAminoAcidModel(LG.INSTANCE, freqModel);
+                } else {
                     System.err.println("Sorry, AncestralSequenceAnnotator does not currently support other files");
                     System.err.println("Soon, we will allow users to enter a file");
                     System.exit(0);
@@ -900,7 +890,7 @@ public class AncestralSequenceAnnotator {
         AncestralStateBeagleTreeLikelihood likelihood = new AncestralStateBeagleTreeLikelihood(
                 alignment,
                 treeModel,
-                new HomogenousBranchSubstitutionModel(siteModel.getSubstitutionModel(), siteModel.getSubstitutionModel().getFrequencyModel()),
+                new HomogeneousBranchModel(siteModel.getSubstitutionModel()),
                 siteModel,
                 rateModel,
                 null,
@@ -909,7 +899,6 @@ public class AncestralSequenceAnnotator {
                 null,
                 alignment.getDataType(),
                 TAG,
-                siteModel.getSubstitutionModel(),
                 false, true
                 );
 
@@ -1179,10 +1168,6 @@ public class AncestralSequenceAnnotator {
                         value = tree.getNodeAttribute(node, attributeName);
                     }
 
-                    //if (value == null) {
-                    //    System.out.println("attribute " + attributeNames[i] + " is null.");
-                    //}
-
                     if (value != null) {
                         clade.attributeLists[i].add(value);
                     }
@@ -1311,12 +1296,9 @@ public class AncestralSequenceAnnotator {
 
             for (int i = 0; i < attributeNames.size(); i++) {
                 String attributeName = attributeNames.get(i);
-//				System.err.println(attributeName);
 
                 double[] values = new double[clade.attributeLists[i].size()];
                 HashMap<String, Integer> hashMap = new HashMap<String, Integer>(clade.attributeLists[i].size());
-//				Hashtable hashMap = new Hashtable(clade.attributeLists[i].size());
-
 
                 if (values.length > 0) {
                     Object v = clade.attributeLists[i].get(0);
@@ -1345,10 +1327,6 @@ public class AncestralSequenceAnnotator {
                                 hashMap.put(value, 1);
 
                             }
-//							if( tree.getNodeTaxon(node) != null &&
-//									tree.getNodeTaxon(node).getId().compareTo("Calanus") == 0) {
-//									System.err.println(value);
-//								}
                         } else if (isBoolean) {
                             values[j] = (((Boolean) clade.attributeLists[i].get(j)) ? 1.0 : 0.0);
                         } else {
@@ -1412,10 +1390,6 @@ public class AncestralSequenceAnnotator {
 
         private void annotateModeAttribute(MutableTree tree, NodeRef node, String label, HashMap<String, Integer> values) {
             if (label.equals(NEW_SEQ)) {
-//				if( tree.isExternal(node) ) {
-//
-//					return;
-//				}
                 String consensusSeq = null;
                 double[] support = null;
                 int numElements = values.keySet().size();
@@ -1433,19 +1407,34 @@ public class AncestralSequenceAnnotator {
                         for (String key : values.keySet()) {
                             int thisCount = values.get(key);
                             weight[i] = thisCount;
-                            pw.write(">" + i + " " + thisCount + "\n");
+                            //TODO I THINK I FIXED IT?!?!
+                            pw.write(">" + i+"\n");// + " " + thisCount + "\n");
                             pw.write(key + "\n");
                             i++;
                         }
                         pw.close();
 
-                        Process p = Runtime.getRuntime().exec(clustalExecutable + " " + fileName + " -OUTPUT=NEXUS");
+                        //Process p = Runtime.getRuntime().exec(kalignExecutable + " " + fileName + " -OUTPUT=NEXUS");
+                        //Process p = Runtime.getRuntime().exec(kalignExecutable + " " + fileName + " -fmsf -o"+fileName + ".fasta");
+//                        System.out.println("Command: " + kalignExecutable + " " + fileName + " -ffasta -o"+fileName + ".fasta");
+//                        Process p = Runtime.getRuntime().exec(kalignExecutable + " " + fileName + " -ffasta -o"+fileName + ".fasta");
+                        Process p = Runtime.getRuntime().exec(kalignExecutable + " " + fileName + " -ffasta -q -o"+fileName + ".fasta");
+
+//                        ByteArrayOutputStream kalignOutput = new ByteArrayOutputStream();
+//                        StreamGobbler errorGobbler = new StreamGobbler(p.getErrorStream(), "ERR");
+//                        StreamGobbler outputGobbler = new StreamGobbler(p.getInputStream(), "OUT", kalignOutput);
+//
+//                        errorGobbler.start();
+//                        outputGobbler.start();
+//
+//                        int exitVal = p.waitFor();
+
                         BufferedReader input =
                                 new BufferedReader
                                         (new InputStreamReader(p.getInputStream()));
                         {
-                            //String line;
 
+//                            String line;
                             while ((/*line = */input.readLine()) != null) {
 //							System.out.println(line);
                             }
@@ -1455,7 +1444,10 @@ public class AncestralSequenceAnnotator {
                         externalCalls++;
 //						System.err.println("clustal call #" + externalCalls);
 
-                        NexusImporter importer = new NexusImporter(new FileReader(fileName + ".nxs"));
+                        //NexusImporter importer = new NexusImporter(new FileReader(fileName + ".nxs"));
+//                        FastaImporter importer = new FastaImporter(new FileReader(new File(fileName + ".fasta")), Nucleotides.INSTANCE);
+                        FastaImporter importer = new FastaImporter(new FileReader(new File(fileName + ".fasta")), new GeneralDataType(new String[0]));//AminoAcids.INSTANCE);
+                                //new FastaImporter(new FileReader(), new AminoAcids.INSTANCE);
 
                         Alignment alignment = importer.importAlignment();
                         // build index
@@ -1677,7 +1669,7 @@ public class AncestralSequenceAnnotator {
     int totalTrees = 0;
     int totalTreesUsed = 0;
     double posteriorLimit = 0.0;
-    String clustalExecutable = null;
+    String kalignExecutable = null;
 
     List<String> attributeNames = new ArrayList<String>();
     List<String> treeAttributeNames = new ArrayList<String>();
@@ -1742,7 +1734,7 @@ public class AncestralSequenceAnnotator {
                         new Arguments.RealOption("limit", "the minimum posterior probability for a node to be annotated"),
                         new Arguments.StringOption("target", "target_file_name", "specifies a user target tree to be annotated"),
                         new Arguments.Option("help", "option to print this message"),
-                        new Arguments.StringOption("clustal", "full_path_to_clustal", "specifies full path to the clustal executable file")
+                        new Arguments.StringOption("kalign", "full_path_to_kalign", "specifies full path to the kalign executable file")
                 });
 
         try {
@@ -1788,9 +1780,10 @@ public class AncestralSequenceAnnotator {
             targetTreeFileName = arguments.getStringOption("target");
         }
 
-        String clustalExecutable = "/usr/local/bin/clustalw";
-        if (arguments.hasOption("clustal")) {
-            clustalExecutable = arguments.getStringOption("clustal");
+        // String kalignExecutable = "/usr/local/bin/clustalw";
+        String kalignExecutable = "/usr/local/bin/kalign";
+        if (arguments.hasOption("kalign")) {
+            kalignExecutable = arguments.getStringOption("kalign");
         }
 
         String[] args2 = arguments.getLeftoverArguments();
@@ -1831,7 +1824,7 @@ public class AncestralSequenceAnnotator {
                 targetTreeFileName,
                 inputFileName,
                 outputFileName,
-                clustalExecutable);
+                kalignExecutable);
 
         System.exit(0);
     }
