@@ -61,7 +61,7 @@ public class LogCombiner {
                        boolean convertToDecimal,
                        boolean renumberOutput, boolean useScale, double scale) throws IOException {
 
-        System.out.println("Creating combined " + (treeFiles ? "tree" : "log") + " file: '" + outputFileName);
+        System.out.println("Creating combined " + (treeFiles ? "tree" : "log") + " file: '" + outputFileName + "'");
 
         System.out.println();
 
@@ -210,7 +210,7 @@ public class LogCombiner {
                 //lineCount++;
 
                 while (line != null) {
-                    String[] parts = line.split("\\s");
+                    String[] parts = line.split("\t");
 
                     long state = -1;
 
@@ -231,8 +231,13 @@ public class LogCombiner {
                         if (state >= burnin && parts.length == columnCount) {
                             for (int j = 1; j < parts.length; j++) {
                                 try {
-                                    // attempt to convert the column value...
-                                    double value = Double.valueOf(parts[j]);
+                                    if (!parts[j].startsWith("{")) {
+                                        // complex log values start with a curly bracket - otherwise attempt to parse
+                                        // it as a number. If it fails, skip the line as a possible corruption.
+
+                                        // attempt to convert the column value...
+                                        double value = Double.valueOf(parts[j]);
+                                    }
                                 } catch (NumberFormatException nfe) {
                                     skip = true;
                                     break;
@@ -269,6 +274,12 @@ public class LogCombiner {
                                 if (!renumberOutput){
                                     stateLineEntry = stateCount;
                                 } else {
+                                	
+//                                	System.out.println("stateCount: " + stateCount);
+//                                	System.out.println("resample: " + resample);
+//                                	System.out.println("stateStep: " + stateStep);
+//                                	System.out.println("resample / stateStep: " + (resample / stateStep));
+                                	
                                     stateLineEntry = stateCount / (resample / stateStep);
                                 }
 
