@@ -1,7 +1,7 @@
 /*
  * TreeAnnotator.java
  *
- * Copyright (C) 2002-2010 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -334,6 +334,7 @@ public class TreeAnnotator {
         if (stepSize < 1) stepSize = 1;
 
         int counter = 0;
+        int bestTreeNumber = 0;
         TreeImporter importer = new NexusImporter(new FileReader(inputFileName));
         try {
             while (importer.hasTree()) {
@@ -345,6 +346,7 @@ public class TreeAnnotator {
                     if (score > bestScore) {
                         bestTree = tree;
                         bestScore = score;
+                        bestTreeNumber = counter + 1;
                     }
                 }
                 if (counter > 0 && counter % stepSize == 0) {
@@ -359,6 +361,7 @@ public class TreeAnnotator {
         }
         progressStream.println();
         progressStream.println();
+        progressStream.println("Best tree: " + bestTree.getId() + " (tree number " + bestTreeNumber + ")");
         if (useSumCladeCredibility) {
             progressStream.println("Highest Sum Clade Credibility: " + bestScore);
         } else {
@@ -936,7 +939,7 @@ public class TreeAnnotator {
 //		}
 
         private String formattedLocation(double x) {
-            return String.format("%5.2f", x);
+            return String.format("%5.8f", x);
         }
 
         private void annotate2DHPDAttribute(MutableTree tree, NodeRef node, String preLabel, String postLabel,
@@ -1366,6 +1369,8 @@ public class TreeAnnotator {
                 heights = HeightsSummary.MEDIAN_HEIGHTS;
             } else if (value.equalsIgnoreCase("ca")) {
                 heights = HeightsSummary.CA_HEIGHTS;
+                System.out.println("Please cite: Heled and Bouckaert: Looking for trees in the forest:\n" +
+                        "summary tree from posterior samples. BMC Evolutionary Biology 2013 13:221.");
             }
         }
 
