@@ -1,3 +1,28 @@
+/*
+ * AnalysisMenuFactory.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.tracer.application;
 
 import jam.framework.AbstractFrame;
@@ -16,6 +41,7 @@ public class AnalysisMenuFactory implements MenuFactory {
     public static final String BAYESIAN_SKYLINE_RECONSTRUCTION = "Bayesian Skyline Reconstruction...";
     public static final String EXTENDED_BAYESIAN_SKYLINE_RECONSTRUCTION = "Extended Bayesian Skyline Reconstruction...";
     public static final String GMRF_SKYRIDE_RECONSTRUCTION = "GMRF Skyride Reconstruction...";
+    public static final String SKY_GRID_RECONSTRUCTION = "SkyGrid Reconstruction...";
     public static final String LINEAGES_THROUGH_TIME = "Lineages Through Time...";
     public static final String TRAIT_THROUGH_TIME = "Trait Through Time...";
 
@@ -43,12 +69,15 @@ public class AnalysisMenuFactory implements MenuFactory {
             item = new JMenuItem(((AnalysisMenuHandler) frame).getBayesianSkylineAction());
             menu.add(item);
 
-            item = new JMenuItem(((AnalysisMenuHandler) frame).getExtendedBayesianSkylineAction());
-            menu.add(item);
+//            item = new JMenuItem(((AnalysisMenuHandler) frame).getExtendedBayesianSkylineAction());
+//            menu.add(item);
 
             item = new JMenuItem(((AnalysisMenuHandler) frame).getGMRFSkyrideAction());
             menu.add(item);
 
+            item = new JMenuItem(((AnalysisMenuHandler) frame).getSkyGridAction());
+            menu.add(item);
+
             item = new JMenuItem(((AnalysisMenuHandler) frame).getLineagesThroughTimeAction());
             menu.add(item);
 //
@@ -66,8 +95,8 @@ public class AnalysisMenuFactory implements MenuFactory {
             item = new JMenuItem(((AnalysisMenuHandler) frame).getAddBayesianSkylineAction());
             menu.add(item);
 
-            item = new JMenuItem(((AnalysisMenuHandler) frame).getAddExtendedBayesianSkylineAction());
-            menu.add(item);
+//            item = new JMenuItem(((AnalysisMenuHandler) frame).getAddExtendedBayesianSkylineAction());
+//            menu.add(item);
 
             item = new JMenuItem(((AnalysisMenuHandler) frame).getAddTimeDensityAction());
             menu.add(item);
@@ -91,9 +120,9 @@ public class AnalysisMenuFactory implements MenuFactory {
             item.setEnabled(false);
             menu.add(item);
 
-            item = new JMenuItem(EXTENDED_BAYESIAN_SKYLINE_RECONSTRUCTION);
-            item.setEnabled(false);
-            menu.add(item);
+//            item = new JMenuItem(EXTENDED_BAYESIAN_SKYLINE_RECONSTRUCTION);
+//            item.setEnabled(false);
+//            menu.add(item);
 
             item = new JMenuItem(GMRF_SKYRIDE_RECONSTRUCTION);
             item.setEnabled(false);
@@ -121,9 +150,9 @@ public class AnalysisMenuFactory implements MenuFactory {
             item.setEnabled(false);
             menu.add(item);
 
-            item = new JMenuItem(ADD_EXTENDED_BAYESIAN_SKYLINE_RECONSTRUCTION);
-            item.setEnabled(false);
-            menu.add(item);
+//            item = new JMenuItem(ADD_EXTENDED_BAYESIAN_SKYLINE_RECONSTRUCTION);
+//            item.setEnabled(false);
+//            menu.add(item);
 
             item = new JMenuItem(ADD_TIME_DENSITY);
             item.setEnabled(false);
