@@ -1,3 +1,28 @@
+/*
+ * TracerApp.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.tracer.application;
 
 import dr.app.util.OSType;
@@ -18,33 +43,38 @@ public class TracerApp extends MultiDocApplication {
         addPreferencesSection(new GeneralPreferencesSection());
     }
 
+    private static boolean lafLoaded = false;
+
     // Main entry point
     static public void main(String[] args) {
         // There is a major issue with languages that use the comma as a decimal separator.
         // To ensure compatibility between programs in the package, enforce the US locale.
         Locale.setDefault(Locale.US);
 
-        boolean lafLoaded = false;
-
         if (OSType.isMac()) {
             System.setProperty("apple.awt.graphics.UseQuartz", "true");
-            System.setProperty("apple.awt.antialiasing","true");
-            System.setProperty("apple.awt.rendering","VALUE_RENDER_QUALITY");
+            System.setProperty("apple.awt.antialiasing", "true");
+            System.setProperty("apple.awt.rendering", "VALUE_RENDER_QUALITY");
 
-            System.setProperty("apple.laf.useScreenMenuBar","true");
-            System.setProperty("apple.awt.draggableWindowBackground","true");
-            System.setProperty("apple.awt.showGrowBox","true");
+            System.setProperty("apple.laf.useScreenMenuBar", "true");
+            System.setProperty("apple.awt.draggableWindowBackground", "true");
+            System.setProperty("apple.awt.showGrowBox", "true");
 
-            // set the Quaqua Look and Feel in the UIManager
             try {
-                UIManager.setLookAndFeel(
-                        "ch.randelshofer.quaqua.QuaquaLookAndFeel"
-                );
-                lafLoaded = true;
-
+                // set the Quaqua Look and Feel in the UIManager
+                javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
+                    public void run() {
+                        try {
+                            UIManager.setLookAndFeel(
+                                    "ch.randelshofer.quaqua.QuaquaLookAndFeel"
+                            );
 
+                            lafLoaded = true;
+                        } catch (Exception e) {
+                        }
+                    }
+                });
             } catch (Exception e) {
-                //
             }
 
             UIManager.put("SystemFont", new Font("Lucida Grande", Font.PLAIN, 13));
@@ -52,10 +82,19 @@ public class TracerApp extends MultiDocApplication {
         }
 
         if (!lafLoaded) {
+
             try {
-                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
-            } catch (Exception e1) {
-                e1.printStackTrace();
+                javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
+                    public void run() {
+                        try {
+                            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
+                        } catch (Exception e) {
+                            e.printStackTrace();
+                        }
+                    }
+                });
+            } catch (Exception e) {
+                e.printStackTrace();
             }
         }
 
@@ -70,7 +109,7 @@ public class TracerApp extends MultiDocApplication {
             final String nameString = "Tracer";
             final String versionString = "v1.6.0pre";
             String aboutString = "<html><center><p>MCMC Trace Analysis Tool<br>" +
-                    "Version " + versionString + ", 2003-2011</p>" +
+                    "Version " + versionString + ", 2003-2013</p>" +
                     "<p>by<br>" +
 
                     "Andrew Rambaut, Marc A. Suchard, Walter Xie and Alexei J. Drummond</p>" +
@@ -109,7 +148,7 @@ public class TracerApp extends MultiDocApplication {
                 for (String fileName : args) {
 
                     File file = new File(fileName);
-                    LogFileTraces[] traces = { new LogFileTraces(fileName, file) };
+                    LogFileTraces[] traces = {new LogFileTraces(fileName, file)};
 
                     frame.processTraces(traces);
                 }
