@@ -85,7 +85,7 @@ public class CoalescentSimulator {
         // sanity check - disjoint trees
 
         if( ! Tree.Utils.allDisjoint(nodes) ) {
-            throw new RuntimeException("non disjoint trees");
+            throw new RuntimeException("subtrees' taxa overlap");
         }
 
         if( nodes.length == 0 ) {
@@ -99,10 +99,10 @@ public class CoalescentSimulator {
             }
         }
 
-        throw new RuntimeException("failed to merge trees after 1000 tries!");
+        throw new RuntimeException("failed to merge trees after 1000 tries.");
 	}
 
-	public SimpleNode[] simulateCoalescent(SimpleNode[] nodes, DemographicFunction demographic, double currentHeight, double maxHeight) {
+    public SimpleNode[] simulateCoalescent(SimpleNode[] nodes, DemographicFunction demographic, double currentHeight, double maxHeight) {
         // If only one node, return it
         // continuing results in an infinite loop
         if( nodes.length == 1 ) return nodes;
