@@ -1,7 +1,7 @@
 /*
  * DataType.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -46,6 +46,7 @@ public abstract class DataType implements Serializable {
     public static final int GENERAL = 4;
     public static final int COVARION = 5;
     public static final int MICRO_SAT = 6;
+
     public static final int P2PTYPE = 7;
     public static final int CONTINUOUS = 8;
 
@@ -89,6 +90,7 @@ public abstract class DataType implements Serializable {
             registerDataType(HiddenCodons.DESCRIPTION + "3-" + GeneticCode.UNIVERSAL.getName(), HiddenCodons.UNIVERSAL_HIDDEN_3);
             registerDataType(NewHiddenNucleotides.DESCRIPTION + "1", NewHiddenNucleotides.NUCLEOTIDE_HIDDEN_1);
             registerDataType(NewHiddenNucleotides.DESCRIPTION + "2", NewHiddenNucleotides.NUCLEOTIDE_HIDDEN_2);
+            registerDataType(NewHiddenNucleotides.DESCRIPTION + "3", NewHiddenNucleotides.NUCLEOTIDE_HIDDEN_3);
             registerDataType(GeneralDataType.DESCRIPTION, GeneralDataType.INSTANCE);
             registerDataType(Microsatellite.DESCRIPTION, Microsatellite.INSTANCE);
             registerDataType(P2P.DESCRIPTION, P2P.INSTANCE);
@@ -401,4 +403,46 @@ public abstract class DataType implements Serializable {
         return (state == getGapState());
     }
 
+    public String getName() {
+        switch (getType()) {
+            case DataType.NUCLEOTIDES:
+                return "Nucleotide";
+            case DataType.AMINO_ACIDS:
+                return "Amino Acid";
+            case DataType.CODONS:
+                return "Codon";
+            case DataType.TWO_STATES:
+                return "Binary";
+            case DataType.COVARION:
+                return "Covarion";
+            case DataType.GENERAL:
+                return "Discrete Traits";
+            case DataType.CONTINUOUS:
+                return "Continuous Traits";
+            case DataType.MICRO_SAT:
+                return "Microsatellite";
+            default:
+                throw new IllegalArgumentException("Unsupported data type");
+
+        }
+    }
+
+    @Override
+    public boolean equals(Object o) {
+        if (this == o) return true;
+        if (!(o instanceof DataType)) return false;
+
+        DataType dataType = (DataType) o;
+
+        if (this.getType() != dataType.getType()) return false;
+
+        return true;
+    }
+
+    @Override
+    public int hashCode() {
+        return getType();
+    }
+
+
 }
