@@ -159,7 +159,7 @@ public class MultidimensionalScalingLikelihood extends AbstractModelLikelihood {
         this.mdsPrecisionParameter = mdsPrecision;
         addVariable(mdsPrecision);
 
-        this.isLeftTruncated = true; // Re-normalize likelihood for strictly positive distances
+        this.isLeftTruncated = false; // Re-normalize likelihood for strictly positive distances
 
         // make sure everything is calculated on first evaluation
         makeDirty();
