@@ -43,6 +43,12 @@ import dr.inference.model.Variable;
  * @version $Id: DiscretizedBranchRates.java,v 1.11 2006/01/09 17:44:30 rambaut Exp $
  */
 public class DiscretizedBranchRates extends AbstractBranchRateModel {
+    // Turn on an off the caching on rates for categories -
+    // if off then the rates will be flagged to update on
+    // a restore.
+    // Currently turned off as it is not working with multiple partitions for
+    // some reason.
+    private static final boolean CACHE_RATES = false;
 
     private final ParametricDistributionModel distributionModel;
 
@@ -51,13 +57,20 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
 
     private final int categoryCount;
     private final double step;
-    private final double[] rates;
-    private boolean normalize = false;
-    private double normalizeBranchRateTo = Double.NaN;
-    private double scaleFactor = 1.0;
-    private TreeModel treeModel;
+    private final double[][] rates;
+    private final boolean normalize;
+    private final double normalizeBranchRateTo;
+
+    private final TreeModel treeModel;
     private final double logDensityNormalizationConstant;
 
+    private double scaleFactor = 1.0;
+    private double storedScaleFactor;
+
+    private boolean updateRateCategories = true;
+    private int currentRateArrayIndex = 0;
+    private int storedRateArrayIndex;
+
     //overSampling control the number of effective categories
 
     public DiscretizedBranchRates(
@@ -84,7 +97,7 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
         categoryCount = (tree.getNodeCount() - 1) * overSampling;
         step = 1.0 / (double) categoryCount;
 
-        rates = new double[categoryCount];
+        rates = new double[2][categoryCount];
 
         this.normalize = normalize;
 
@@ -102,26 +115,9 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
         }
 
         addModel(model);
-        // AR - commented out: changes to the tree are handled by model changed events fired by rateCategories
-//        addModel(tree);
         addModel(rateCategories);
-        // AR - commented out: changes to rateCategoryParameter are handled by model changed events fired by rateCategories
-//        addVariable(rateCategoryParameter);
 
-        if (normalize) {
-            tree.addModelListener(new ModelListener() {
-
-                public void modelChangedEvent(Model model, Object object, int index) {
-                    computeFactor();
-                }
-
-                public void modelRestored(Model model) {
-                    computeFactor();
-                }
-            });
-        }
-
-        setupRates();
+        updateRateCategories = true;
 
         // Each parameter take any value in [1, \ldots, categoryCount]
         // NB But this depends on the transition kernel employed.  Using swap-only results in a different constant
@@ -132,7 +128,6 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
 
     private void computeFactor() {
 
-
         //scale mean rate to 1.0 or separate parameter
 
         double treeRate = 0.0;
@@ -143,7 +138,7 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
             NodeRef node = treeModel.getNode(i);
             if (!treeModel.isRoot(node)) {
                 int rateCategory = (int) Math.round(rateCategories.getNodeValue(treeModel, node));
-                treeRate += rates[rateCategory] * treeModel.getBranchLength(node);
+                treeRate += rates[currentRateArrayIndex][rateCategory] * treeModel.getBranchLength(node);
                 treeTime += treeModel.getBranchLength(node);
 
                 //System.out.println("rates and time\t" + rates[rateCategory] + "\t" + treeModel.getBranchLength(node));
@@ -157,57 +152,70 @@ public class DiscretizedBranchRates extends AbstractBranchRateModel {
 
     public void handleModelChangedEvent(Model model, Object object, int index) {
         if (model == distributionModel) {
-            setupRates();
+            updateRateCategories = true;
             fireModelChanged();
         } else if (model == rateCategories) {
-            // AR - commented out: if just the rate categories have changed the rates will be the same
-//            setupRates();
             fireModelChanged(null, index);
         }
     }
 
     protected final void handleVariableChangedEvent(Variable variable, int index, Parameter.ChangeType type) {
-        // AR - commented out: changes to rateCategoryParameter are handled by model changed events
-//        setupRates();
-    }
+        // nothing to do here
+   }
 
     protected void storeState() {
+        if (CACHE_RATES) {
+            storedRateArrayIndex = currentRateArrayIndex;
+            storedScaleFactor = scaleFactor;
+        }
     }
 
     protected void restoreState() {
-        setupRates();
+        if (CACHE_RATES) {
+            currentRateArrayIndex = storedRateArrayIndex;
+            scaleFactor = storedScaleFactor;
+        } else {
+            updateRateCategories = true;
+        }
     }
 
     protected void acceptState() {
     }
 
-    public double getBranchRate(final Tree tree, final NodeRef node) {
+    public final double getBranchRate(final Tree tree, final NodeRef node) {
 
         assert !tree.isRoot(node) : "root node doesn't have a rate!";
 
+        if (updateRateCategories) {
+            setupRates();
+        }
+
         int rateCategory = (int) Math.round(rateCategories.getNodeValue(tree, node));
 
         //System.out.println(rates[rateCategory] + "\t"  + rateCategory);
-        return rates[rateCategory] * scaleFactor;
+        return rates[currentRateArrayIndex][rateCategory] * scaleFactor;
     }
 
     /**
      * Calculates the actual rates corresponding to the category indices.
      */
-    protected void setupRates() {
+    private void setupRates() {
+
+        if (CACHE_RATES) {
+            // flip the current array index
+            currentRateArrayIndex = 1 - currentRateArrayIndex;
+        }
 
         double z = step / 2.0;
         for (int i = 0; i < categoryCount; i++) {
-            rates[i] = distributionModel.quantile(z);
+            rates[currentRateArrayIndex][i] = distributionModel.quantile(z);
             //System.out.print(rates[i]+"\t");
             z += step;
         }
-        /*if(distributionModel.getClass().getName().equals("dr.inference.distribution.LogNormalDistributionModel")) {
-            LogNormalDistributionModel lndm = (LogNormalDistributionModel) distributionModel;
-            System.out.println("chur " + lndm.getS() +"\t" + lndm.getM());
-        }
-        else { System.out.println(distributionModel.getClass().getName());}*/
+
         if (normalize) computeFactor();
+
+        updateRateCategories = false;
     }
 
     public double getLogLikelihood() {
