@@ -1,7 +1,7 @@
 /*
- * CoalescentLikelihood.java
+ * AbstractCoalescentLikelihood.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -26,6 +26,7 @@
 package dr.evomodel.coalescent;
 
 import dr.evolution.coalescent.IntervalList;
+import dr.evolution.coalescent.IntervalType;
 import dr.evolution.coalescent.Intervals;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
@@ -46,7 +47,7 @@ import java.util.Set;
  * @author Alexei Drummond
  * @version $Id: CoalescentLikelihood.java,v 1.43 2006/07/28 11:27:32 rambaut Exp $
  */
-public abstract class AbstractCoalescentLikelihood extends AbstractModelLikelihood implements Units {
+public abstract class AbstractCoalescentLikelihood extends AbstractModelLikelihood implements Units, CoalescentIntervalProvider {
 
     // PUBLIC STUFF
 
@@ -152,7 +153,6 @@ public abstract class AbstractCoalescentLikelihood extends AbstractModelLikeliho
     public final double getLogLikelihood() {
         if (!eventsKnown) {
             setupIntervals();
-            likelihoodKnown = false;
         }
 
         if (!likelihoodKnown) {
@@ -194,7 +194,7 @@ public abstract class AbstractCoalescentLikelihood extends AbstractModelLikeliho
         if (excludedLeafSets.length == 0) return null;
 
         Set<NodeRef> excludeNodesBelow = new HashSet<NodeRef>();
-        for( Set<String> excludedLeafSet : excludedLeafSets ) {
+        for (Set<String> excludedLeafSet : excludedLeafSets) {
             excludeNodesBelow.add(Tree.Utils.getCommonAncestorNode(tree, excludedLeafSet));
         }
         return excludeNodesBelow;
@@ -219,6 +219,7 @@ public abstract class AbstractCoalescentLikelihood extends AbstractModelLikeliho
         intervals.getIntervalCount();
 
         eventsKnown = true;
+        likelihoodKnown = false;
     }
 
 
@@ -252,6 +253,34 @@ public abstract class AbstractCoalescentLikelihood extends AbstractModelLikeliho
 
     }
 
+    public double getCoalescentInterval(int i) {
+        if (!eventsKnown) {
+            setupIntervals();
+        }
+        return intervals.getInterval(i);
+    }
+
+    public int getCoalescentIntervalDimension() {
+        if (!eventsKnown) {
+            setupIntervals();
+        }
+        return intervals.getIntervalCount();
+    }
+
+    public int getCoalescentIntervalLineageCount(int i) {
+        if (!eventsKnown) {
+            setupIntervals();
+        }
+        return intervals.getLineageCount(i);
+    }
+
+    public IntervalType getCoalescentIntervalType(int i) {
+        if (!eventsKnown) {
+            setupIntervals();
+        }
+        return intervals.getIntervalType(i);
+    }
+
     public String toString() {
         return Double.toString(logLikelihood);
 
