@@ -1,9 +1,31 @@
+/*
+ * AbstractMultivariateTraitLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.continuous;
 
-import dr.evolution.tree.NodeRef;
-import dr.evolution.tree.Tree;
-import dr.evolution.tree.TreeTrait;
-import dr.evolution.tree.TreeTraitProvider;
+import dr.evolution.tree.*;
 import dr.evolution.util.Taxon;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.tree.TreeModel;
@@ -51,9 +73,11 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     public static final String IGNORE_PHYLOGENY = "ignorePhylogeny";
     public static final String ASCERTAINMENT = "ascertainedTaxon";
     public static final String EXCHANGEABLE_TIPS = "exchangeableTips";
+    public static final String DRIFT_MODELS = "driftModels";
+    private BranchRateModel branchRateModel;
 
     public AbstractMultivariateTraitLikelihood(String traitName,
-                                               TreeModel treeModel,
+                                               MultivariateTraitTree treeModel,
                                                MultivariateDiffusionModel diffusionModel,
                                                CompoundParameter traitParameter,
                                                List<Integer> missingIndices,
@@ -65,11 +89,11 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                                                boolean reportAsMultivariate,
                                                boolean reciprocalRates) {
         this(traitName, treeModel, diffusionModel, traitParameter, null, missingIndices, cacheBranches,
-                scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate, reciprocalRates);
+                scaleByTime, useTreeLength, rateModel, null, samplingDensity, reportAsMultivariate, reciprocalRates);
     }
 
     public AbstractMultivariateTraitLikelihood(String traitName,
-                                               TreeModel treeModel,
+                                               MultivariateTraitTree treeModel,
                                                MultivariateDiffusionModel diffusionModel,
                                                CompoundParameter traitParameter,
                                                Parameter deltaParameter,
@@ -78,6 +102,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                                                boolean scaleByTime,
                                                boolean useTreeLength,
                                                BranchRateModel rateModel,
+                                               List<BranchRateModel> driftModels,
                                                Model samplingDensity,
                                                boolean reportAsMultivariate,
                                                boolean reciprocalRates) {
@@ -86,7 +111,8 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
 
         this.traitName = traitName;
         this.treeModel = treeModel;
-        this.rateModel = rateModel;
+        this.branchRateModel = rateModel;
+        this.driftModels = driftModels;
         this.diffusionModel = diffusionModel;
         this.traitParameter = traitParameter;
         this.missingIndices = missingIndices;
@@ -98,13 +124,17 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             addVariable(deltaParameter);
         }
 
-
-
         if (rateModel != null) {
-            hasRateModel = true;
+            hasBranchRateModel = true;
             addModel(rateModel);
         }
 
+        if (driftModels != null) {
+            for (BranchRateModel drift : driftModels) {
+                addModel(drift);
+            }
+        }
+
         if (samplingDensity != null) {
             addModel(samplingDensity);
         }
@@ -133,7 +163,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         if (dim % dimTrait != 0)
             throw new RuntimeException("dim is not divisible by dimTrait");
 
-        recalculateTreeLength();                           
+        recalculateTreeLength();
         printInformtion();
 
     }
@@ -142,7 +172,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         StringBuffer sb = new StringBuffer("Creating multivariate diffusion model:\n");
         sb.append("\tTrait: ").append(traitName).append("\n");
         sb.append("\tDiffusion process: ").append(diffusionModel.getId()).append("\n");
-        sb.append("\tHeterogenity model: ").append(rateModel != null ? rateModel.getId() : "homogeneous").append("\n");
+        sb.append("\tHeterogenity model: ").append(branchRateModel != null ? branchRateModel.getId() : "homogeneous").append("\n");
         sb.append("\tTree normalization: ").append(scaleByTime ? (useTreeLength ? "length" : "height") : "off").append("\n");
         sb.append("\tUsing reciprocal (precision) rates: ").append(reciprocalRates).append("\n");
         if (scaleByTime) {
@@ -159,7 +189,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
 
 
         sb.append("\n\tDiffusion dimension   : ").append(dimTrait).append("\n");
-        sb.append(  "\tNumber of observations: ").append(numData).append("\n");
+        sb.append("\tNumber of observations: ").append(numData).append("\n");
         Logger.getLogger("dr.evomodel").info(sb.toString());
     }
 
@@ -210,15 +240,30 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         Logger.getLogger("dr.evomodel").info(sb.toString());
     }
 
-    public double getRescaledBranchLength(NodeRef node) {
+    public double[] getShiftForBranchLength(NodeRef node) {
+        if (driftModels != null) {
+            final int dim = driftModels.size();
+            double[] drift = new double[dim];
+            double realTimeBranchLength = treeModel.getBranchLength(node);
+            for (int i = 0; i < dim; ++i) {
+                drift[i] = driftModels.get(i).getBranchRate(treeModel, node) * realTimeBranchLength;
+            }
+            return drift;
+        } else {
+            throw new RuntimeException("getShiftForBranchLength should not be called.");
+        }
+        // But really should get values from driftModel.getBranchRate(treeModel, node);
+    }
+
+    public double getRescaledBranchLengthForPrecision(NodeRef node) {
 
         double length = treeModel.getBranchLength(node);
 
-        if (hasRateModel) {
+        if (hasBranchRateModel) {
             if (reciprocalRates) {
-                length /= rateModel.getBranchRate(treeModel, node); // branch rate scales as precision (inv-time)
+                length /= branchRateModel.getBranchRate(treeModel, node); // branch rate scales as precision (inv-time)
             } else {
-                length *= rateModel.getBranchRate(treeModel, node); // branch rate scales as variance (time)
+                length *= branchRateModel.getBranchRate(treeModel, node); // branch rate scales as variance (time)
             }
         }
 
@@ -259,7 +304,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 if (event.isTreeChanged()) {
                     recalculateTreeLength();
                     updateAllNodes();
-                } else  if (event.isHeightChanged()) {
+                } else if (event.isHeightChanged()) {
                     recalculateTreeLength();
                     if (useTreeLength || (scaleByTime && treeModel.isRoot(event.getNode())))
                         updateAllNodes();
@@ -283,7 +328,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             } else {
                 throw new RuntimeException("Unexpected object throwing events in AbstractMultivariateTraitLikelihood");
             }
-        } else if (model == rateModel) {
+        } else if (model == branchRateModel) {
             if (index == -1) {
                 updateAllNodes();
             } else {
@@ -315,6 +360,15 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         likelihoodKnown = false;
     }
 
+    protected double getTreeLength() {
+        double treeLength = 0;
+        for (int i = 0; i < treeModel.getNodeCount(); i++) {
+            NodeRef node = treeModel.getNode(i);
+            if (!treeModel.isRoot(node))
+                treeLength += treeModel.getBranchLength(node); // Bug was here
+        }
+        return treeLength;
+    }
 
     public void recalculateTreeLength() {
 
@@ -322,17 +376,16 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             return;
 
         if (useTreeLength) {
-            treeLength = 0;
-            for (int i = 0; i < treeModel.getNodeCount(); i++) {
-                NodeRef node = treeModel.getNode(i);
-                if (!treeModel.isRoot(node))
-                    treeLength += treeModel.getBranchLength(node); // Bug was here
-            }
+            treeLength = getTreeLength();
         } else { // Normalizing by tree height.
             treeLength = treeModel.getNodeHeight(treeModel.getRoot());
         }
     }
 
+    public BranchRateModel getBranchRateModel() {
+        return branchRateModel;
+    }
+
     // **************************************************************
     // VariableListener IMPLEMENTATION
     // **************************************************************
@@ -390,7 +443,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     protected void acceptState() {
     } // nothing to do
 
-    public TreeModel getTreeModel() {
+    public MultivariateTraitTree getTreeModel() {
         return treeModel;
     }
 
@@ -470,7 +523,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         if (treeTraits == null) {
             final double[] trait = getRootNodeTrait();
             if (trait.length == 1 || reportAsMultivariate) {
-                treeTraits = new TreeTrait[] {
+                treeTraits = new TreeTrait[]{
                         new TreeTrait.DA() {
                             public String getTraitName() {
                                 return traitName;
@@ -545,7 +598,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
             MultivariateDiffusionModel diffusionModel = (MultivariateDiffusionModel) xo.getChild(MultivariateDiffusionModel.class);
-            TreeModel treeModel = (TreeModel) xo.getChild(TreeModel.class);
+            MultivariateTraitTree treeModel = (MultivariateTraitTree) xo.getChild(MultivariateTraitTree.class);
 
             boolean cacheBranches = xo.getAttribute(CACHE_BRANCHES, false);
             boolean integrate = xo.getAttribute(INTEGRATE, false);
@@ -556,6 +609,22 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
 
             BranchRateModel rateModel = (BranchRateModel) xo.getChild(BranchRateModel.class);
 
+            List<BranchRateModel> driftModels = null;
+            if (xo.hasChildNamed(DRIFT_MODELS)) {
+                driftModels = new ArrayList<BranchRateModel>();
+                XMLObject cxo = xo.getChild(DRIFT_MODELS);
+                final int number = cxo.getChildCount();
+                if (number != diffusionModel.getPrecisionmatrix().length) {
+                    throw new XMLParseException("Wrong number of drift models (" + number + ") for a trait of" +
+                            " dimension " + diffusionModel.getPrecisionmatrix().length + " in " + xo.getId()
+                    );
+                }
+                for (int i = 0; i < number; ++i) {
+                    driftModels.add((BranchRateModel) cxo.getChild(i));
+                }
+            }
+
+
             TreeTraitParserUtilities utilities = new TreeTraitParserUtilities();
             String traitName = TreeTraitParserUtilities.DEFAULT_TRAIT_NAME;
 
@@ -623,10 +692,17 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                                 scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
                                 mean, pseudoObservations, reciprocalRates, exchangeableTips);
                     } else {
-                        like = new FullyConjugateMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
-                                traitParameter, deltaParameter, missingIndices, cacheBranches,
-                                scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
-                                mean, pseudoObservations, reciprocalRates);
+                        if (driftModels == null) {
+                            like = new FullyConjugateMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
+                                    traitParameter, deltaParameter, missingIndices, cacheBranches,
+                                    scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
+                                    mean, pseudoObservations, reciprocalRates);
+                        } else {
+                            like = new FullyConjugateMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
+                                    traitParameter, deltaParameter, missingIndices, cacheBranches,
+                                    scaleByTime, useTreeLength, rateModel, driftModels, samplingDensity, reportAsMultivariate,
+                                    mean, pseudoObservations, reciprocalRates);
+                        }
                     }
                 }
             } else {
@@ -656,7 +732,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 Taxon taxon = (Taxon) cxo.getChild(Taxon.class);
                 if (!integrate) {
                     throw new XMLParseException("Ascertainment correction is currently only implemented" +
-                        " for integrated multivariate trait likelihood models");
+                            " for integrated multivariate trait likelihood models");
                 }
                 like.setAscertainedTaxon(taxon);
             }
@@ -695,11 +771,11 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                                 new XMLSyntaxRule[]{new ElementRule(Parameter.class)}),
                 }, true),
 //                        true),
-                new ElementRule(ASCERTAINMENT, new XMLSyntaxRule[] {
+                new ElementRule(ASCERTAINMENT, new XMLSyntaxRule[]{
                         new ElementRule(Taxon.class)
                 }, true),
                 new ElementRule(MultivariateDiffusionModel.class),
-                new ElementRule(TreeModel.class),
+                new ElementRule(MultivariateTraitTree.class),
                 new ElementRule(BranchRateModel.class, true),
                 AttributeRule.newDoubleArrayRule("cut", true),
                 AttributeRule.newBooleanRule(REPORT_MULTIVARIATE, true),
@@ -715,7 +791,10 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 TreeTraitParserUtilities.jitterRules(true),
                 new ElementRule(CHECK, new XMLSyntaxRule[]{
                         new ElementRule(Parameter.class)
-                }, true)
+                }, true),
+                new ElementRule(DRIFT_MODELS, new XMLSyntaxRule[]{
+                        new ElementRule(BranchRateModel.class, 1, Integer.MAX_VALUE),
+                }, true),
         };
 
 
@@ -724,7 +803,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         }
     };
 
-    TreeModel treeModel = null;
+    MultivariateTraitTree treeModel = null;
     MultivariateDiffusionModel diffusionModel = null;
     String traitName = null;
     CompoundParameter traitParameter;
@@ -735,8 +814,10 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     private double storedLogLikelihood;
     protected boolean likelihoodKnown = false;
     private boolean storedLikelihoodKnown = false;
-    private BranchRateModel rateModel = null;
-    private boolean hasRateModel = false;
+    protected List<BranchRateModel> driftModels = null;
+    private BranchRateModel driftOne = null;
+    private BranchRateModel driftTwo = null;
+    private boolean hasBranchRateModel = false;
 
     private double treeLength;
     private double storedTreeLength;
