@@ -1,11 +1,37 @@
+/*
+ * FullyConjugateMultivariateTraitLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.continuous;
 
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.evolution.tree.NodeRef;
 import dr.evomodel.branchratemodel.BranchRateModel;
-import dr.evomodel.tree.TreeModel;
 import dr.inference.model.CompoundParameter;
 import dr.inference.model.Model;
 import dr.inference.model.Parameter;
+import dr.math.distributions.MultivariateNormalDistribution;
 import dr.math.distributions.WishartSufficientStatistics;
 import dr.math.interfaces.ConjugateWishartStatisticsProvider;
 import dr.math.matrixAlgebra.Matrix;
@@ -23,7 +49,7 @@ import java.util.List;
 public class FullyConjugateMultivariateTraitLikelihood extends IntegratedMultivariateTraitLikelihood implements ConjugateWishartStatisticsProvider {
 
     public FullyConjugateMultivariateTraitLikelihood(String traitName,
-                                                     TreeModel treeModel,
+                                                     MultivariateTraitTree treeModel,
                                                      MultivariateDiffusionModel diffusionModel,
                                                      CompoundParameter traitParameter,
                                                      Parameter deltaParameter,
@@ -49,17 +75,43 @@ public class FullyConjugateMultivariateTraitLikelihood extends IntegratedMultiva
     }
 
 
-    protected double getRescaledLengthToRoot(NodeRef nodeRef) {
+    public FullyConjugateMultivariateTraitLikelihood(String traitName,
+                                                     MultivariateTraitTree treeModel,
+                                                     MultivariateDiffusionModel diffusionModel,
+                                                     CompoundParameter traitParameter,
+                                                     Parameter deltaParameter,
+                                                     List<Integer> missingIndices,
+                                                     boolean cacheBranches,
+                                                     boolean scaleByTime,
+                                                     boolean useTreeLength,
+                                                     BranchRateModel rateModel,
+                                                     List<BranchRateModel> driftModels,
+                                                     Model samplingDensity,
+                                                     boolean reportAsMultivariate,
+                                                     double[] rootPriorMean,
+                                                     double rootPriorSampleSize,
+                                                     boolean reciprocalRates) {
+
+        super(traitName, treeModel, diffusionModel, traitParameter, deltaParameter, missingIndices, cacheBranches, scaleByTime,
+                useTreeLength, rateModel, driftModels, samplingDensity, reportAsMultivariate, reciprocalRates);
+
+        // fully-conjugate multivariate normal with own mean and prior sample size
+        this.rootPriorMean = rootPriorMean;
+        this.rootPriorSampleSize = rootPriorSampleSize;
+
+        priorInformationKnown = false;
+    }
+
+
+    public double getRescaledLengthToRoot(NodeRef nodeRef) {
 
         double length = 0;
-        NodeRef parent = treeModel.getParent(nodeRef);
 
-        if (!treeModel.isRoot(parent)) {
-            length += getRescaledLengthToRoot(parent);
+        if (!treeModel.isRoot(nodeRef)) {
+            NodeRef parent = treeModel.getParent(nodeRef);
+            length += getRescaledBranchLengthForPrecision(nodeRef) + getRescaledLengthToRoot(parent);
         }
 
-        length += getRescaledBranchLength(nodeRef);
-
         return length;
     }
 
@@ -160,11 +212,51 @@ public class FullyConjugateMultivariateTraitLikelihood extends IntegratedMultiva
         priorInformationKnown = false;
     }
 
+    public double getPriorSampleSize() {
+        return rootPriorSampleSize;
+    }
+
+    public double[] getPriorMean() {
+        return rootPriorMean;
+    }
+
     @Override
     public boolean getComputeWishartSufficientStatistics() {
         return computeWishartStatistics;
     }
 
+    protected void checkLogLikelihood(double loglikelihood, double logRemainders,
+                                      double[] conditionalRootMean, double conditionalRootPrecision,
+                                      double[][] traitPrecision) {
+
+//        System.err.println("root cmean    : " + new Vector(conditionalRootMean));
+//        System.err.println("root cprec    : " + conditionalRootPrecision);
+//        System.err.println("diffusion prec: " + new Matrix(traitPrecision));
+//
+//        System.err.println("prior mean    : " + new Vector(rootPriorMean));
+//        System.err.println("prior prec    : " + rootPriorSampleSize);
+
+        double upperPrecision = conditionalRootPrecision * rootPriorSampleSize / (conditionalRootPrecision + rootPriorSampleSize);
+//        System.err.println("root cprec    : " + upperPrecision);
+
+        double[][] newPrec = new double[traitPrecision.length][traitPrecision.length];
+
+        for (int i = 0; i < traitPrecision.length; ++i) {
+            for (int j = 0; j < traitPrecision.length; ++j) {
+                newPrec[i][j] = traitPrecision[i][j] * upperPrecision;
+            }
+        }
+        MultivariateNormalDistribution mvn = new MultivariateNormalDistribution(rootPriorMean, newPrec);
+        double logPdf = mvn.logPdf(conditionalRootMean);
+
+        System.err.println("Got here subclass: " + loglikelihood);
+        System.err.println("logValue         : " + (logRemainders + logPdf));
+        System.err.println("");
+//        System.err.println("logRemainders    : " + logRemainders);
+//        System.err.println("logPDF           : " + logPdf);
+//        System.exit(-1);
+    }
+
     protected double integrateLogLikelihoodAtRoot(double[] conditionalRootMean,
                                                   double[] marginalRootMean,
                                                   double[][] notUsed,
