@@ -1,9 +1,34 @@
+/*
+ * IntegratedMultivariateTraitLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.continuous;
 
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.branchratemodel.BranchRateModel;
-import dr.evomodel.tree.TreeModel;
 import dr.inference.loggers.LogColumn;
 import dr.inference.model.CompoundParameter;
 import dr.inference.model.Model;
@@ -19,7 +44,6 @@ import dr.util.Citation;
 
 import java.util.Arrays;
 import java.util.List;
-import java.util.logging.Logger;
 
 /**
  * A multivariate trait likelihood that analytically integrates out the unobserved trait values at all internal
@@ -32,7 +56,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     public static final double LOG_SQRT_2_PI = 0.5 * Math.log(2 * Math.PI);
 
     public IntegratedMultivariateTraitLikelihood(String traitName,
-                                                 TreeModel treeModel,
+                                                 MultivariateTraitTree treeModel,
                                                  MultivariateDiffusionModel diffusionModel,
                                                  CompoundParameter traitParameter,
                                                  List<Integer> missingIndices,
@@ -46,7 +70,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     }
 
     public IntegratedMultivariateTraitLikelihood(String traitName,
-                                                 TreeModel treeModel,
+                                                 MultivariateTraitTree treeModel,
                                                  MultivariateDiffusionModel diffusionModel,
                                                  CompoundParameter traitParameter,
                                                  Parameter deltaParameter,
@@ -55,12 +79,38 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                                                  BranchRateModel rateModel, Model samplingDensity,
                                                  boolean reportAsMultivariate,
                                                  boolean reciprocalRates) {
+        this(traitName, treeModel, diffusionModel, traitParameter, deltaParameter, missingIndices, cacheBranches,
+                scaleByTime, useTreeLength, rateModel, null, samplingDensity, reportAsMultivariate, reciprocalRates);
+    }
+
+
+    private final CacheHelper cacheHelper;
+
+    public IntegratedMultivariateTraitLikelihood(String traitName,
+                                                 MultivariateTraitTree treeModel,
+                                                 MultivariateDiffusionModel diffusionModel,
+                                                 CompoundParameter traitParameter,
+                                                 Parameter deltaParameter,
+                                                 List<Integer> missingIndices,
+                                                 boolean cacheBranches, boolean scaleByTime, boolean useTreeLength,
+                                                 BranchRateModel rateModel,
+                                                 List<BranchRateModel> driftModels,
+                                                 Model samplingDensity,
+                                                 boolean reportAsMultivariate,
+                                                 boolean reciprocalRates) {
 
         super(traitName, treeModel, diffusionModel, traitParameter, deltaParameter, missingIndices, cacheBranches, scaleByTime,
-                useTreeLength, rateModel, samplingDensity, reportAsMultivariate, reciprocalRates);
+                useTreeLength, rateModel, driftModels, samplingDensity, reportAsMultivariate, reciprocalRates);
 
 
+        // Delegate caches to helper
         meanCache = new double[dim * treeModel.getNodeCount()];
+        if (driftModels != null) {
+            cacheHelper = new DriftCacheHelper(dim * treeModel.getNodeCount(), cacheBranches); // new DriftCacheHelper ....
+        } else {
+            cacheHelper = new CacheHelper(dim * treeModel.getNodeCount(), cacheBranches);
+        }
+
         drawnStates = new double[dim * treeModel.getNodeCount()];
         upperPrecisionCache = new double[treeModel.getNodeCount()];
         lowerPrecisionCache = new double[treeModel.getNodeCount()];
@@ -73,9 +123,6 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             storedLogRemainderDensityCache = new double[treeModel.getNodeCount()];
         }
 
-        missing = new boolean[treeModel.getNodeCount()];
-        Arrays.fill(missing, true); // All internal and root nodes are missing
-
         // Set up reusable temporary storage
         Ay = new double[dimTrait];
         tmpM = new double[dimTrait][dimTrait];
@@ -83,23 +130,20 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         zeroDimVector = new double[dim];
 
-        setTipDataValuesForAllNodes(missingIndices);
+        missingTraits = new MissingTraits.CompletelyMissing(treeModel, missingIndices, dim);
+        setTipDataValuesForAllNodes();
 
     }
 
-    private void setTipDataValuesForAllNodes(List<Integer> missingIndices) {
+    private void setTipDataValuesForAllNodes() {
         for (int i = 0; i < treeModel.getExternalNodeCount(); i++) {
             NodeRef node = treeModel.getExternalNode(i);
             setTipDataValuesForNode(node);
         }
-        for (Integer i : missingIndices) {
-            int whichTip = i / dim;
-            Logger.getLogger("dr.evomodel").info(
-                    "\tMarking taxon " + treeModel.getTaxonId(whichTip) + " as completely missing");
-            missing[whichTip] = true;
-        }
+        missingTraits.handleMissingTips();
     }
 
+    @SuppressWarnings("unused")
     public double getTotalTreePrecision() {
         getLogLikelihood(); // Do peeling if necessary
         final int rootIndex = treeModel.getRoot().getNumber();
@@ -113,19 +157,24 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         if (traitValue.length < dim) {
             throw new RuntimeException("The trait parameter for the tip with index, " + index + ", is too short");
         }
-        System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
-        missing[index] = false;
+
+        cacheHelper.setTipMeans(traitValue, dim, index, node);
+//        System.arraycopy(traitValue, 0, meanCache
+////                cacheHelper.getMeanCache()
+//                , dim * index, dim);
     }
 
     public double[] getTipDataValues(int index) {
         double[] traitValue = new double[dim];
-        System.arraycopy(meanCache, dim * index, traitValue, 0, dim);
+        System.arraycopy(cacheHelper.getMeanCache(), dim * index, traitValue, 0, dim);
         return traitValue;
     }
 
     public void setTipDataValuesForNode(int index, double[] traitValue) {
         // Set tip data values
-        System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+        // cacheHelper.copyToMeanCache(traitValue, dim*index, dim);
+        cacheHelper.setTipMeans(traitValue, dim, index);
+        //System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
         makeDirty();
     }
 
@@ -166,7 +215,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         if (computeWishartStatistics) {
 //            if (wishartStatistics == null) {
-                wishartStatistics = new WishartSufficientStatistics(dimTrait);
+            wishartStatistics = new WishartSufficientStatistics(dimTrait);
 //            } else {
 //                wishartStatistics.clear();
 //            }
@@ -176,7 +225,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         postOrderTraverse(treeModel, treeModel.getRoot(), traitPrecision, logDetTraitPrecision, computeWishartStatistics);
 
         if (DEBUG) {
-            System.err.println("mean: " + new Vector(meanCache));
+            System.err.println("mean: " + new Vector(cacheHelper.getMeanCache()));
+            System.err.println("correctedMean: " + new Vector(cacheHelper.getCorrectedMeanCache()));
             System.err.println("upre: " + new Vector(upperPrecisionCache));
             System.err.println("lpre: " + new Vector(lowerPrecisionCache));
             System.err.println("cach: " + new Vector(logRemainderDensityCache));
@@ -193,7 +243,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             double thisLogLikelihood = 0;
 
             // Get conditional mean of datum conditional on root
-            System.arraycopy(meanCache, rootIndex * dim + datum * dimTrait, conditionalRootMean, 0, dimTrait);
+            // System.arraycopy(meanCache, rootIndex * dim + datum * dimTrait, conditionalRootMean, 0, dimTrait);
+            System.arraycopy(cacheHelper.getMeanCache(), rootIndex * dim + datum * dimTrait, conditionalRootMean, 0, dimTrait);
 
             if (DEBUG) {
                 System.err.println("Datum #" + datum);
@@ -245,23 +296,43 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         }
 
         logLikelihood += sumLogRemainders();
+        if (DEBUG) {
+            System.out.println("logLikelihood is " + logLikelihood);
+        }
 
         if (DEBUG) { // Root trait is univariate!!!
             System.err.println("logLikelihood (final) = " + logLikelihood);
 //            checkViaLargeMatrixInversion();
         }
 
+        if (DEBUG_PNAS) {
+            checkLogLikelihood(logLikelihood, sumLogRemainders(), conditionalRootMean,
+                    conditionalRootPrecision, traitPrecision);
+        }
+
         areStatesRedrawn = false;  // Should redraw internal node states when needed
         return logLikelihood;
     }
 
+    protected void checkLogLikelihood(double loglikelihood, double logRemainders,
+                                      double[] conditionalRootMean, double conditionalRootPrecision,
+                                      double[][] traitPrecision) {
+        // Do nothing; for checking PNAS paper
+    }
+
     protected void handleVariableChangedEvent(Variable variable, int index, Parameter.ChangeType type) {
         if (variable == traitParameter) { // A tip value got updated
             if (index > dimTrait * treeModel.getExternalNodeCount()) {
                 throw new RuntimeException("Attempting to update an invalid index");
             }
-            meanCache[index] = traitParameter.getValue(index);
+            cacheHelper.setMeanCache(index, traitParameter.getValue(index));
+            // meanCache[index] = traitParameter.getValue(index);
             likelihoodKnown = false;
+//            if (!cacheBranches) {
+//                throw new RuntimeException("Must cache means in IMTL if they are random");
+//            }
+            // TODO Need better solution.  If tips are random, cacheBranches should be true (to get reset).
+            // TODO However, jitter calls setParameterValue() on the tips at initialization
         }
         super.handleVariableChangedEvent(variable, index, type);
     }
@@ -298,7 +369,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         areStatesRedrawn = false;
     }
 
-    void postOrderTraverse(TreeModel treeModel, NodeRef node, double[][] precisionMatrix,
+    void postOrderTraverse(MultivariateTraitTree treeModel, NodeRef node, double[][] precisionMatrix,
                            double logDetPrecisionMatrix, boolean cacheOuterProducts) {
 
         final int thisNumber = node.getNumber();
@@ -307,11 +378,11 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
             // Fill in precision scalar, traitValues already filled in
 
-            if (missing[thisNumber]) {
+            if (missingTraits.isCompletelyMissing(thisNumber)) {
                 upperPrecisionCache[thisNumber] = 0;
                 lowerPrecisionCache[thisNumber] = 0; // Needed in the pre-order traversal
             } else { // not missing tip trait
-                upperPrecisionCache[thisNumber] = 1.0 / getRescaledBranchLength(node);
+                upperPrecisionCache[thisNumber] = 1.0 / getRescaledBranchLengthForPrecision(node);
                 lowerPrecisionCache[thisNumber] = Double.POSITIVE_INFINITY;
             }
             return;
@@ -335,24 +406,32 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         lowerPrecisionCache[thisNumber] = totalPrecision;
 
-        // Multiple child0 and child1 densities
+        // Multiply child0 and child1 densities
 
-        if (totalPrecision == 0) {
-            System.arraycopy(zeroDimVector, 0, meanCache, meanThisOffset, dim);
-        } else {
-
-//            computeWeightedMeanCache(meanThisOffset, meanOffset0, meanOffset1, precision0, precision1);
-
-            computeWeightedAverage(
-                    meanCache, meanOffset0, precision0,
-                    meanCache, meanOffset1, precision1,
-                    meanCache, meanThisOffset, dim);
-        }
+        // Delegate this!
+        cacheHelper.computeMeanCaches(meanThisOffset, meanOffset0, meanOffset1,
+                totalPrecision, precision0, precision1, missingTraits, node, childNode0, childNode1);
+//        if (totalPrecision == 0) {
+//            System.arraycopy(zeroDimVector, 0, meanCache, meanThisOffset, dim);
+//        } else {
+//            // Delegate in case either child is partially missing
+//            // computeCorrectedWeightedAverage
+//            missingTraits.computeWeightedAverage(meanCache,
+//                    meanOffset0, precision0,
+//                    meanOffset1, precision1,
+//                    meanThisOffset, dim);
+//        }
+        // In this delegation, you can call
+        //getShiftForBranchLength(node);
 
         if (!treeModel.isRoot(node)) {
             // Integrate out trait value at this node
-            double thisPrecision = 1.0 / getRescaledBranchLength(node);
-            upperPrecisionCache[thisNumber] = totalPrecision * thisPrecision / (totalPrecision + thisPrecision);
+            double thisPrecision = 1.0 / getRescaledBranchLengthForPrecision(node);
+            if (Double.isInfinite(thisPrecision)) {
+                upperPrecisionCache[thisNumber] = totalPrecision;
+            } else {
+                upperPrecisionCache[thisNumber] = totalPrecision * thisPrecision / (totalPrecision + thisPrecision);
+            }
         }
 
         // Compute logRemainderDensity
@@ -396,18 +475,27 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
             for (int i = 0; i < dimTrait; i++) {
 
-                final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
-                final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
+                // In case of no drift, getCorrectedMeanCache() simply returns mean cache
+                // final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
+                final double wChild0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i] * precision0;
+                // final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
+                final double wChild1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i] * precision1;
 
                 for (int j = 0; j < dimTrait; j++) {
 
-                    final double child0j = meanCache[childOffset0 + k * dimTrait + j];
-                    final double child1j = meanCache[childOffset1 + k * dimTrait + j];
+                    // subtract "correction"
+                    //final double child0j = meanCache[childOffset0 + k * dimTrait + j];
+                    final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
+                    // subtract "correction"
+                    //final double child1j = meanCache[childOffset1 + k * dimTrait + j];
+                    final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
 
                     childSS0 += wChild0i * precisionMatrix[i][j] * child0j;
                     childSS1 += wChild1i * precisionMatrix[i][j] * child1j;
 
-                    crossSS += (wChild0i + wChild1i) * precisionMatrix[i][j] * meanCache[thisOffset + k * dimTrait + j];
+                    // make sure meanCache in following is not "corrected"
+                    // crossSS += (wChild0i + wChild1i) * precisionMatrix[i][j] * meanCache[thisOffset + k * dimTrait + j];
+                    crossSS += (wChild0i + wChild1i) * precisionMatrix[i][j] * cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
                 }
             }
 
@@ -430,18 +518,23 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
             for (int i = 0; i < dimTrait; i++) {
 
-                final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
-                final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
+                // final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
+                // final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
+                final double wChild0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i] * precision0;
+                final double wChild1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i] * precision1;
 
                 for (int j = 0; j < dimTrait; j++) {
 
-                    final double child0j = meanCache[childOffset0 + k * dimTrait + j];
-                    final double child1j = meanCache[childOffset1 + k * dimTrait + j];
+                    //final double child0j = meanCache[childOffset0 + k * dimTrait + j];
+                    //final double child1j = meanCache[childOffset1 + k * dimTrait + j];
+                    final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
+                    final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
 
                     outerProduct[i][j] += wChild0i * child0j;
                     outerProduct[i][j] += wChild1i * child1j;
 
-                    outerProduct[i][j] -= (wChild0i + wChild1i) * meanCache[thisOffset + k * dimTrait + j];
+                    //outerProduct[i][j] -= (wChild0i + wChild1i) * meanCache[thisOffset + k * dimTrait + j];
+                    outerProduct[i][j] -= (wChild0i + wChild1i) * cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
                 }
             }
         }
@@ -468,9 +561,9 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     public double[] getTraitForNode(Tree tree, NodeRef node, String traitName) {
 
-        if (tree != treeModel) {
-            throw new RuntimeException("Can only reconstruct states on treeModel given to constructor");
-        }
+//        if (tree != treeModel) {
+//            throw new RuntimeException("Can only reconstruct states on treeModel given to constructor");
+//        }
 
         getLogLikelihood();
 
@@ -502,7 +595,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         super.storeState();
 
         if (cacheBranches) {
-            System.arraycopy(meanCache, 0, storedMeanCache, 0, meanCache.length);
+            //     System.arraycopy(meanCache, 0, storedMeanCache, 0, meanCache.length);
+            cacheHelper.store();
             System.arraycopy(upperPrecisionCache, 0, storedUpperPrecisionCache, 0, upperPrecisionCache.length);
             System.arraycopy(lowerPrecisionCache, 0, storedLowerPrecisionCache, 0, lowerPrecisionCache.length);
             System.arraycopy(logRemainderDensityCache, 0, storedLogRemainderDensityCache, 0, logRemainderDensityCache.length);
@@ -515,9 +609,10 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         if (cacheBranches) {
             double[] tmp;
 
-            tmp = storedMeanCache;
-            storedMeanCache = meanCache;
-            meanCache = tmp;
+            cacheHelper.restore();
+            //  tmp = storedMeanCache;
+            //  storedMeanCache = meanCache;
+            //  meanCache = tmp;
 
             tmp = storedUpperPrecisionCache;
             storedUpperPrecisionCache = upperPrecisionCache;
@@ -548,10 +643,10 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     // Computes the weighted average of two vectors, used many times in these computations
 
-    protected static void computeWeightedAverage(double[] in0, int offset0, double weight0,
-                                                 double[] in1, int offset1, double weight1,
-                                                 double[] out2, int offset2,
-                                                 int length) {
+    public static void computeWeightedAverage(double[] in0, int offset0, double weight0,
+                                              double[] in1, int offset1, double weight1,
+                                              double[] out2, int offset2,
+                                              int length) {
 
         final double totalInverseWeight = 1.0 / (weight0 + weight1);
         for (int i = 0; i < length; i++) {
@@ -559,13 +654,58 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         }
     }
 
+
+    protected void computeCorrectedWeightedAverage(int offset0, double weight0, NodeRef childNode0,
+                                                   int offset1, double weight1, NodeRef childNode1,
+                                                   int offset2,
+                                                   int length, NodeRef thisNode) {
+
+        final double totalInverseWeight = 1.0 / (weight0 + weight1);
+        // TODO fix
+//        final double length0 = getRescaledBranchLength(childNode0);
+//        final double length1 = getRescaledBranchLength(childNode1);
+//        final double thisLength = getRescaledBranchLength(thisNode);
+        double[] shift;
+        if (!treeModel.isRoot(thisNode)) {
+            shift = getShiftForBranchLength(thisNode);
+        } else {
+            shift = null;
+        }
+        double[] shiftChild0 = getShiftForBranchLength(childNode0);
+        double[] shiftChild1 = getShiftForBranchLength(childNode1);
+
+        if (treeModel.isExternal(childNode0)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset0 + i] = meanCache[offset0 + i] - shiftChild0[i];
+            }
+        }
+
+        if (treeModel.isExternal(childNode1)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset1 + i] = meanCache[offset1 + i] - shiftChild1[i];
+            }
+        }
+
+        for (int i = 0; i < length; i++) {
+
+            // meanCache[offset2 + i] = ((meanCache[offset0 + i] -  shiftChild0[i]) * weight0 + (meanCache[offset1 + i] - shiftChild1[i]) * weight1) * totalInverseWeight;
+            meanCache[offset2 + i] = (correctedMeanCache[offset0 + i] * weight0 + correctedMeanCache[offset1 + i] * weight1) * totalInverseWeight;
+            if (!treeModel.isRoot(thisNode)) {
+                correctedMeanCache[offset2 + i] = meanCache[offset2 + i] - shift[i];
+            } else {
+                correctedMeanCache[offset2 + i] = meanCache[offset2 + i];
+            }
+        }
+    }
+
+
     protected abstract double[][] computeMarginalRootMeanAndVariance(double[] conditionalRootMean,
                                                                      double[][] treePrecisionMatrix,
                                                                      double[][] treeVarianceMatrix,
                                                                      double conditionalRootPrecision);
 
 
-    private void preOrderTraverseSample(TreeModel treeModel, NodeRef node, int parentIndex, double[][] treePrecision,
+    private void preOrderTraverseSample(MultivariateTraitTree treeModel, NodeRef node, int parentIndex, double[][] treePrecision,
                                         double[][] treeVariance) {
 
         final int thisIndex = node.getNumber();
@@ -578,7 +718,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             double rootPrecision = lowerPrecisionCache[rootIndex];
 
             for (int datum = 0; datum < numData; datum++) {
-                System.arraycopy(meanCache, thisIndex * dim + datum * dimTrait, rootMean, 0, dimTrait);
+                // System.arraycopy(meanCache, thisIndex * dim + datum * dimTrait, rootMean, 0, dimTrait);
+                System.arraycopy(cacheHelper.getMeanCache(), thisIndex * dim + datum * dimTrait, rootMean, 0, dimTrait);
 
                 double[][] variance = computeMarginalRootMeanAndVariance(rootMean, treePrecision, treeVariance,
                         rootPrecision);
@@ -599,16 +740,21 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             }
         } else { // draw conditional on parentState
 
-            if (!missing[thisIndex]) {
+            if (!missingTraits.isCompletelyMissing(thisIndex)
+                    && !missingTraits.isPartiallyMissing(thisIndex)) {
 
-                System.arraycopy(meanCache, thisIndex * dim, drawnStates, thisIndex * dim, dim);
+                //System.arraycopy(meanCache, thisIndex * dim, drawnStates, thisIndex * dim, dim);
+                System.arraycopy(cacheHelper.getMeanCache(), thisIndex * dim, drawnStates, thisIndex * dim, dim);
 
             } else {
 
+                if (missingTraits.isPartiallyMissing(thisIndex)) {
+                    throw new RuntimeException("Partially missing values are not yet implemented");
+                }
                 // This code should work for sampling a missing tip trait as well, but needs testing
 
                 // parent trait at drawnStates[parentOffset]
-                double precisionToParent = 1.0 / getRescaledBranchLength(node);
+                double precisionToParent = 1.0 / getRescaledBranchLengthForPrecision(node);
                 double precisionOfNode = lowerPrecisionCache[thisIndex];
                 double totalPrecision = precisionOfNode + precisionToParent;
 
@@ -631,7 +777,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
                     for (int i = 0; i < dimTrait; i++) {
                         mean[i] = (drawnStates[parentOffset + i] * precisionToParent
-                                + meanCache[thisOffset + i] * precisionOfNode) / totalPrecision;
+                                //  + meanCache[thisOffset + i] * precisionOfNode) / totalPrecision;
+                                + cacheHelper.getMeanCache()[thisOffset + i] * precisionOfNode) / totalPrecision;
                         for (int j = 0; j < dimTrait; j++) {
                             var[i][j] = treeVariance[i][j] / totalPrecision;
                         }
@@ -668,12 +815,147 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     protected boolean areStatesRedrawn = false;
 
     protected double[] meanCache;
+    protected double[] correctedMeanCache;
+
+
+    class CacheHelper {
+
+        public CacheHelper(int cacheLength, boolean cacheBranches) {
+            // modify code later so we can uncomment the following
+
+            meanCache = new double[cacheLength];
+            this.cacheBranches = cacheBranches;
+            if (cacheBranches) {
+                storedMeanCache = new double[cacheLength];
+            }
+
+        }
+
+        public double[] getMeanCache() {
+            return meanCache;
+        }
+
+        public double[] getCorrectedMeanCache() {
+            return meanCache;
+        }
+
+        public void store() {
+            // if (cacheBranches) {
+            System.arraycopy(meanCache, 0, storedMeanCache, 0, meanCache.length);
+            // }
+        }
+
+        public void restore() {
+            //  if (cacheBranches) {
+            double[] tmp = storedMeanCache;
+            storedMeanCache = meanCache;
+            meanCache = tmp;
+            // }
+        }
+
+        public void doFunkyStuff(NodeRef node) {
+
+        }
+
+        protected boolean cacheBranches;
+        //  private double[] meanCache;
+        //  private double[] storedMeanCache;
+
+        public void computeMeanCaches(int meanThisOffset, int meanOffset0, int meanOffset1,
+                                      double precision0, double precision1, MissingTraits missingTraits) {
+            //To change body of created methods use File | Settings | File Templates.
+        }
+
+        public void computeMeanCaches(int meanThisOffset, int meanOffset0, int meanOffset1,
+                                      double totalPrecision, double precision0, double precision1, MissingTraits missingTraits,
+                                      NodeRef thisNode, NodeRef node0, NodeRef node1) {
+            if (totalPrecision == 0) {
+                System.arraycopy(zeroDimVector, 0, meanCache, meanThisOffset, dim);
+            } else {
+                // Delegate in case either child is partially missing
+                // computeCorrectedWeightedAverage
+                missingTraits.computeWeightedAverage(meanCache,
+                        meanOffset0, precision0,
+                        meanOffset1, precision1,
+                        meanThisOffset, dim);
+            }
+        }
+
+        // public void setTipMeans(double[] meanCache, double[] traitValue, int dim, int index, NodeRef node) {
+        public void setTipMeans(double[] traitValue, int dim, int index, NodeRef node) {
+            System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+        }
+
+        public void setTipMeans(double[] traitValue, int dim, int index) {
+            System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+        }
+
+        public void copyToMeanCache(double[] src, int destPos, int length) {
+            System.arraycopy(src, 0, meanCache, destPos, length);
+        }
+
+        public void setMeanCache(int index, double value) {
+            meanCache[index] = value;
+        }
+
+
+    }
+
+    ;
+
+
+    class DriftCacheHelper extends CacheHelper {
+
+        public DriftCacheHelper(int cacheLength, boolean cacheBranches) {
+            super(cacheLength, cacheBranches);
+            correctedMeanCache = new double[cacheLength];
+        }
+
+        public double[] getCorrectedMeanCache() {
+            return correctedMeanCache;
+        }
+
+        public void doFunkyStuff(NodeRef node) {
+            getShiftForBranchLength(node);
+        }
+
+        public void setTipMeans(double[] traitValue, int dim, int index, NodeRef node) {
+            System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+            /*
+            double[] shift = getShiftForBranchLength(node);
+            for (int i = 0; i < dim; i++) {
+                correctedMeanCache[dim * index + i] = traitValue[i] - shift[i];
+            }
+            */
+        }
+
+
+        public void computeMeanCaches(int meanThisOffset, int meanOffset0, int meanOffset1,
+                                      double totalPrecision, double precision0, double precision1, MissingTraits missingTraits,
+                                      NodeRef thisNode, NodeRef node0, NodeRef node1) {
+            if (totalPrecision == 0) {
+                System.arraycopy(zeroDimVector, 0, meanCache, meanThisOffset, dim);
+            } else {
+                // Delegate in case either child is partially missing
+                // computeCorrectedWeightedAverage
+                computeCorrectedWeightedAverage(
+                        meanOffset0, precision0, node0,
+                        meanOffset1, precision1, node1,
+                        meanThisOffset, dim, thisNode);
+            }
+        }
+
+
+        // private double[] correctedMeanCache;
+    }
+
+
+    // protected boolean hasDrift;
+
     protected double[] upperPrecisionCache;
     protected double[] lowerPrecisionCache;
     private double[] logRemainderDensityCache;
 
-    protected boolean[] missing;
-
     private double[] storedMeanCache;
     private double[] storedUpperPrecisionCache;
     private double[] storedLowerPrecisionCache;
@@ -682,8 +964,10 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     private double[] drawnStates;
 
     protected final boolean integrateRoot = true; // Set to false if conditioning on root value (not fully implemented)
+    //  protected final boolean integrateRoot = false;
     protected static boolean DEBUG = false;
     protected static boolean DEBUG_PREORDER = false;
+    protected static boolean DEBUG_PNAS = false;
 
     private double[] zeroDimVector;
 
@@ -694,4 +978,6 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     protected double[][] tmpM;
     protected double[] tmp2;
 
+    protected final MissingTraits missingTraits;
+
 }
