@@ -1,5 +1,31 @@
+/*
+ * SampledMultivariateTraitLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.continuous;
 
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.branchratemodel.BranchRateModel;
@@ -18,7 +44,7 @@ import java.util.List;
 public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTraitLikelihood {
 
     public SampledMultivariateTraitLikelihood(String traitName,
-                                              TreeModel treeModel,
+                                              MultivariateTraitTree treeModel,
                                               MultivariateDiffusionModel diffusionModel,
                                               CompoundParameter traitParameter,
                                               List<Integer> missingIndices,
@@ -53,7 +79,7 @@ public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTrai
         return logLikelihood;
     }
 
-    protected  double calculateAscertainmentCorrection(int taxonIndex) {
+    protected double calculateAscertainmentCorrection(int taxonIndex) {
         throw new RuntimeException("Ascertainment correction not yet implemented for sampled trait likelihoods");
     }
 
@@ -69,7 +95,7 @@ public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTrai
 
                 double[] tipTrait = treeModel.getMultivariateNodeTrait(tip, traitName);
                 double[] parentTrait = treeModel.getMultivariateNodeTrait(parent, traitName);
-                double time = getRescaledBranchLength(tip);
+                double time = getRescaledBranchLengthForPrecision(tip);
 
                 logLikelihood += diffusionModel.getLogLikelihood(parentTrait, tipTrait, time);
             }
@@ -89,7 +115,7 @@ public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTrai
             if (!validLogLikelihoods[nodeNumber]) { // recompute
 
                 childTrait = treeModel.getMultivariateNodeTrait(node, traitName);
-                double time = getRescaledBranchLength(node);
+                double time = getRescaledBranchLengthForPrecision(node);
                 if (parentTrait == null)
                     parentTrait = treeModel.getMultivariateNodeTrait(treeModel.getParent(node), traitName);
                 logL = diffusionModel.getLogLikelihood(parentTrait, childTrait, time);
@@ -114,7 +140,7 @@ public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTrai
 
         if (parentTrait != null) {
 
-            double time = getRescaledBranchLength(node);
+            double time = getRescaledBranchLengthForPrecision(node);
             logL = diffusionModel.getLogLikelihood(parentTrait, childTrait, time);
             if (new Double(logL).isNaN()) {
                 System.err.println("AbstractMultivariateTraitLikelihood: likelihood is undefined");
@@ -149,7 +175,7 @@ public class SampledMultivariateTraitLikelihood extends AbstractMultivariateTrai
 
 
     public double[] getTraitForNode(Tree treeModel, NodeRef node, String traitName) {
-        return ((TreeModel)treeModel).getMultivariateNodeTrait(node,traitName);
+        return ((TreeModel) treeModel).getMultivariateNodeTrait(node, traitName);
     }
 
 }
\ No newline at end of file
