@@ -1,8 +1,33 @@
+/*
+ * SemiConjugateMultivariateTraitLikelihood.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.continuous;
 
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.evolution.tree.NodeRef;
 import dr.evomodel.branchratemodel.BranchRateModel;
-import dr.evomodel.tree.TreeModel;
 import dr.inference.model.CompoundParameter;
 import dr.inference.model.Model;
 import dr.math.distributions.MultivariateNormalDistribution;
@@ -20,7 +45,7 @@ import java.util.List;
 public class SemiConjugateMultivariateTraitLikelihood extends IntegratedMultivariateTraitLikelihood {
 
     public SemiConjugateMultivariateTraitLikelihood(String traitName,
-                                                    TreeModel treeModel,
+                                                    MultivariateTraitTree treeModel,
                                                     MultivariateDiffusionModel diffusionModel,
                                                     CompoundParameter traitParameter,
                                                     List<Integer> missingIndices,
@@ -44,15 +69,15 @@ public class SemiConjugateMultivariateTraitLikelihood extends IntegratedMultivar
         return false;  // No need for outer products, as Gibbs sampling of diffusion matrix is not possible
     }
 
-    protected  double calculateAscertainmentCorrection(int taxonIndex) {
+    protected double calculateAscertainmentCorrection(int taxonIndex) {
         throw new RuntimeException("Ascertainment correction not yet implemented for semi-conjugate trait likelihoods");
     }
 
-    protected double getRescaledLengthToRoot(NodeRef node) {
+    public double getRescaledLengthToRoot(NodeRef node) {
         double length = 0;
         final NodeRef root = treeModel.getRoot();
         while (node != root) {
-            length += getRescaledBranchLength(node);
+            length += getRescaledBranchLengthForPrecision(node);
             node = treeModel.getParent(node);
         }
         return length;
