@@ -1,7 +1,7 @@
 /*
  * PrecisionMatrixGibbsOperator.java
  *
- * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,11 +25,13 @@
 
 package dr.evomodel.operators;
 
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.evolution.tree.NodeRef;
 import dr.evomodel.continuous.AbstractMultivariateTraitLikelihood;
 import dr.evomodel.continuous.SampledMultivariateTraitLikelihood;
-import dr.evomodel.tree.TreeModel;
 import dr.inference.distribution.MultivariateDistributionLikelihood;
+import dr.inference.distribution.MultivariateNormalDistributionModel;
+import dr.inference.distribution.WishartGammalDistributionModel;
 import dr.inference.model.MatrixParameter;
 import dr.inference.model.Parameter;
 import dr.inference.operators.GibbsOperator;
@@ -37,50 +39,102 @@ import dr.inference.operators.MCMCOperator;
 import dr.inference.operators.OperatorFailedException;
 import dr.inference.operators.SimpleMCMCOperator;
 import dr.math.distributions.WishartDistribution;
+import dr.math.distributions.WishartStatistics;
 import dr.math.distributions.WishartSufficientStatistics;
 import dr.math.interfaces.ConjugateWishartStatisticsProvider;
 import dr.math.matrixAlgebra.IllegalDimension;
 import dr.math.matrixAlgebra.SymmetricMatrix;
+import dr.util.Attribute;
 import dr.xml.*;
 
+import java.util.List;
+
+//import dr.math.matrixAlgebra.Matrix;
+
 /**
  * @author Marc Suchard
  */
 public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements GibbsOperator {
 
     public static final String VARIANCE_OPERATOR = "precisionGibbsOperator";
-//    public static final String PRECISION_MATRIX = "precisionMatrix";
+    //    public static final String PRECISION_MATRIX = "precisionMatrix";
     public static final String TREE_MODEL = "treeModel";
-//    public static final String OUTCOME = "outcome";
-    public static final String MEAN = "mean";
+    //    public static final String OUTCOME = "outcome";
+//    public static final String MEAN = "mean";
+    public static final String DISTRIBUTION = "distribution";
     public static final String PRIOR = "prior";
 //    public static final String TRAIT_MODEL = "traitModel";
 
     private final AbstractMultivariateTraitLikelihood traitModel;
+    private final MultivariateDistributionLikelihood multivariateLikelihood;
+    private final Parameter meanParam;
     private final MatrixParameter precisionParam;
-    //    private WishartDistribution priorDistribution;
-    private final double priorDf;
+
+    private double priorDf;
     private SymmetricMatrix priorInverseScaleMatrix;
-    private final TreeModel treeModel;
+    private final MultivariateTraitTree treeModel;
     private final int dim;
     private double numberObservations;
     private final String traitName;
     private final boolean isSampledTraitLikelihood;
+    private double pathWeight = 1.0;
+
+    private boolean wishartIsModel = false;
+    private WishartGammalDistributionModel priorModel = null;
+
+    public PrecisionMatrixGibbsOperator(
+            MultivariateDistributionLikelihood likelihood,
+            WishartStatistics priorDistribution,
+            double weight) {
+        super();
+
+        // Unnecessary variables
+        this.traitModel = null;
+        this.treeModel = null;
+        this.traitName = null;
+        this.isSampledTraitLikelihood = false;
+
+        this.multivariateLikelihood = likelihood;
+        MultivariateNormalDistributionModel density = (MultivariateNormalDistributionModel) likelihood.getDistribution();
+        this.meanParam = density.getMeanParameter();
+        this.precisionParam = density.getPrecisionMatrixParameter();
+        this.dim = meanParam.getDimension();
+
+        setupWishartStatistics(priorDistribution);
+        if (priorDistribution instanceof WishartGammalDistributionModel) {
+            wishartIsModel = true;
+            priorModel = (WishartGammalDistributionModel) priorDistribution;
+        }
+
+        setWeight(weight);
+    }
+
+    private void setupWishartStatistics(WishartStatistics priorDistribution) {
+        this.priorDf = priorDistribution.getDF();
+        this.priorInverseScaleMatrix = null;
+        double[][] scale = priorDistribution.getScaleMatrix();
+        if (scale != null)
+            this.priorInverseScaleMatrix =
+                    (SymmetricMatrix) (new SymmetricMatrix(scale)).inverse();
+    }
 
     public PrecisionMatrixGibbsOperator(
             AbstractMultivariateTraitLikelihood traitModel,
-            WishartDistribution priorDistribution,
+            WishartStatistics priorDistribution,
             double weight) {
         super();
         this.traitModel = traitModel;
+        this.meanParam = null;
         this.precisionParam = (MatrixParameter) traitModel.getDiffusionModel().getPrecisionParameter();
-//        this.priorDistribution = priorDistribution;
-        this.priorDf = priorDistribution.df();
-        this.priorInverseScaleMatrix = null;
-        if (priorDistribution.scaleMatrix() != null)
-            this.priorInverseScaleMatrix =
-                    (SymmetricMatrix) (new SymmetricMatrix(priorDistribution.scaleMatrix())).inverse();
+
+        setupWishartStatistics(priorDistribution);
+        if (priorDistribution instanceof WishartGammalDistributionModel) {
+            wishartIsModel = true;
+            priorModel = (WishartGammalDistributionModel) priorDistribution;
+        }
+
         setWeight(weight);
+
         this.treeModel = traitModel.getTreeModel();
         traitName = traitModel.getTraitName();
         dim = precisionParam.getRowDimension(); // assumed to be square
@@ -89,9 +143,18 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
 
         if (!isSampledTraitLikelihood &&
                 !(traitModel instanceof ConjugateWishartStatisticsProvider)) {
-            throw new RuntimeException("Only implemented for a SampledMultivariateTraitLikelihood and " +
+            throw new RuntimeException("Only implemented for a SampledMultivariateTraitLikelihood or " +
                     "ConjugateWishartStatisticsProvider");
         }
+
+        multivariateLikelihood = null;
+    }
+
+    public void setPathParameter(double beta) {
+        if (beta < 0 || beta > 1) {
+            throw new IllegalArgumentException("Illegal path weight of " + beta);
+        }
+        pathWeight = beta;
     }
 
     public int getStepCount() {
@@ -114,6 +177,30 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
 //        }
 //    }
 
+    private void incrementOuterProduct(double[][] S,
+                                       MultivariateDistributionLikelihood likelihood) {
+
+        double[] mean = likelihood.getDistribution().getMean();
+
+        numberObservations = 0;
+        List<Attribute<double[]>> dataList = likelihood.getDataList();
+        int count = 0;
+        for (Attribute<double[]> d : dataList) {
+
+            double[] data = d.getAttributeValue();
+            for (int i = 0; i < dim; i++) {
+                data[i] -= mean[i];
+            }
+
+            for (int i = 0; i < dim; i++) {  // symmetric matrix,
+                for (int j = i; j < dim; j++) {
+                    S[j][i] = S[i][j] += data[i] * data[j];
+                }
+            }
+            numberObservations += 1;
+        }
+    }
+
     private void incrementOuterProduct(double[][] S,
                                        ConjugateWishartStatisticsProvider integratedLikelihood) {
 
@@ -143,7 +230,7 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
             NodeRef parent = treeModel.getParent(node);
             double[] parentTrait = treeModel.getMultivariateNodeTrait(parent, traitName);
             double[] childTrait = treeModel.getMultivariateNodeTrait(node, traitName);
-            double time = traitModel.getRescaledBranchLength(node);
+            double time = traitModel.getRescaledBranchLengthForPrecision(node);
 
             if (time > 0) {
 
@@ -177,11 +264,18 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
         if (isSampledTraitLikelihood) {
             incrementOuterProduct(S, treeModel.getRoot());
         } else { // IntegratedTraitLikelihood
-            incrementOuterProduct(S, (ConjugateWishartStatisticsProvider) traitModel);
+            if (traitModel != null) { // is a tree
+                incrementOuterProduct(S, (ConjugateWishartStatisticsProvider) traitModel);
+            } else { // is a normal-normal-wishart model
+                incrementOuterProduct(S, multivariateLikelihood);
+            }
         }
 
         try {
             S2 = new SymmetricMatrix(S);
+            if (pathWeight != 1.0) {
+                S2 = (SymmetricMatrix) S2.product(pathWeight);
+            }
             if (priorInverseScaleMatrix != null)
                 S2 = priorInverseScaleMatrix.add(S2);
             inverseS2 = (SymmetricMatrix) S2.inverse();
@@ -197,11 +291,51 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
 
     public double doOperation() throws OperatorFailedException {
 
+        if (wishartIsModel) {
+            setupWishartStatistics(priorModel);
+        }
+
         final double[][] scaleMatrix = getOperationScaleMatrixAndSetObservationCount();
         final double treeDf = numberObservations;
-        final double df = priorDf + treeDf;
+        final double df = priorDf + treeDf * pathWeight;
 
         double[][] draw = WishartDistribution.nextWishart(df, scaleMatrix);
+//        int tries  = 0;
+//        int limit = 100;
+//        boolean success = false;
+//
+//        double[][] draw = null;
+//
+//        while (!success && tries < limit) {
+//
+//
+//        draw = WishartDistribution.nextWishart(df, scaleMatrix);
+//
+//        Matrix m = new Matrix(draw);
+//        try {
+//            double logDet = m.logDeterminant();
+//            if (Double.isNaN(logDet)) {
+//                System.err.println("Bad proposal!");
+//                System.err.println("df = " + df);
+//
+////                System.err.println(m);
+////                System.exit(-1);
+//            } else {
+//                success = true;
+//            }
+//
+//        } catch (IllegalDimension illegalDimension) {
+//            illegalDimension.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
+//            System.exit(-1);
+//        }
+//
+//            tries++;
+//        }
+//
+//        if (tries >= limit) {
+//            System.err.println("Too many attempts!");
+//            System.exit(-1);
+//        }
 
         for (int i = 0; i < dim; i++) {
             Parameter column = precisionParam.getParameter(i);
@@ -229,23 +363,59 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
 
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
+            if (xo.getChildCount() != 2) {
+                throw new XMLParseException(
+                        "Element with id = '" + xo.getName() + "' should contain either:\n" +
+                                "\t 1 multivariateTraitLikelihood and 1 multivariateDistributionLikelihood (prior), or\n" +
+                                "\t 2 multivariateDistributionLikelihoods (likelihood and prior)\n"
+                );
+            }
+
             double weight = xo.getDoubleAttribute(WEIGHT);
             AbstractMultivariateTraitLikelihood traitModel = (AbstractMultivariateTraitLikelihood) xo.getChild(AbstractMultivariateTraitLikelihood.class);
+            MultivariateDistributionLikelihood prior = null;
+            MatrixParameter precMatrix = null;
+            MultivariateDistributionLikelihood likelihood = null;
 
-            MatrixParameter precMatrix = (MatrixParameter) traitModel.getDiffusionModel().getPrecisionParameter();
+            if (traitModel != null) {
+                precMatrix = (MatrixParameter) traitModel.getDiffusionModel().getPrecisionParameter();
+                prior = (MultivariateDistributionLikelihood) xo.getChild(MultivariateDistributionLikelihood.class);
+            } else { // generic likelihood and prior
+                for (int i = 0; i < xo.getChildCount(); ++i) {
+                    MultivariateDistributionLikelihood density = (MultivariateDistributionLikelihood) xo.getChild(i);
+                    if (density.getDistribution() instanceof WishartStatistics) {
+                        prior = density;
+                    } else if (density.getDistribution() instanceof MultivariateNormalDistributionModel) {
+                        likelihood = density;
+                        precMatrix = ((MultivariateNormalDistributionModel)
+                                density.getDistribution()).getPrecisionMatrixParameter();
+                    }
+                }
 
-            MultivariateDistributionLikelihood prior = (MultivariateDistributionLikelihood) xo.getChild(MultivariateDistributionLikelihood.class);
-            if (!(prior.getDistribution() instanceof WishartDistribution))
-                throw new RuntimeException("Only a Wishart distribution is conjugate for Gibbs sampling");
+                if (prior == null || likelihood == null) {
+                    throw new XMLParseException(
+                            "Must provide a multivariate normal likelihood and Wishart prior in element '" +
+                                    xo.getName() + "'\n"
+                    );
+                }
+            }
+
+            if (!(prior.getDistribution() instanceof WishartStatistics)) {
+                throw new XMLParseException("Only a Wishart distribution is conjugate for Gibbs sampling");
+            }
 
             // Make sure precMatrix is square and dim(precMatrix) = dim(parameter)
+            if (precMatrix.getColumnDimension() != precMatrix.getRowDimension()) {
+                throw new XMLParseException("The variance matrix is not square or of wrong dimension");
+            }
 
-            if (precMatrix.getColumnDimension() != precMatrix.getRowDimension())
-                throw new XMLParseException("The variance matrix is not square");
-
-            return new PrecisionMatrixGibbsOperator(
-                    traitModel, (WishartDistribution) prior.getDistribution(), weight
-            );
+            if (traitModel != null) {
+                return new PrecisionMatrixGibbsOperator(
+                        traitModel, (WishartStatistics) prior.getDistribution(), weight
+                );
+            } else {
+                return new PrecisionMatrixGibbsOperator(likelihood, (WishartStatistics) prior.getDistribution(), weight);
+            }
         }
 
         //************************************************************************
@@ -266,8 +436,8 @@ public class PrecisionMatrixGibbsOperator extends SimpleMCMCOperator implements
 
         private XMLSyntaxRule[] rules = new XMLSyntaxRule[]{
                 AttributeRule.newDoubleRule(WEIGHT),
-                new ElementRule(AbstractMultivariateTraitLikelihood.class),
-                new ElementRule(MultivariateDistributionLikelihood.class),
+                new ElementRule(AbstractMultivariateTraitLikelihood.class, true),
+                new ElementRule(MultivariateDistributionLikelihood.class, 1, 2),
         };
     };
 }
