@@ -29,7 +29,6 @@ import dr.evolution.tree.MutableTree;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evolution.util.Taxon;
-import dr.evomodel.speciation.SpeciesBindings.GeneTreeInfo;
 import dr.evomodel.tree.TreeModel;
 import dr.evomodelxml.speciation.MulSpeciesBindingsParser;
 import dr.inference.loggers.LogColumn;
@@ -63,7 +62,6 @@ public class MulSpeciesBindings extends AbstractModel  implements Loggable {
 
     // grj Species definition    
     private final AlloppSpeciesBindings.ApSpInfo[] apspecies;
-    private final AlloppSpeciesBindings.Individual[] indivs;
     private final Taxon[] taxa;
     private final Map<Taxon, Integer> taxon2index = new HashMap<Taxon, Integer>();
 	private final int spsq[][];
@@ -98,26 +96,26 @@ public class MulSpeciesBindings extends AbstractModel  implements Loggable {
         
         // make the flattened arrays
         int n = 0;
-        for (int s = 0; s < apspecies.length; s++) {
-        	n += apspecies[s].individuals.length;
+        for (AlloppSpeciesBindings.ApSpInfo apspi : apspecies) {
+            n += apspi.individuals.length;
         }
-        indivs = new AlloppSpeciesBindings.Individual[n];
+        AlloppSpeciesBindings.Individual [] indivs = new AlloppSpeciesBindings.Individual[n];
         n = 0;
-        for (int s = 0; s < apspecies.length; s++) {
-        	for (int i = 0; i < apspecies[s].individuals.length; i++, n++) {
-        		indivs[n] =  apspecies[s].individuals[i];
-        	}
+        for (AlloppSpeciesBindings.ApSpInfo apspi : apspecies) {
+            for (int i = 0; i < apspi.individuals.length; i++, n++) {
+                indivs[n] = apspi.individuals[i];
+            }
         }
         int t = 0;
-        for (int i = 0; i < indivs.length; i++) {
-        	t += indivs[i].taxa.length;
+        for (AlloppSpeciesBindings.Individual indiv : indivs) {
+            t += indiv.taxa.length;
         }  
         taxa = new Taxon[t];
         t = 0;
-        for (int i = 0; i < indivs.length; i++) {
-        	for (int j = 0; j < indivs[i].taxa.length; j++, t++) {
-        		taxa[t] =  indivs[i].taxa[j];
-        	}
+        for (AlloppSpeciesBindings.Individual indiv : indivs) {
+            for (int j = 0; j < indiv.taxa.length; j++, t++) {
+                taxa[t] = indiv.taxa[j];
+            }
         }
         // set up maps to indices
         for (int i = 0; i < taxa.length; i++) {
@@ -180,12 +178,6 @@ public class MulSpeciesBindings extends AbstractModel  implements Loggable {
 	public String apspeciesName(int i) {
 		return apspecies[i].name;
 	}
-    
-    // grjtodo spandseq2spseqindex(), spseqindex2sp(), spseqindex2seq(), 
-    // spseqindex2spandseq() pasted from AlloppSpeciesBindings.
-	public int spandseq2spseqindex(int sp, int seq) {
-		return spsq[sp][seq];
-	}
 	
 	
 	//grj
@@ -605,7 +597,7 @@ public class MulSpeciesBindings extends AbstractModel  implements Loggable {
     	}       
 
         
-    	/* grjtodo 
+    	/* grjtodo-oneday
     	 * This is a bit odd. It collects individuals as (sp, iv) indices
     	 * that `belong' to a node in the sense that any taxon (sequence)
     	 * of an individual belongs to the clade of the node.
@@ -733,7 +725,7 @@ public class MulSpeciesBindings extends AbstractModel  implements Loggable {
         
         // grj
 		private void permuteOneAssignment(int sp, int iv) {
-			// grjtodo tetraonly
+			// grjtodo-tetraonly
 			int tx;
 			if (apspecies[sp].individuals[iv].taxa.length == 2) {
 				tx = taxon2index.get(apspecies[sp].individuals[iv].taxa[0]);
