@@ -1,7 +1,7 @@
 /*
  * TransmissionDemographicModel.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -115,24 +115,31 @@ public class TransmissionDemographicModel extends DemographicModel {
         throw new RuntimeException("getDemographicFunction not used for TransmissionDemographicModel");
     }
 
-    public DemographicFunction getDemographicFunction(double transmissionTime, double donorSize) {
+    protected int getIndexFromHost(int host) {
+        return 0;
+    }
+
+    // Attempting to get the transmission model to be host-specific
+    public DemographicFunction getDemographicFunction(double transmissionTime, double donorSize, int host) {
+
+        final int index = getIndexFromHost(host);
 
         if (model == 0) {
             // constant
-            double N0 = N0Parameter.getParameterValue(0);
+            double N0 = N0Parameter.getParameterValue(index);
             ((ConstantPopulation) hostDemographic).setN0(N0);
 
         } else if (model == 1) {
             // exponential
-            double N1 = N1Parameter.getParameterValue(0);
+            double N1 = N1Parameter.getParameterValue(index);
             ((TransmissionDemographicFunction) hostDemographic).setTransmissionTime(transmissionTime);
             ((TransmissionDemographicFunction) hostDemographic).setDonorSize(donorSize);
             ((TransmissionDemographicFunction) hostDemographic).setBottleNeckProportion(N1);
             if (growthRateParameter != null) {
-                double r = growthRateParameter.getParameterValue(0);
+                double r = growthRateParameter.getParameterValue(index);
                 ((TransmissionExponentialGrowth) hostDemographic).setGrowthRate(r);
             } else {
-                double d = doublingTimeParameter.getParameterValue(0);
+                double d = doublingTimeParameter.getParameterValue(index);
                 ((TransmissionExponentialGrowth) hostDemographic).setDoublingTime(d);
             }
 
@@ -141,15 +148,15 @@ public class TransmissionDemographicModel extends DemographicModel {
             ((TransmissionDemographicFunction) hostDemographic).setTransmissionTime(transmissionTime);
             ((TransmissionDemographicFunction) hostDemographic).setDonorSize(donorSize);
 
-            double N0 = N0Parameter.getParameterValue(0);
+            double N0 = N0Parameter.getParameterValue(index);
             ((TransmissionLogisticGrowth) hostDemographic).setN0(N0);
-            double N1 = N1Parameter.getParameterValue(0);
+            double N1 = N1Parameter.getParameterValue(index);
             ((TransmissionDemographicFunction) hostDemographic).setBottleNeckProportion(N1);
             if (growthRateParameter != null) {
-                double r = growthRateParameter.getParameterValue(0);
+                double r = growthRateParameter.getParameterValue(index);
                 ((TransmissionLogisticGrowth) hostDemographic).setGrowthRate(r);
             } else {
-                double d = doublingTimeParameter.getParameterValue(0);
+                double d = doublingTimeParameter.getParameterValue(index);
                 ((TransmissionLogisticGrowth) hostDemographic).setDoublingTime(d);
             }
 
