@@ -2,7 +2,9 @@ package dr.evomodel.tree;
 
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
+import dr.evolution.tree.TreeTrait;
 import dr.evolution.util.Taxon;
+import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.math.MathUtils;
 import dr.inference.operators.MCMCOperator;
 import dr.inference.operators.OperatorFailedException;
@@ -55,6 +57,10 @@ public class EmpiricalTreeDistributionModel extends TreeModel {
 
         currentTreeIndex = MathUtils.nextInt(trees.length);
 
+        // Force computation of node heights now rather than later in the evaluation
+        // where multithreading may get conflicts.
+        trees[currentTreeIndex].getNodeHeight(trees[currentTreeIndex].getRoot());
+
 //        System.err.println(") new tree = " + currentTreeIndex);
 
         fireModelChanged(new TreeModel.TreeChangedEvent());
@@ -181,11 +187,11 @@ public class EmpiricalTreeDistributionModel extends TreeModel {
     }
 
     public void setUnits(final Type units) {
-         trees[currentTreeIndex].setUnits(units);
+        trees[currentTreeIndex].setUnits(units);
     }
 
     public void setAttribute(final String name, final Object value) {
-         trees[currentTreeIndex].setAttribute(name, value);
+        trees[currentTreeIndex].setAttribute(name, value);
     }
 
     public Object getAttribute(final String name) {
@@ -195,7 +201,7 @@ public class EmpiricalTreeDistributionModel extends TreeModel {
     public Iterator<String> getAttributeNames() {
         return trees[currentTreeIndex].getAttributeNames();
     }
-    
+
     public static final String EMPIRICAL_TREE_DISTRIBUTION_MODEL = "empiricalTreeDistributionModel";
 
     private final Tree[] trees;
