@@ -18,7 +18,6 @@ import java.util.logging.Logger;
 /**
  * @author Marc A. Suchard
  */
-@Deprecated
 public class AncestralStateTreeLikelihood extends TreeLikelihood implements TreeTraitProvider {
     public static final String STATES_KEY = "states";
 
@@ -52,6 +51,7 @@ public class AncestralStateTreeLikelihood extends TreeLikelihood implements Tree
         super(patternList, treeModel, siteModel, branchRateModel, null, useAmbiguities, false, storePartials,
                 false, forceRescaling);
         this.dataType = dataType;
+        this.tag = tag;
 
         reconstructedStates = new int[treeModel.getNodeCount()][patternCount];
         storedReconstructedStates = new int[treeModel.getNodeCount()][patternCount];
@@ -59,7 +59,7 @@ public class AncestralStateTreeLikelihood extends TreeLikelihood implements Tree
         this.useMAP = useMAP;
         this.returnMarginalLogLikelihood = returnML;
       
-        treeTraits.addTrait(new TreeTrait.IA() {
+        treeTraits.addTrait(STATES_KEY, new TreeTrait.IA() {
             public String getTraitName() {
                 return tag;
             }
@@ -345,6 +345,7 @@ public class AncestralStateTreeLikelihood extends TreeLikelihood implements Tree
     private int[][] reconstructedStates;
     private int[][] storedReconstructedStates;
 
+    private String tag;
     private boolean areStatesRedrawn = false;
     private boolean storedAreStatesRedrawn = false;
 
