@@ -1,7 +1,32 @@
+/*
+ * GMRFSkyrideBlockUpdateOperatorParser.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodelxml.coalescent.operators;
 
-import dr.evomodel.coalescent.GMRFSkyrideLikelihood;
 import dr.evomodel.coalescent.GMRFMultilocusSkyrideLikelihood;
+import dr.evomodel.coalescent.GMRFSkyrideLikelihood;
 import dr.evomodel.coalescent.operators.GMRFMultilocusSkyrideBlockUpdateOperator;
 import dr.evomodel.coalescent.operators.GMRFSkyrideBlockUpdateOperator;
 import dr.inference.operators.CoercableMCMCOperator;
@@ -19,6 +44,7 @@ import java.util.logging.*;
 public class GMRFSkyrideBlockUpdateOperatorParser extends AbstractXMLObjectParser {
 
     public static final String BLOCK_UPDATE_OPERATOR = "gmrfBlockUpdateOperator";
+    public static final String GRID_BLOCK_UPDATE_OPERATOR = "gmrfGridBlockUpdateOperator";
     public static final String SCALE_FACTOR = "scaleFactor";
     public static final String MAX_ITERATIONS = "maxIterations";
     public static final String STOP_VALUE = "stopValue";
@@ -29,6 +55,13 @@ public class GMRFSkyrideBlockUpdateOperatorParser extends AbstractXMLObjectParse
         return BLOCK_UPDATE_OPERATOR;
     }
 
+    public String[] getParserNames() {
+        return new String[]{
+                getParserName(),
+                GRID_BLOCK_UPDATE_OPERATOR,
+        };
+    }
+
     public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
         boolean logRecord = xo.getAttribute(KEEP_LOG_RECORD, false);
@@ -74,16 +107,16 @@ public class GMRFSkyrideBlockUpdateOperatorParser extends AbstractXMLObjectParse
         double stopValue = xo.getAttribute(STOP_VALUE, 0.01);
 
 
-
-        if (xo.getAttribute(OLD_SKYRIDE, true)) {
-
+        if (xo.getAttribute(OLD_SKYRIDE, true)
+                && !(xo.getName().compareTo(GRID_BLOCK_UPDATE_OPERATOR) == 0)
+                ) {
             GMRFSkyrideLikelihood gmrfLikelihood = (GMRFSkyrideLikelihood) xo.getChild(GMRFSkyrideLikelihood.class);
             return new GMRFSkyrideBlockUpdateOperator(gmrfLikelihood, weight, mode, scaleFactor,
-                maxIterations, stopValue);
-        }else{
+                    maxIterations, stopValue);
+        } else {
             GMRFMultilocusSkyrideLikelihood gmrfMultilocusLikelihood = (GMRFMultilocusSkyrideLikelihood) xo.getChild(GMRFMultilocusSkyrideLikelihood.class);
             return new GMRFMultilocusSkyrideBlockUpdateOperator(gmrfMultilocusLikelihood, weight, mode, scaleFactor,
-                maxIterations, stopValue);
+                    maxIterations, stopValue);
         }
 
     }
