@@ -1,6 +1,31 @@
+/*
+ * TreeTraitParserUtilities.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodelxml.treelikelihood;
 
-import dr.evomodel.tree.TreeModel;
+import dr.evolution.tree.MultivariateTraitTree;
 import dr.inference.model.CompoundParameter;
 import dr.inference.model.Parameter;
 import dr.inference.model.ParameterParser;
@@ -59,10 +84,10 @@ public class TreeTraitParserUtilities {
 
     public void jitter(XMLObject xo, int length, List<Integer> missingIndices) throws XMLParseException {
         XMLObject cxo = xo.getChild(TreeTraitParserUtilities.JITTER);
-                      Parameter traits = (Parameter) cxo.getChild(Parameter.class);
-                      double[] window = cxo.getDoubleArrayAttribute(TreeTraitParserUtilities.WINDOW); // Must be included, no default value
-                      boolean duplicates = cxo.getAttribute(TreeTraitParserUtilities.DUPLICATES, true); // default = true
-                      jitter(traits, length, missingIndices, window, duplicates, true);
+        Parameter traits = (Parameter) cxo.getChild(Parameter.class);
+        double[] window = cxo.getDoubleArrayAttribute(TreeTraitParserUtilities.WINDOW); // Must be included, no default value
+        boolean duplicates = cxo.getAttribute(TreeTraitParserUtilities.DUPLICATES, true); // default = true
+        jitter(traits, length, missingIndices, window, duplicates, true);
     }
 
     public void randomize(XMLObject xo) throws XMLParseException {
@@ -186,7 +211,7 @@ public class TreeTraitParserUtilities {
     public TraitsAndMissingIndices parseTraitsFromTaxonAttributes(
             XMLObject xo,
             String inTraitName,
-            TreeModel treeModel,
+            MultivariateTraitTree treeModel,
             boolean integrateOutInternalStates) throws XMLParseException {
 
         XMLObject xoc = xo.getChild(TRAIT_PARAMETER);
@@ -260,7 +285,7 @@ public class TreeTraitParserUtilities {
                         if (existingTraitParameter) {
                             throw new RuntimeException("Trait length must match trait parameter dimension for taxon, " +
                                     taxonName + ": " +
-                            sampleSize + " != " + traitParam.getDimension());
+                                    sampleSize + " != " + traitParam.getDimension());
                         } else {
                             traitParam.setDimension(sampleSize);
                         }
