@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -44,10 +44,11 @@ import java.util.logging.Logger;
  * @version $Id: MarkovChain.java,v 1.10 2006/06/21 13:34:42 rambaut Exp $
  */
 public final class MarkovChain {
-
     private final static boolean DEBUG = false;
     private final static boolean PROFILE = true;
 
+    public static final double EVALUATION_TEST_THRESHOLD = 1e-6;
+
     private final OperatorSchedule schedule;
     private final Acceptor acceptor;
     private final Prior prior;
@@ -60,14 +61,17 @@ public final class MarkovChain {
 
     private boolean useCoercion = true;
 
-    private final int fullEvaluationCount;
+    private final long fullEvaluationCount;
     private final int minOperatorCountForFullEvaluation;
 
-    private static final double EVALUATION_TEST_THRESHOLD = 1e-6;
+    private double evaluationTestThreshold = EVALUATION_TEST_THRESHOLD;
+
 
     public MarkovChain(Prior prior, Likelihood likelihood,
                        OperatorSchedule schedule, Acceptor acceptor,
-                       int fullEvaluationCount, int minOperatorCountForFullEvaluation, boolean useCoercion) {
+                       long fullEvaluationCount, int minOperatorCountForFullEvaluation, double evaluationTestThreshold,
+                       boolean useCoercion) {
+
         currentLength = 0;
         this.prior = prior;
         this.likelihood = likelihood;
@@ -77,6 +81,7 @@ public final class MarkovChain {
 
         this.fullEvaluationCount = fullEvaluationCount;
         this.minOperatorCountForFullEvaluation = minOperatorCountForFullEvaluation;
+        this.evaluationTestThreshold = evaluationTestThreshold;
 
         currentScore = evaluate(likelihood, prior);
     }
@@ -144,7 +149,7 @@ public final class MarkovChain {
             throw new IllegalArgumentException(message);
         }
 
-            pleaseStop = false;
+        pleaseStop = false;
         isStopped = false;
 
         String diagnostic = "";
@@ -257,7 +262,7 @@ public final class MarkovChain {
                     likelihood.makeDirty();
                     final double testScore = evaluate(likelihood, prior);
 
-                    if (Math.abs(testScore - score) > EVALUATION_TEST_THRESHOLD) {
+                    if (Math.abs(testScore - score) > evaluationTestThreshold) {
                         Logger.getLogger("error").severe(
                                 "State was not correctly calculated after an operator move.\n"
                                         + "Likelihood evaluation: " + score
@@ -300,7 +305,7 @@ public final class MarkovChain {
                 if (usingFullEvaluation) {
                     oldScore = score; // for the usingFullEvaluation test
                     diagnostic = likelihood instanceof CompoundLikelihood ?
-                            ((CompoundLikelihood)likelihood).getDiagnosis() : "";
+                            ((CompoundLikelihood) likelihood).getDiagnosis() : "";
                 }
             } else {
                 if (DEBUG) {
@@ -325,9 +330,9 @@ public final class MarkovChain {
                 final double testScore = evaluate(likelihood, prior);
 
                 final String d2 = likelihood instanceof CompoundLikelihood ?
-                        ((CompoundLikelihood)likelihood).getDiagnosis() : "";
+                        ((CompoundLikelihood) likelihood).getDiagnosis() : "";
 
-                if (Math.abs(testScore - oldScore) > EVALUATION_TEST_THRESHOLD) {
+                if (Math.abs(testScore - oldScore) > evaluationTestThreshold) {
 
 
                     final Logger logger = Logger.getLogger("error");
@@ -364,6 +369,8 @@ public final class MarkovChain {
                 }
             }
 
+            fireEndCurrentIteration(currentState);
+
             currentState += 1;
         }
 
@@ -475,7 +482,7 @@ public final class MarkovChain {
         return isStopped;
     }
 
-    private double evaluate(Likelihood likelihood, Prior prior) {
+    protected double evaluate(Likelihood likelihood, Prior prior) {
 
         double logPosterior = 0.0;
 
@@ -540,6 +547,15 @@ public final class MarkovChain {
         listeners.remove(listener);
     }
 
+    public void addMarkovChainDelegate(MarkovChainDelegate delegate) {
+        delegates.add(delegate);
+    }
+
+    public void removeMarkovChainDelegate(MarkovChainDelegate delegate) {
+        delegates.remove(delegate);
+    }
+
+
     private void fireBestModel(long state, Model bestModel) {
 
         for (MarkovChainListener listener : listeners) {
@@ -551,6 +567,10 @@ public final class MarkovChain {
         for (MarkovChainListener listener : listeners) {
             listener.currentState(state, currentModel);
         }
+
+        for (MarkovChainDelegate delegate : delegates) {
+            delegate.currentState(state);
+        }
     }
 
     private void fireFinished(long chainLength) {
@@ -558,7 +578,18 @@ public final class MarkovChain {
         for (MarkovChainListener listener : listeners) {
             listener.finished(chainLength);
         }
+
+        for (MarkovChainDelegate delegate : delegates) {
+            delegate.finished(chainLength);
+        }
+    }
+
+    private void fireEndCurrentIteration(long state) {
+        for (MarkovChainDelegate delegate : delegates) {
+            delegate.currentStateEnd(state);
+        }
     }
 
     private final ArrayList<MarkovChainListener> listeners = new ArrayList<MarkovChainListener>();
+    private final ArrayList<MarkovChainDelegate> delegates = new ArrayList<MarkovChainDelegate>();
 }
