@@ -27,6 +27,7 @@ package dr.inference.mcmc;
 
 import dr.inference.loggers.Logger;
 import dr.inference.markovchain.MarkovChain;
+import dr.inference.markovchain.MarkovChainDelegate;
 import dr.inference.markovchain.MarkovChainListener;
 import dr.inference.model.Likelihood;
 import dr.inference.model.Model;
@@ -65,9 +66,27 @@ public class MCMC implements Identifiable, Spawnable {
             OperatorSchedule schedule,
             Logger[] loggers) {
 
-        init(options, likelihood, Prior.UNIFORM_PRIOR, schedule, loggers);
+        init(options, likelihood, Prior.UNIFORM_PRIOR, schedule, loggers, new MarkovChainDelegate[0]);
     }
 
+    /**
+     * Must be called before calling chain.
+     *
+     * @param options    the options for this MCMC analysis
+     * @param schedule   operator schedule to be used in chain.
+     * @param likelihood the likelihood for this MCMC
+     * @param loggers    an array of loggers to record output of this MCMC run
+     * @param delegates    an array of delegates to handle tasks related to the MCMC
+     */
+    public void init(
+            MCMCOptions options,
+            Likelihood likelihood,
+            OperatorSchedule schedule,
+            Logger[] loggers,
+            MarkovChainDelegate[] delegates) {
+
+        init(options, likelihood, Prior.UNIFORM_PRIOR, schedule, loggers, delegates);
+    }
 
     /**
      * Must be called before calling chain.
@@ -85,11 +104,34 @@ public class MCMC implements Identifiable, Spawnable {
             OperatorSchedule schedule,
             Logger[] loggers) {
 
+        init(options, likelihood, prior, schedule, loggers, new MarkovChainDelegate[0]);
+    }
+
+    /**
+     * Must be called before calling chain.
+     *
+     * @param options    the options for this MCMC analysis
+     * @param prior      the prior disitrbution on the model parameters.
+     * @param schedule   operator schedule to be used in chain.
+     * @param likelihood the likelihood for this MCMC
+     * @param loggers    an array of loggers to record output of this MCMC run
+     * @param delegates    an array of delegates to handle tasks related to the MCMC
+     */
+    public void init(
+            MCMCOptions options,
+            Likelihood likelihood,
+            Prior prior,
+            OperatorSchedule schedule,
+            Logger[] loggers,
+            MarkovChainDelegate[] delegates) {
+
         MCMCCriterion criterion = new MCMCCriterion();
         criterion.setTemperature(options.getTemperature());
 
         mc = new MarkovChain(prior, likelihood, schedule, criterion,
-                options.fullEvaluationCount(), options.minOperatorCountForFullEvaluation(), options.useCoercion());
+                options.getFullEvaluationCount(), options.minOperatorCountForFullEvaluation(),
+                options.getEvaluationTestThreshold(),
+                options.useCoercion());
 
         this.options = options;
         this.loggers = loggers;
@@ -104,6 +146,11 @@ public class MCMC implements Identifiable, Spawnable {
         while ((getChainLength() / stepsPerReport) > 1000) {
             stepsPerReport *= 2;
         }*/
+
+        for(MarkovChainDelegate delegate : delegates) {
+            delegate.setup(options, schedule, mc);
+        }
+        this.delegates = delegates;
     }
 
     /**
@@ -114,14 +161,12 @@ public class MCMC implements Identifiable, Spawnable {
      * @param operators  an array of MCMC operators
      * @param loggers    an array of loggers to record output of this MCMC run
      */
-    public void init(int chainlength,
+    public void init(long chainlength,
                      Likelihood likelihood,
                      MCMCOperator[] operators,
                      Logger[] loggers) {
 
-        MCMCOptions options = new MCMCOptions();
-        options.setCoercionDelay(0);
-        options.setChainLength(chainlength);
+        MCMCOptions options = new MCMCOptions(chainlength);
         MCMCCriterion criterion = new MCMCCriterion();
         criterion.setTemperature(1);
         OperatorSchedule schedule = new SimpleOperatorSchedule();
@@ -172,9 +217,13 @@ public class MCMC implements Identifiable, Spawnable {
         if (!stopping) {
             mc.addMarkovChainListener(chainListener);
 
+            for(MarkovChainDelegate delegate : delegates) {
+                mc.addMarkovChainDelegate(delegate);
+            }
+
             long chainLength = getChainLength();
 
-            final int coercionDelay = getCoercionDelay();
+            final long coercionDelay = getCoercionDelay();
 
             if (coercionDelay > 0) {
                 // Run the chain for coercionDelay steps with coercion disabled
@@ -192,11 +241,15 @@ public class MCMC implements Identifiable, Spawnable {
             mc.terminateChain();
 
             mc.removeMarkovChainListener(chainListener);
+
+            for(MarkovChainDelegate delegate : delegates) {
+                mc.removeMarkovChainDelegate(delegate);
+            }
         }
         timer.stop();
     }
 
-    private final MarkovChainListener chainListener = new MarkovChainListener() {
+    protected final MarkovChainListener chainListener = new MarkovChainListener() {
 
         // MarkovChainListener interface *******************************************
         // for receiving messages from subordinate MarkovChain
@@ -441,11 +494,11 @@ public class MCMC implements Identifiable, Spawnable {
 
 
     //PRIVATE METHODS *****************************************
-    private int getCoercionDelay() {
+    protected long getCoercionDelay() {
 
-        int delay = options.getCoercionDelay();
+        long delay = options.getCoercionDelay();
         if (delay < 0) {
-            delay = (int)(options.getChainLength() / 100);
+            delay = (long)(options.getChainLength() / 100);
         }
         if (options.useCoercion()) return delay;
 
@@ -464,6 +517,7 @@ public class MCMC implements Identifiable, Spawnable {
         showOperatorAnalysis = soa;
     }
 
+
     public void setOperatorAnalysisFile(File operatorAnalysisFile) {
         this.operatorAnalysisFile = operatorAnalysisFile;
     }
@@ -479,27 +533,28 @@ public class MCMC implements Identifiable, Spawnable {
     // PRIVATE TRANSIENTS
 
     //private FileLogger operatorLogger = null;
-    private final boolean isAdapting = true;
-    private boolean stopping = false;
-    private boolean showOperatorAnalysis = true;
+    protected final boolean isAdapting = true;
+    protected boolean stopping = false;
+    protected boolean showOperatorAnalysis = true;
     protected File operatorAnalysisFile = null;
-    private final dr.util.Timer timer = new dr.util.Timer();
-    private long currentState = 0;
+    protected final dr.util.Timer timer = new dr.util.Timer();
+    protected long currentState = 0;
     //private int stepsPerReport = 1000;
-    private final NumberFormatter formatter = new NumberFormatter(8);
+    protected final NumberFormatter formatter = new NumberFormatter(8);
 
     /**
      * this markov chain does most of the work.
      */
-    private MarkovChain mc;
+    protected MarkovChain mc;
 
     /**
      * the options of this MCMC analysis
      */
-    private MCMCOptions options;
+    protected MCMCOptions options;
 
-    private Logger[] loggers;
-    private OperatorSchedule schedule;
+    protected Logger[] loggers;
+    protected OperatorSchedule schedule;
+    private MarkovChainDelegate[] delegates;
 
     private String id = null;
 }
