@@ -1,7 +1,7 @@
 /*
- * MCMC.java
+ * MarginalLikelihoodEstimator.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -31,30 +31,30 @@ import dr.inference.markovchain.MarkovChain;
 import dr.inference.markovchain.MarkovChainListener;
 import dr.inference.model.Model;
 import dr.inference.model.PathLikelihood;
-import dr.inference.operators.CombinedOperatorSchedule;
-import dr.inference.operators.OperatorAnalysisPrinter;
-import dr.inference.operators.OperatorSchedule;
+import dr.inference.operators.*;
 import dr.inference.prior.Prior;
 import dr.util.Identifiable;
 import dr.xml.*;
 import org.apache.commons.math.MathException;
 import org.apache.commons.math.distribution.BetaDistributionImpl;
 
+import java.util.ArrayList;
+import java.util.List;
+
 /**
- * An MCMC analysis that estimates parameters of a probabilistic model.
- *
  * @author Andrew Rambaut
  * @author Alex Alekseyenko
- * @version $Id: MCMC.java,v 1.41 2005/07/11 14:06:25 rambaut Exp $
+ * @author Marc Suchard
+ * @author Guy Baele
  */
 public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
 
     public MarginalLikelihoodEstimator(String id, int chainLength, int burninLength, int pathSteps, double fixedRunValue,
 //                                       boolean linear, boolean lacing,
-PathScheme scheme,
-PathLikelihood pathLikelihood,
-OperatorSchedule schedule,
-MCLogger logger) {
+                                       PathScheme scheme,
+                                       PathLikelihood pathLikelihood,
+                                       OperatorSchedule schedule,
+                                       List<MCLogger> loggers) {
 
         this.id = id;
         this.chainLength = chainLength;
@@ -76,9 +76,9 @@ MCLogger logger) {
         this.pathLikelihood = pathLikelihood;
         pathLikelihood.setPathParameter(pathParameter);
 
-        mc = new MarkovChain(Prior.UNIFORM_PRIOR, pathLikelihood, schedule, criterion, 0, 0, true);
+        mc = new MarkovChain(Prior.UNIFORM_PRIOR, pathLikelihood, schedule, criterion, 0, 0, 0.0, true);
 
-        this.logger = logger;
+        this.loggers = loggers;
     }
 
     private void setDefaultBurnin() {
@@ -93,15 +93,27 @@ MCLogger logger) {
         setDefaultBurnin();
         mc.setCurrentLength(burnin);
         scheme.init();
+        ((CombinedOperatorSchedule) schedule).reset();
         for (pathParameter = scheme.nextPathParameter(); pathParameter >= 0; pathParameter = scheme.nextPathParameter()) {
             pathLikelihood.setPathParameter(pathParameter);
-            reportIteration(pathParameter, chainLength, burnin);
+            reportIteration(pathParameter, chainLength, burnin, scheme.pathSteps, scheme.step);
+
+            for (int i = 0; i < schedule.getOperatorCount(); ++i) {
+                MCMCOperator operator = schedule.getOperator(i);
+                if (operator instanceof GibbsOperator) {
+                    ((GibbsOperator)operator).setPathParameter(pathParameter);
+                }
+            }
+
             long cl = mc.getCurrentLength();
             mc.setCurrentLength(0);
             mc.runChain(burnin, false/*, 0*/);
             mc.setCurrentLength(cl);
             mc.runChain(chainLength, false);
-            (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
+
+            // AR - I have commented this out. Was it being used for debugging or did it have
+            // actual usefulness?
+            // (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
             ((CombinedOperatorSchedule) schedule).reset();
         }
     }
@@ -122,21 +134,21 @@ MCLogger logger) {
     }
 
     public class FixedThetaRun extends Integrator {
-    	private double value;
+        private double value;
 
-    	public FixedThetaRun(double value) {
-    		super(1);
-    		this.value = value;
-    	}
+        public FixedThetaRun(double value) {
+            super(1);
+            this.value = value;
+        }
 
-    	double nextPathParameter() {
-    		if (step == 0) {
-    			step++;
-    			return value;
-    		} else {
-    			return -1.0;
-    		}
-    	}
+        double nextPathParameter() {
+            if (step == 0) {
+                step++;
+                return value;
+            } else {
+                return -1.0;
+            }
+        }
 
     }
 
@@ -149,52 +161,52 @@ MCLogger logger) {
             if (step > pathSteps) {
                 return -1;
             }
-            double pathParameter = 1.0 - (double)step / (double)(pathSteps - 1);
+            double pathParameter = 1.0 - (double)step / (double)(pathSteps);
             step = step + 1;
             return pathParameter;
         }
     }
 
     public class SigmoidIntegrator extends Integrator {
-    	private double alpha;
+        private double alpha;
 
-    	public SigmoidIntegrator(double alpha, int pathSteps) {
-    		super(pathSteps);
-    		this.alpha = alpha;
-    	}
+        public SigmoidIntegrator(double alpha, int pathSteps) {
+            super(pathSteps);
+            this.alpha = alpha;
+        }
 
-    	double nextPathParameter() {
-    		if (step == 0) {
-    			step++;
-    			return 1.0;
-    		} else if (step == pathSteps) {
-    			step++;
-    			return 0.0;
-    		} else if (step > pathSteps) {
-    			return -1.0;
-    		} else {
-    			double xvalue = ((pathSteps - step)/((double)pathSteps)) - 0.5;
-    			step++;
-    			return Math.exp(alpha*xvalue)/(Math.exp(alpha*xvalue) + Math.exp(-alpha*xvalue));
-    		}
-    	}
+        double nextPathParameter() {
+            if (step == 0) {
+                step++;
+                return 1.0;
+            } else if (step == pathSteps) {
+                step++;
+                return 0.0;
+            } else if (step > pathSteps) {
+                return -1.0;
+            } else {
+                double xvalue = ((pathSteps - step)/((double)pathSteps)) - 0.5;
+                step++;
+                return Math.exp(alpha*xvalue)/(Math.exp(alpha*xvalue) + Math.exp(-alpha*xvalue));
+            }
+        }
     }
 
     public class BetaQuantileIntegrator extends Integrator {
-    	private double alpha;
+        private double alpha;
 
-    	public BetaQuantileIntegrator(double alpha, int pathSteps) {
-    		super(pathSteps);
-    		this.alpha = alpha;
-    	}
+        public BetaQuantileIntegrator(double alpha, int pathSteps) {
+            super(pathSteps);
+            this.alpha = alpha;
+        }
 
-    	double nextPathParameter() {
-    		if (step > pathSteps)
+        double nextPathParameter() {
+            if (step > pathSteps)
                 return -1;
-    		double result = Math.pow((pathSteps - step)/((double)pathSteps), 1.0/alpha);
-    		step++;
-    		return result;
-    	}
+            double result = Math.pow((pathSteps - step)/((double)pathSteps), 1.0/alpha);
+            step++;
+            return result;
+        }
     }
 
     public class BetaIntegrator extends Integrator {
@@ -289,13 +301,15 @@ MCLogger logger) {
         }
     }*/
 
-    private void reportIteration(double pathParameter, long chainLength, long burnin) {
-        System.out.println("Attempting theta = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
+    private void reportIteration(double pathParameter, long chainLength, long burnin, long totalSteps, long steps) {
+        System.out.println("Attempting theta ("+steps+"/" + (totalSteps+1) +") = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
     }
 
     public void run() {
 
-        logger.startLogging();
+        for (MCLogger logger : loggers) {
+            logger.startLogging();
+        }
         mc.addMarkovChainListener(chainListener);
 
         /*switch (scheme) {
@@ -316,9 +330,9 @@ MCLogger logger) {
         }*/
 
         switch (scheme) {
-        	case FIXED:
-        		integrate(new FixedThetaRun(fixedRunValue));
-        		break;
+            case FIXED:
+                integrate(new FixedThetaRun(fixedRunValue));
+                break;
             case LINEAR:
                 integrate(new LinearIntegrator(pathSteps));
                 break;
@@ -332,11 +346,11 @@ MCLogger logger) {
                 integrate(new BetaIntegrator(alphaFactor, betaFactor, pathSteps));
                 break;
             case BETA_QUANTILE:
-            	integrate(new BetaQuantileIntegrator(alphaFactor, pathSteps));
-            	break;
+                integrate(new BetaQuantileIntegrator(alphaFactor, pathSteps));
+                break;
             case SIGMOID:
-            	integrate(new SigmoidIntegrator(alphaFactor, pathSteps));
-            	break;
+                integrate(new SigmoidIntegrator(alphaFactor, pathSteps));
+                break;
             default:
                 throw new RuntimeException("Illegal path scheme");
         }
@@ -357,7 +371,9 @@ MCLogger logger) {
             currentState = state;
 
             if (currentState >= burnin) {
-                logger.log(state);
+                for (MCLogger logger : loggers) {
+                    logger.log(state);
+                }
             }
         }
 
@@ -375,7 +391,9 @@ MCLogger logger) {
             currentState = chainLength;
             (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
 //            logger.log(currentState);
-            logger.stopLogging();
+            for (MCLogger logger : loggers) {
+                logger.stopLogging();
+            }
         }
     };
 
@@ -423,7 +441,15 @@ MCLogger logger) {
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
             PathLikelihood pathLikelihood = (PathLikelihood) xo.getChild(PathLikelihood.class);
-            MCLogger logger = (MCLogger) xo.getChild(MCLogger.class);
+
+            List<MCLogger> loggerList = new ArrayList<MCLogger>();
+            for (int i = 0; i < xo.getChildCount(); i++) {
+                if (xo.getChild(i) instanceof MCLogger) {
+                    loggerList.add((MCLogger)xo.getChild(i));
+                }
+            }
+
+//            MCLogger logger = (MCLogger) xo.getChild(MCLogger.class);
 
             int chainLength = xo.getIntegerAttribute(CHAIN_LENGTH);
             int pathSteps = xo.getIntegerAttribute(PATH_STEPS);
@@ -438,7 +464,7 @@ MCLogger logger) {
             }
             double fixedRunValue = -1.0;
             if (xo.hasAttribute(FIXED_VALUE)) {
-            	fixedRunValue = xo.getDoubleAttribute(FIXED_VALUE);
+                fixedRunValue = xo.getDoubleAttribute(FIXED_VALUE);
             }
 
             // deprecated
@@ -484,7 +510,7 @@ MCLogger logger) {
             }
 
             MarginalLikelihoodEstimator mle = new MarginalLikelihoodEstimator(MARGINAL_LIKELIHOOD_ESTIMATOR, chainLength,
-                    burninLength, pathSteps, fixedRunValue, scheme, pathLikelihood, os, logger);
+                    burninLength, pathSteps, fixedRunValue, scheme, pathLikelihood, os, loggerList);
 
             if (!xo.getAttribute(SPAWN, true))
                 mle.setSpawnable(false);
@@ -503,9 +529,9 @@ MCLogger logger) {
             } else if (scheme == PathScheme.BETA) {
                 alphaBetaText += "(" + mle.getAlphaFactor() + "," + mle.getBetaFactor() + ")";
             } else if (scheme == PathScheme.BETA_QUANTILE) {
-            	alphaBetaText += "(" + mle.getAlphaFactor() + ")";
+                alphaBetaText += "(" + mle.getAlphaFactor() + ")";
             } else if (scheme == PathScheme.SIGMOID) {
-            	alphaBetaText += "(" + mle.getAlphaFactor() + ")";
+                alphaBetaText += "(" + mle.getAlphaFactor() + ")";
             }
             java.util.logging.Logger.getLogger("dr.inference").info("\nCreating the Marginal Likelihood Estimator chain:" +
                     "\n  chainLength=" + chainLength +
@@ -514,7 +540,10 @@ MCLogger logger) {
                     "\n  If you use these results, please cite:" +
                     "\n    Guy Baele, Philippe Lemey, Trevor Bedford, Andrew Rambaut, Marc A. Suchard, and Alexander V. Alekseyenko." +
                     "\n    2012. Improving the accuracy of demographic and molecular clock model comparison while accommodating " +
-                    "\n          phylogenetic uncertainty. Mol. Biol. Evol. (in press).");
+                    "\n          phylogenetic uncertainty. Mol. Biol. Evol. 29(9):2157-2167." +
+                    "\n    and " +
+                    "\n    Guy Baele, Wai Lok Sibon Li, Alexei J. Drummond, Marc A. Suchard, and Philippe Lemey. 2013" +
+                    "\n    Accurate model selection of relaxed molecular clocks in Bayesian phylogenetics. Mol. Biol. Evol. 30(2):239-243.\n");
             return mle;
         }
 
@@ -550,7 +579,7 @@ MCLogger logger) {
                         new XMLSyntaxRule[]{new ElementRule(MCMC.class, 1, Integer.MAX_VALUE)}, false),
                 //new ElementRule(MCMC.class),
                 new ElementRule(PathLikelihood.class),
-                new ElementRule(MCLogger.class)
+                new ElementRule(MCLogger.class, 1, Integer.MAX_VALUE)
         };
 
     };
@@ -564,7 +593,7 @@ MCLogger logger) {
     }
 
     enum PathScheme {
-    	FIXED("fixed"),
+        FIXED("fixed"),
         LINEAR("linear"),
         GEOMETRIC("geometric"),
         BETA("beta"),
@@ -617,7 +646,7 @@ MCLogger logger) {
     private final double pathDelta;
     private double pathParameter;
 
-    private final MCLogger logger;
+    private final List<MCLogger> loggers;
 
     private final PathLikelihood pathLikelihood;
 
