@@ -105,7 +105,7 @@ public class MCMCMC implements Runnable {
         timer.start();
 
         if (isPreBurninNeeded()) {
-            int preBurnin = mcmcOptions.getCoercionDelay();
+            long preBurnin = mcmcOptions.getCoercionDelay();
             if (preBurnin > 0) {
                 MarkovChainListener burninListener = new BurninListener(preBurnin);
 
@@ -144,7 +144,6 @@ public class MCMCMC implements Runnable {
             threads[i].start();
         }
 
-
         while (chains[coldChain].getCurrentLength() < getChainLength()) {
 
             // wait for all the threads to complete their alloted chain length
@@ -203,7 +202,7 @@ public class MCMCMC implements Runnable {
         timer.stop();
     }
 
-    private void runChains(int length, boolean disableCoerce) {
+    private void runChains(long length, boolean disableCoerce) {
 
         Thread[] threads = new Thread[chains.length];
         for (int i = 0; i < chains.length; i++) {
