@@ -1,7 +1,7 @@
 /*
  * MatrixParameter.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -28,9 +28,6 @@ package dr.inference.model;
 import dr.xml.*;
 
 import java.util.StringTokenizer;
-//import org.w3c.dom.Document;
-//import org.w3c.dom.Element;
-//import dr.xml.*;
 
 /**
  * @author Marc Suchard
@@ -89,7 +86,6 @@ public class MatrixParameter extends CompoundParameter {
                 addParameter(row);
             }
         }
-
     }
 
     public int getColumnDimension() {
@@ -101,7 +97,7 @@ public class MatrixParameter extends CompoundParameter {
     }
 
     public String toSymmetricString() {
-        StringBuffer sb = new StringBuffer("{");
+        StringBuilder sb = new StringBuilder("{");
         int dim = getRowDimension();
         int total = dim * (dim + 1) / 2;
         for (int i = 0; i < dim; i++) {
@@ -165,6 +161,7 @@ public class MatrixParameter extends CompoundParameter {
 
     private static final String ROW_DIMENSION = "rows";
     private static final String COLUMN_DIMENSION = "columns";
+    private static final String TRANSPOSE = "transpose";
 
     public static XMLObjectParser PARSER = new AbstractXMLObjectParser() {
 
@@ -175,7 +172,14 @@ public class MatrixParameter extends CompoundParameter {
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
             final String name = xo.hasId() ? xo.getId() : null;
-            MatrixParameter matrixParameter = new MatrixParameter(name);
+            boolean transposed = xo.getAttribute(TRANSPOSE, false);
+
+            MatrixParameter matrixParameter;
+            if (!transposed) {
+                matrixParameter = new MatrixParameter(name);
+            } else {
+                matrixParameter = new TransposedMatrixParameter(name);
+            }
 
             if (xo.hasAttribute(ROW_DIMENSION)) {
                 int rowDimension = xo.getIntegerAttribute(ROW_DIMENSION);
@@ -196,6 +200,7 @@ public class MatrixParameter extends CompoundParameter {
                 else if (dim != parameter.getDimension())
                     throw new XMLParseException("All parameters must have the same dimension to construct a rectangular matrix");
             }
+
             return matrixParameter;
         }
 
@@ -214,13 +219,12 @@ public class MatrixParameter extends CompoundParameter {
         private final XMLSyntaxRule[] rules = {
                 new ElementRule(Parameter.class, 0, Integer.MAX_VALUE),
                 AttributeRule.newIntegerRule(ROW_DIMENSION, true),
-                AttributeRule.newIntegerRule(COLUMN_DIMENSION, true)
+                AttributeRule.newIntegerRule(COLUMN_DIMENSION, true),
+                AttributeRule.newBooleanRule(TRANSPOSE, true),
         };
 
         public Class getReturnType() {
             return MatrixParameter.class;
         }
     };
-
-
 }
