@@ -26,7 +26,7 @@ public abstract class AbstractTraceList extends FilteredTraceList {
 
         Trace trace = getTrace(index);        
         TraceCorrelation traceCorrelation = new TraceCorrelation(
-                trace.getValues(start, trace.getValuesSize(), selected),
+                trace.getValues(start, trace.getValuesSize()),
                 trace.getTraceType(), getStepSize());
         trace.setTraceStatistics(traceCorrelation);
 
