@@ -27,6 +27,7 @@ package dr.inferencexml;
 
 import dr.inference.loggers.Logger;
 import dr.inference.markovchain.MarkovChain;
+import dr.inference.markovchain.MarkovChainDelegate;
 import dr.inference.mcmc.MCMC;
 import dr.inference.mcmc.MCMCOptions;
 import dr.inference.model.CompoundLikelihood;
@@ -48,10 +49,35 @@ public class MCMCParser extends AbstractXMLObjectParser {
     public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
         MCMC mcmc = new MCMC(xo.getAttribute(NAME, "mcmc1"));
-        MCMCOptions options = new MCMCOptions();
+
+        long chainLength = xo.getLongIntegerAttribute(CHAIN_LENGTH);
+        boolean useCoercion = xo.getAttribute(COERCION, true);
+        long coercionDelay = chainLength / 100;
+        if (xo.hasAttribute(PRE_BURNIN)) {
+            coercionDelay = xo.getIntegerAttribute(PRE_BURNIN);
+        }
+        coercionDelay = xo.getAttribute(COERCION_DELAY, coercionDelay);
+        double temperature = xo.getAttribute(TEMPERATURE, 1.0);
+        long fullEvaluationCount = xo.getAttribute(FULL_EVALUATION, 2000);
+
+        double evaluationTestThreshold = MarkovChain.EVALUATION_TEST_THRESHOLD;
+        if (System.getProperty("mcmc.evaluation.threshold") != null) {
+            evaluationTestThreshold = Double.parseDouble(System.getProperty("mcmc.evaluation.threshold"));
+        }
+        evaluationTestThreshold = xo.getAttribute(EVALUATION_THRESHOLD, evaluationTestThreshold);
+
+        int minOperatorCountForFullEvaluation = xo.getAttribute(MIN_OPS_EVALUATIONS, 1);
+
+        MCMCOptions options = new MCMCOptions(chainLength,
+                fullEvaluationCount,
+                minOperatorCountForFullEvaluation,
+                evaluationTestThreshold,
+                useCoercion,
+                coercionDelay,
+                temperature);
+
         OperatorSchedule opsched = (OperatorSchedule) xo.getChild(OperatorSchedule.class);
         Likelihood likelihood = (Likelihood) xo.getChild(Likelihood.class);
-        ArrayList<Logger> loggers = new ArrayList<Logger>();
 
         likelihood.setUsed();
 
@@ -75,21 +101,18 @@ public class MCMCParser extends AbstractXMLObjectParser {
 //            }
 //        }
 
-        options.setChainLength(xo.getLongIntegerAttribute(CHAIN_LENGTH));
-        options.setUseCoercion(xo.getAttribute(COERCION, true));
-        if (xo.hasAttribute(PRE_BURNIN)) {
-            options.setCoercionDelay(xo.getAttribute(PRE_BURNIN, (int)(options.getChainLength() / 100)));
-        }
-        options.setCoercionDelay(xo.getAttribute(COERCION_DELAY, (int)(options.getChainLength() / 100)));
-        options.setTemperature(xo.getAttribute(TEMPERATURE, 1.0));
-        options.setFullEvaluationCount(xo.getAttribute(FULL_EVALUATION, 2000));
-        options.setMinOperatorCountForFullEvaluation(xo.getAttribute(MIN_OPS_EVALUATIONS, 1));
+
+        ArrayList<Logger> loggers = new ArrayList<Logger>();
+        ArrayList<MarkovChainDelegate> delegates = new ArrayList<MarkovChainDelegate>();
 
         for (int i = 0; i < xo.getChildCount(); i++) {
             Object child = xo.getChild(i);
             if (child instanceof Logger) {
                 loggers.add((Logger) child);
             }
+            else if (child instanceof MarkovChainDelegate) {
+                delegates.add((MarkovChainDelegate) child);
+            }
         }
 
         mcmc.setShowOperatorAnalysis(true);
@@ -97,17 +120,23 @@ public class MCMCParser extends AbstractXMLObjectParser {
             mcmc.setOperatorAnalysisFile(XMLParser.getLogFile(xo, OPERATOR_ANALYSIS));
         }
 
+
         Logger[] loggerArray = new Logger[loggers.size()];
         loggers.toArray(loggerArray);
 
+
         java.util.logging.Logger.getLogger("dr.inference").info("Creating the MCMC chain:" +
                 "\n  chainLength=" + options.getChainLength() +
                 "\n  autoOptimize=" + options.useCoercion() +
                 (options.useCoercion() ? "\n  autoOptimize delayed for " + options.getCoercionDelay() + " steps" : "") +
-                (options.fullEvaluationCount() == 0 ? "\n  full evaluation test off" : "")
-                );
+                (options.getFullEvaluationCount() == 0 ? "\n  full evaluation test off" : "")
+        );
+
+        MarkovChainDelegate[] delegateArray = new MarkovChainDelegate[delegates.size()];
+        delegates.toArray(delegateArray);
+
+        mcmc.init(options, likelihood, opsched, loggerArray, delegateArray);
 
-        mcmc.init(options, likelihood, opsched, loggerArray);
 
         MarkovChain mc = mcmc.getMarkovChain();
         double initialScore = mc.getCurrentScore();
@@ -152,12 +181,14 @@ public class MCMCParser extends AbstractXMLObjectParser {
             AttributeRule.newDoubleRule(TEMPERATURE, true),
             AttributeRule.newIntegerRule(FULL_EVALUATION, true),
             AttributeRule.newIntegerRule(MIN_OPS_EVALUATIONS, true),
+            AttributeRule.newDoubleRule(EVALUATION_THRESHOLD, true),
             AttributeRule.newBooleanRule(SPAWN, true),
             AttributeRule.newStringRule(NAME, true),
             AttributeRule.newStringRule(OPERATOR_ANALYSIS, true),
             new ElementRule(OperatorSchedule.class),
             new ElementRule(Likelihood.class),
-            new ElementRule(Logger.class, 1, Integer.MAX_VALUE)
+            new ElementRule(Logger.class, 1, Integer.MAX_VALUE),
+            new ElementRule(MarkovChainDelegate.class, 0, Integer.MAX_VALUE)
     };
 
     public static final String COERCION = "autoOptimize";
@@ -167,6 +198,7 @@ public class MCMCParser extends AbstractXMLObjectParser {
     public static final String MCMC = "mcmc";
     public static final String CHAIN_LENGTH = "chainLength";
     public static final String FULL_EVALUATION = "fullEvaluation";
+    public static final String EVALUATION_THRESHOLD  = "evaluationThreshold";
     public static final String MIN_OPS_EVALUATIONS = "minOpsFullEvaluations";
     public static final String WEIGHT = "weight";
     public static final String TEMPERATURE = "temperature";
