@@ -1,7 +1,7 @@
 /*
  * PriorParsers.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -28,13 +28,30 @@ package dr.inferencexml.distribution;
 import dr.inference.distribution.DistributionLikelihood;
 import dr.inference.model.Likelihood;
 import dr.inference.model.Statistic;
-import dr.math.distributions.*;
-import dr.util.DataTable;
-import dr.xml.*;
+import dr.math.distributions.BetaDistribution;
+import dr.math.distributions.ExponentialDistribution;
+import dr.math.distributions.GammaDistribution;
+import dr.math.distributions.HalfTDistribution;
+import dr.math.distributions.InverseGammaDistribution;
+import dr.math.distributions.LaplaceDistribution;
+import dr.math.distributions.LogNormalDistribution;
+import dr.math.distributions.NormalDistribution;
+import dr.math.distributions.PoissonDistribution;
+import dr.math.distributions.UniformDistribution;
+import dr.xml.AbstractXMLObjectParser;
+import dr.xml.AttributeRule;
+import dr.xml.ElementRule;
+import dr.xml.XMLObject;
+import dr.xml.XMLObjectParser;
+import dr.xml.XMLParseException;
+import dr.xml.XMLSyntaxRule;
+import dr.xml.XORRule;
 
 /**
  */
 public class PriorParsers {
+    public final static boolean DEBUG = false;
+
     public static final String UNIFORM_PRIOR = "uniformPrior";
     public static final String EXPONENTIAL_PRIOR = "exponentialPrior";
     public static final String POISSON_PRIOR = "poissonPrior";
@@ -53,8 +70,10 @@ public class PriorParsers {
     public static final String SHAPE = "shape";
     public static final String SHAPEB = "shapeB";
     public static final String SCALE = "scale";
+    public static final String DF = "df";
     public static final String OFFSET = "offset";
     public static final String UNINFORMATIVE = "uninformative";
+    public static final String HALF_T_PRIOR = "halfTPrior";
 
     /**
      * A special parser that reads a convenient short form of priors on parameters.
@@ -74,15 +93,33 @@ public class PriorParsers {
                 throw new XMLParseException("Uniform prior " + xo.getName() + " cannot take a bound at infinity, " +
                         "because it returns 1/(high-low) = 1/inf");
 
-            DistributionLikelihood likelihood = new DistributionLikelihood(new UniformDistribution(lower, upper), true);
+            DistributionLikelihood likelihood = new DistributionLikelihood(new UniformDistribution(lower, upper));
+            if (DEBUG) {
+                System.out.println("Uniform prior: " + xo.getChildCount());
+            }
             for (int j = 0; j < xo.getChildCount(); j++) {
+                if (DEBUG) {
+                    System.out.println(xo.getChild(j));
+                }
                 if (xo.getChild(j) instanceof Statistic) {
+                    if (DEBUG) {
+                        //System.out.println((Statistic) xo.getChild(j));
+                        Statistic test = (Statistic) xo.getChild(j);
+                        System.out.println(test.getDimension());
+                        for (int i = 0; i < test.getDimension(); i++) {
+                            System.out.println("  " + test.getDimensionName(i) + " - " + test.getStatisticValue(i));
+                        }
+                        System.out.println(test.getClass());
+                    }
                     likelihood.addData((Statistic) xo.getChild(j));
+                    if (DEBUG) {
+                    	likelihood.calculateLogLikelihood();
+                    	System.out.println("likelihood: " + likelihood.getLogLikelihood());
+                    }
                 } else {
                     throw new XMLParseException("illegal element in " + xo.getName() + " element");
                 }
             }
-
             return likelihood;
         }
 
@@ -126,9 +163,31 @@ public class PriorParsers {
             final double offset = xo.hasAttribute(OFFSET) ? xo.getDoubleAttribute(OFFSET) : 0.0;
 
             DistributionLikelihood likelihood = new DistributionLikelihood(new ExponentialDistribution(1.0 / scale), offset);
+            if (DEBUG) {
+                System.out.println("Exponential prior: " + xo.getChildCount());
+            }
             for (int j = 0; j < xo.getChildCount(); j++) {
+                if (DEBUG) {
+                    System.out.println(xo.getChild(j));
+                }
                 if (xo.getChild(j) instanceof Statistic) {
+                    if (DEBUG) {
+                    	System.out.println("scale: " + scale);
+                    	System.out.println("offset: " + offset);
+                        //System.out.println((Statistic) xo.getChild(j));
+                        Statistic test = (Statistic) xo.getChild(j);
+                        System.out.println(test.getDimension());
+                        for (int i = 0; i < test.getDimension(); i++) {
+                            System.out.println("  " + test.getDimensionName(i) + " - " + test.getStatisticValue(i));
+                        }
+                        System.out.println(test.getClass());
+                    }
                     likelihood.addData((Statistic) xo.getChild(j));
+                    if (DEBUG) {
+                    	likelihood.makeDirty();
+                    	likelihood.calculateLogLikelihood();
+                    	System.out.println("likelihood: " + likelihood.getLogLikelihood());
+                    }
                 } else {
                     throw new XMLParseException("illegal element in " + xo.getName() + " element");
                 }
@@ -204,6 +263,50 @@ public class PriorParsers {
         }
     };
 
+    /**
+     * A special parser that reads a convenient short form of priors on parameters.
+     */
+    public static XMLObjectParser HALF_T_PARSER = new AbstractXMLObjectParser() {
+
+        public String getParserName() {
+            return HALF_T_PRIOR;
+        }
+
+        public Object parseXMLObject(XMLObject xo) throws XMLParseException {
+
+            double scale = xo.getDoubleAttribute(SCALE);
+            double df = xo.getDoubleAttribute(DF);
+
+            DistributionLikelihood likelihood = new DistributionLikelihood(new HalfTDistribution(scale, df));
+            for (int j = 0; j < xo.getChildCount(); j++) {
+                if (xo.getChild(j) instanceof Statistic) {
+                    likelihood.addData((Statistic) xo.getChild(j));
+                } else {
+                    throw new XMLParseException("illegal element in " + xo.getName() + " element");
+                }
+            }
+
+            return likelihood;
+        }
+
+        public XMLSyntaxRule[] getSyntaxRules() {
+            return rules;
+        }
+
+        private final XMLSyntaxRule[] rules = {
+                AttributeRule.newDoubleRule(SCALE),
+                AttributeRule.newDoubleRule(DF),
+                new ElementRule(Statistic.class, 1, Integer.MAX_VALUE)
+        };
+
+        public String getParserDescription() {
+            return "Calculates the prior probability of some data under a given half-T distribution.";
+        }
+
+        public Class getReturnType() {
+            return Likelihood.class;
+        }
+    };
 
     /**
      * A special parser that reads a convenient short form of priors on parameters.
@@ -377,8 +480,7 @@ public class PriorParsers {
         }
 
         public String[] getParserNames() {
-            return new String[] { INVGAMMA_PRIOR, INVGAMMA_PRIOR_CORRECT };
-
+            return new String[]{INVGAMMA_PRIOR, INVGAMMA_PRIOR_CORRECT};
         }
 
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
