@@ -1,3 +1,28 @@
+/*
+ * MultivariateNormalDistribution.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.math.distributions;
 
 import dr.math.MathUtils;
@@ -6,7 +31,7 @@ import dr.math.matrixAlgebra.*;
 /**
  * @author Marc Suchard
  */
-public class MultivariateNormalDistribution implements MultivariateDistribution {
+public class MultivariateNormalDistribution implements MultivariateDistribution, RandomGenerator {
 
     public static final String TYPE = "MultivariateNormal";
 
@@ -43,9 +68,34 @@ public class MultivariateNormalDistribution implements MultivariateDistribution
         if (logDet == null) {
             logDet = Math.log(calculatePrecisionMatrixDeterminate(precision));
         }
+        if (Double.isInfinite(logDet)) {
+            if (isDiagonal(precision)) {
+                logDet = logDetForDiagonal(precision);
+            }
+        }
         return logDet;
     }
 
+    private boolean isDiagonal(double x[][]) {
+        for (int i = 0; i < x.length; ++i) {
+            for (int j = i + 1; j < x.length; ++j) {
+                if (x[i][j] != 0.0) {
+                    return false;
+                }
+            }
+        }
+        return true;
+    }
+
+    private double logDetForDiagonal(double x[][]) {
+        double logDet = 0;
+        for (int i = 0; i < x.length; ++i) {
+            logDet += Math.log(x[i][i]);
+        }
+        return logDet;
+    }
+
+
     public double[][] getScaleMatrix() {
         return precision;
     }
@@ -243,4 +293,14 @@ public class MultivariateNormalDistribution implements MultivariateDistribution
     }
 
     public static final double logNormalize = -0.5 * Math.log(2.0 * Math.PI);
+
+    // RandomGenerator interface
+    public Object nextRandom() {
+        return nextMultivariateNormal();
+    }
+
+    public double logPdf(Object x) {
+        double[] v = (double[]) x;
+        return logPdf(v);
+    }
 }
