@@ -1,3 +1,28 @@
+/*
+ * WishartDistribution.java
+ *
+ * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.math.distributions;
 
 import dr.math.GammaFunction;
@@ -9,7 +34,7 @@ import dr.math.matrixAlgebra.Matrix;
 /**
  * @author Marc Suchard
  */
-public class WishartDistribution implements MultivariateDistribution {
+public class WishartDistribution implements MultivariateDistribution, WishartStatistics {
 
     public static final String TYPE = "Wishart";
 
@@ -120,14 +145,10 @@ public class WishartDistribution implements MultivariateDistribution {
 
     }
 
-    public double df() {
+    public double getDF() {
         return df;
     }
 
-    public double[][] scaleMatrix() {
-        return scaleMatrix;
-    }
-
     public double[][] nextWishart() {
         return nextWishart(df, scaleMatrix);
     }
@@ -228,17 +249,16 @@ public class WishartDistribution implements MultivariateDistribution {
         double logDensity = 0;
 
         try {
-            if (!W.isPD())
-                return Double.NEGATIVE_INFINITY;
+//            if (!W.isPD()) { // TODO isPD() does not appear to work
+//                return Double.NEGATIVE_INFINITY;
+//            }
 
-            final double det = W.determinant();
+            logDensity = W.logDeterminant(); // Returns NaN is W is not positive-definite.
 
-            if (det <= 0) {
+            if (Double.isInfinite(logDensity) || Double.isNaN(logDensity)) {
                 return Double.NEGATIVE_INFINITY;
             }
 
-            logDensity = Math.log(det);
-
             logDensity *= 0.5;
             logDensity *= df - dim - 1;
 
