@@ -309,12 +309,7 @@ public class UncorrelatedRelaxedClockTest extends TraceCorrelationAssert {
 
         // MCMC
         MCMC mcmc = new MCMC("mcmc1");
-        MCMCOptions options = new MCMCOptions();
-        options.setChainLength(10000000);
-        options.setUseCoercion(true); // autoOptimize = true
-        options.setCoercionDelay(100);
-        options.setTemperature(1.0);
-        options.setFullEvaluationCount(2000);
+        MCMCOptions options = new MCMCOptions(10000000);
 
         mcmc.setShowOperatorAnalysis(true);
         mcmc.init(options, posterior, schedule, loggers);
